/*
 * Copyright (c) 2015 by inovat, innovative systeme - verkehr - tunnel - technik, Dipl.-Ing. H. C. Kniss
 *
 * This file is part of de.bsvrz.kex.tls.osi7.conversion.EakConversionInterface
 *
 * de.bsvrz.kex.tls.osi7.conversion.EakConversionInterface is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  de.bsvrz.kex.tls.osi7.conversion.EakConversionInterface is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with de.bsvrz.kex.tls.osi7.conversion.EakConversionInterface.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Diese Datei ist Teil von de.bsvrz.kex.tls.osi7.conversion.EakConversionInterface.
 *
 *  de.bsvrz.kex.tls.osi7.conversion.EakConversionInterface ist Freie Software: Sie knnen es unter den Bedingungen
 *  der GNU General Public License, wie von der Free Software Foundation,
 *  Version 3 der Lizenz oder (nach Ihrer Wahl) jeder spteren
 *  verffentlichten Version, weiterverbreiten und/oder modifizieren.
 *
 *  de.bsvrz.kex.tls.osi7.conversion.EakConversionInterface wird in der Hoffnung, dass es ntzlich sein wird, aber
 *  OHNE JEDE GEWHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 *  Gewhrleistung der MARKTFHIGKEIT oder EIGNUNG FR EINEN BESTIMMTEN ZWECK.
 *  Siehe die GNU General Public License fr weitere Details.
 *
 *  Sie sollten eine Kopie der GNU General Public License zusammen mit diesem
 *  Programm erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 *
 * Contact Information:
 * inovat, Dipl.-Ing. H. C. Kniss
 * An der Krautwiese 37
 * D-53783 Eitorf
 * +49 (0)2243 8464 193
 * info@inovat.de
 * www.inovat.de
 */



package de.bsvrz.kex.tls.osi7.conversion;

//~ NICHT JDK IMPORTE =========================================================

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.kex.tls.osi7.SingleTelegram;

//~ INTERFACES ================================================================

/**
 * Interfacedefinition, welche von allen Umsetzungsmodulen fr Eak erfllt werden muss.
 *
 * @author inovat, innovative systeme - verkehr - tunnel - technik
 * @author Dipl.-Ing. Hans Christian Kni (HCK)
 * @version $Revision$ / $Date$ / ($Author$)
 */
public interface EakConversionInterface {

    // ------------------------------------------------------------------------
    // SONSTIGE METHODEN
    // ------------------------------------------------------------------------

    /**
     * Liefert die laufende Nummer des Datensatzes der Einzelergebnisdaten der FG2. Entspricht dem Wert <code>laufende
     * Nummer des Datensatzes</code> im Typ 31 der FG 2.
     *
     * @return Laufende Nummer des Datensatzes.
     */
    public int getAxlCurrentNumber();

    /**
     * Liefert die IntervallDauer der Ergebnisdaten der FG2 in Sekunden. Entspricht dem Wert <code>Intervalllnge</code> im
     * Typ 64 der FG 2.
     *
     * @return IntervallDauer in Sekungen.
     */
    public int getAxlIntervalDuration();

    /**
     * Liefert die Gesamtzahl Datensatzes der Einzelergebnisdaten der FG2. Entspricht dem Wert <code>Gesamtzahl der
     * Datenstze</code> im Typ 31 der FG 2.
     *
     * @return Gesamtzahl der Datenstze.
     */
    public int getAxlTotalNumber();

    /**
     * Ermglicht den Zugriff auf die aktuelle Datenverteilerverbindung.
     *
     * @return Gibt das aktuelle aktuelle Datenverteilerverbindung zurck.
     */
    public ClientDavInterface getClientDavInterface();

    /**
     * Liefert die Jobnummer des Einzeltelegramms.
     *
     * @return Jobnummer fr das Einzeltelegramm (0=spontan, 1..255=Antwort auf Telegramm mit dieser Jobnummer).
     */
    public int getJobNumber();

    /**
     * Liefert die IntervallDauer der Ergebnisdaten der FG1 in Sekunden. Entspricht dem Wert <code>Intervalllnge</code> im
     * Typ 48 und 64 der FG 1.
     *
     * @return IntervallDauer in Sekungen.
     */
    public long getLveIntervalDuration();

    /**
     * Liefert die IntervallArt der Ergebnisdaten der FG1. Entspricht dem Wert <code>Art des Intervalls</code> im Typ 48
     * und 64 der FG 1.
     *
     * @return Intervalltyp:<br>
     *         <pre>
     *         1 = Kurzzeitdaten
     *         3 = streckenbezogene Daten
     *         </pre>
     */
    public int getLveIntervalType();

    /**
     * Liefert den Zeitstempel fr das Einzeltelegramm aller Funktionsgruppen. Der Wert ist die Zeitdifferenz zum
     * 01.01.1970 (Mitternacht) in Millisekunden in der Zeitzone UTC.
     *
     * @return Zeitstempel fr das Einzeltelegramm.
     */
    public long getTimeStamp();

    /**
     * Liefert die aktuelle Folgenummer der FG4. Entspricht dem Wert <code>Folgenummer</code> im Typ 31 der FG 4.
     *
     * @return Folgenummer der Datenstze.
     */
    public int getWzgNextSequenceNumber();

    /**
     * Liefert die IntervallDauer der Ergebnisdaten der FG9 in Sekunden. Entspricht dem Wert <code>Intervalllnge</code> im
     * Typ der FG 9.
     *
     * @return IntervallDauer in Sekungen.
     */
    public long getZfrIntervalDuration();

    /**
     * Liefert die aktuelle Folgenummer der FG9. Entspricht dem Wert <code>Folgenummer</code> im Typ 31 der FG 9.
     *
     * @return Folgenummer der Datenstze.
     */
    public int getZfrNextSequenceNumber();

    //~ METHODEN ==============================================================

    /**
     * Verarbeitet ein EinzelTelegramm.
     *
     * @param singleTelegram Das zu verarbeitende Einzeltelegramm {@link de.bsvrz.kex.tls.osi7.SingleTelegram}.
     */
    void processSingleTelegram(SingleTelegram singleTelegram);
}


//~Formatiert mit 'inovat Kodierkonvention' am 24.02.12
