/*
 * Copyright (c) 2015 by inovat, innovative systeme - verkehr - tunnel - technik, Dipl.-Ing. H. C. Kniss
 *
 * This file is part of de.bsvrz.kex.tls.osi7.deausfallueberwachung.Auftrag
 *
 * de.bsvrz.kex.tls.osi7.deausfallueberwachung.Auftrag is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  de.bsvrz.kex.tls.osi7.deausfallueberwachung.Auftrag is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with de.bsvrz.kex.tls.osi7.deausfallueberwachung.Auftrag.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Diese Datei ist Teil von de.bsvrz.kex.tls.osi7.deausfallueberwachung.Auftrag.
 *
 *  de.bsvrz.kex.tls.osi7.deausfallueberwachung.Auftrag ist Freie Software: Sie knnen es unter den Bedingungen
 *  der GNU General Public License, wie von der Free Software Foundation,
 *  Version 3 der Lizenz oder (nach Ihrer Wahl) jeder spteren
 *  verffentlichten Version, weiterverbreiten und/oder modifizieren.
 *
 *  de.bsvrz.kex.tls.osi7.deausfallueberwachung.Auftrag wird in der Hoffnung, dass es ntzlich sein wird, aber
 *  OHNE JEDE GEWHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 *  Gewhrleistung der MARKTFHIGKEIT oder EIGNUNG FR EINEN BESTIMMTEN ZWECK.
 *  Siehe die GNU General Public License fr weitere Details.
 *
 *  Sie sollten eine Kopie der GNU General Public License zusammen mit diesem
 *  Programm erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 *
 * Contact Information:
 * inovat, Dipl.-Ing. H. C. Kniss
 * An der Krautwiese 37
 * D-53783 Eitorf
 * +49 (0)2243 8464 193
 * info@inovat.de
 * www.inovat.de
 */



package de.bsvrz.kex.tls.osi7.deausfallueberwachung;

//~ NICHT JDK IMPORTE =========================================================

import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.kex.tls.TlsApplication;
import de.bsvrz.kex.tls.osi7.De;
import de.bsvrz.sys.funclib.debug.Debug;

import java.text.SimpleDateFormat;
import java.util.Date;

//~ JDK IMPORTE ===============================================================

//~ KLASSEN ===================================================================

/**
 * Enthlt alle Informationen fr einen Aggregationsauftrag.
 *
 * @author inovat, innovative systeme - verkehr - tunnel - technik
 * @author Dipl.-Ing. Hans Christian Kni (HCK)
 * @version $Revision$ / $Date$ / ($Author$)
 */
public class Auftrag {

    /**
     * DebugLogger fr Debug-Ausgaben.
     */
    private static final Debug debug = Debug.getLogger();
    
   
    /**
     * Datenmodell der Konfiguration.
     */
    private static DataModel _konfig;

    //~ FELDER ================================================================

    /**
     * Datenbeschreibung (ATG, ASP), welches ueberwacht wird
     */
    private DataDescription _datenBeschreibung;

    /**
     * De, welches ueberwacht wird
     */
    private De _de;

    /**
     * PID des Objekttyps, welcher ueberwacht wird
     */
    private String _pidObjektTyp;

    /**
     * Zeitpunkt der geplanten Ausfhrung fr diesen Auftrag in Millisekunden seit 1.1.1970
     */
    private long _ausfuehrungsZeitPunkt = 0;

    /**
     * Datumsformatierung fr Ausgabe toString()
     */
    SimpleDateFormat _datumsFormatierer = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss,SSS");

    //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==============

    /**
     * Konstruktor, um den initialen Erstauftrag fr ein Verwaltungsobjekt und das erste zu berechnende Intervall
     * anzulegen
     *
     * @param konfig                   Das Objekt zum Zugriff auf die Konfiguration
     * @param de                       Das aktuelle DE, welches ueberwacht wird
     * @param datenBeschreibung        Die Datenbeschreibung (ATG/ASP), die ueberwacht wird
     * @param pidObjektTyp             Die PID des Objekttyps (passend zur datenBeschreibung), die ueberwacht wird. Dies kann
     *                                 z. B. der Fahrstreifen sein, der auf die ueberwachte DE referenziert.
     * @param intervallBeginn          Der Intervallbeginn der aktuellen gueltigen Daten in ms seit dem 01.01.1970
     * @param intervallDauerInSekunden Die aktuell gueltige Intervalldauer fuer diese Daten. Aus Intervallbeginn,
     *                                 Intervalldauer und der AnzahlIntervalleFuerDatenAusfallErkennung wird der
     *                                 Zeitpunkt ermittelt, an dem spaetestens der naechste gueltige Datensatz
     *                                 empfangen werden muss. Sonst wird ein leerer Datensatz verschickt.
     */
    public Auftrag(DataModel konfig, De de, DataDescription datenBeschreibung, String pidObjektTyp, long intervallBeginn, long intervallDauerInSekunden) {
        _konfig                = konfig;
        _de                    = de;
        _datenBeschreibung     = datenBeschreibung;
        _pidObjektTyp          = pidObjektTyp;
        _ausfuehrungsZeitPunkt = intervallBeginn + intervallDauerInSekunden * 1000 * TlsApplication.getAnzahlIntervalleFuerDatenAusfallErkennung();
    }

    //~ METHODEN ==============================================================

    /**
     * Fhrt den Auftrag aus, indem am zugeordneten DeConversionModul der De die Methode {@link
     * de.bsvrz.kex.tls.osi7.conversion.DeConversionModul#sendData(de.bsvrz.dav.daf.main.ResultData)} aufgerufen wird.
     */
    public void ausfuehren() {

        // Jetzt fr alle angemeldeten Objekte auf diese Konvertierung einen ResultData mit Data == null
        // erzeugen und versenden (Austausch des Zielobjekts):
        SystemObject[] objects     = _de.getObjectRefsToDe(_konfig.getType(_pidObjektTyp));
        ResultData[]   results     = new ResultData[objects.length];
        long           zeitStempel = System.currentTimeMillis();

        for (int i = 0; i < objects.length; ++i) {
            results[i] = new ResultData(objects[i], _datenBeschreibung, zeitStempel, null);
            debug.info(String.format("DeAusfallUeberwachung hat 'KEINE DATEN' versandt: %s-%s-%s", _datumsFormatierer.format(new Date(zeitStempel)), _datenBeschreibung.toString(), objects[i].getPidOrNameOrId()));
        }

        _de.getConversionModul().sendData(results);
        
        
    }

    //~ GET METHODEN ==========================================================

    /**
     * Liefert den Zeitpunkt der geplanten Ausfhrung fr diesen Auftrag in Millisekunden seit 1.1.1970
     *
     * @return Zeitpunkt der geplanten Ausfhrung fr diesen Auftrag in Millisekunden seit 1.1.1970
     */
    public long getAusfuehrungsZeitPunkt() {
        return _ausfuehrungsZeitPunkt;
    }

    //~ METHODEN ==============================================================

    /**
     * Vergleicht diesen Auftrag mit einem anderen Auftrag auf Gleichheit wobei der  AusfuehrungsZeitPunkts des neuen Auftrags
     * weiter in der Zukunft liegen muss, damit sich keine Auftraege mit alten Zeitstempeln (z. B. nachgelieferte Daten)
     * einreihen koennen.
     * 
     * @param auftragNeu Der zu vergleichende Auftrag
     * @return <code>true</code>, wenn dieser Auftrag gleich aber aelter als zu vergleichendene Auftrag ist, sonst <code>false</code>.
     */
    public boolean istAelterAls(Auftrag auftragNeu) {
        if ((this._de.equals(auftragNeu._de)) && (this._datenBeschreibung.equals(auftragNeu._datenBeschreibung)) && (this._pidObjektTyp.equals(auftragNeu._pidObjektTyp)) &&
                (this._ausfuehrungsZeitPunkt < auftragNeu._ausfuehrungsZeitPunkt)) {
            return true;
        }

        return false;
    }

    @Override
    public String toString() {
        final StringBuilder sb = new StringBuilder();
        sb.append("Auftrag");
        sb.append("{_ausfuehrungsZeitPunkt=").append(_datumsFormatierer.format(new Date(_ausfuehrungsZeitPunkt)));
        sb.append(", _datenBeschreibung=").append(_datenBeschreibung);
        sb.append(", _de=").append(_de.getDe().getPidOrNameOrId());
        sb.append(", _pidObjektTyp='").append(_pidObjektTyp).append('\'');
        sb.append('}');
        return sb.toString();
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 07.03.12
