/*
 * Copyright (c) 2015 by inovat, innovative systeme - verkehr - tunnel - technik, Dipl.-Ing. H. C. Kniss
 *
 * This file is part of de.bsvrz.kex.tls.osi7.conversion.Fg004Default
 *
 * de.bsvrz.kex.tls.osi7.conversion.Fg004Default is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  de.bsvrz.kex.tls.osi7.conversion.Fg004Default is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with de.bsvrz.kex.tls.osi7.conversion.Fg004Default.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Diese Datei ist Teil von de.bsvrz.kex.tls.osi7.conversion.Fg004Default.
 *
 *  de.bsvrz.kex.tls.osi7.conversion.Fg004Default ist Freie Software: Sie knnen es unter den Bedingungen
 *  der GNU General Public License, wie von der Free Software Foundation,
 *  Version 3 der Lizenz oder (nach Ihrer Wahl) jeder spteren
 *  verffentlichten Version, weiterverbreiten und/oder modifizieren.
 *
 *  de.bsvrz.kex.tls.osi7.conversion.Fg004Default wird in der Hoffnung, dass es ntzlich sein wird, aber
 *  OHNE JEDE GEWHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 *  Gewhrleistung der MARKTFHIGKEIT oder EIGNUNG FR EINEN BESTIMMTEN ZWECK.
 *  Siehe die GNU General Public License fr weitere Details.
 *
 *  Sie sollten eine Kopie der GNU General Public License zusammen mit diesem
 *  Programm erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 *
 * Contact Information:
 * inovat, Dipl.-Ing. H. C. Kniss
 * An der Krautwiese 37
 * D-53783 Eitorf
 * +49 (0)2243 8464 193
 * info@inovat.de
 * www.inovat.de
 */



package de.bsvrz.kex.tls.osi7.conversion;

//~ NICHT JDK IMPORTE =========================================================

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.IntegerValueState;
import de.bsvrz.kex.tls.osi7.De;
import de.bsvrz.kex.tls.osi7.DeBlock;
import de.bsvrz.kex.tls.osi7.Eak;
import de.bsvrz.kex.tls.osi7.IllegalOsi7TelegramStructureException;
import de.bsvrz.kex.tls.osi7.TlsApplicationLayer;
import de.bsvrz.kex.tls.osi7.TlsData;
import de.bsvrz.sys.funclib.debug.Debug;
import de.bsvrz.sys.funclib.hexdump.HexDumper;

import java.util.ArrayList;
import java.util.List;
import java.util.Vector;

//~ JDK IMPORTE ===============================================================

//~ KLASSEN ===================================================================

/**
 * Umsetungsmodul fr die FG 4, Standard (alle Typen der FG 4 gem TLS 2002) Wird standardmig fr alle FG 4 De
 * verwendet, bei denen keine spezielle Umsezungsklasse in der Konfiguration angegeben wurde. Erweiterung: untersttzt
 * auch nachgelieferte Telegramme (Standard ID+32)
 *
 * @author inovat, innovative systeme - verkehr - tunnel - technik
 * @author inovat, innovative systeme - verkehr - tunnel - technik
 * @version $Revision$ / $Date$ / ($Author$)
 */
public class Fg004Default extends FgAllDefault {

    // ------------------------------------------------------------------------
    // VARIABLENDEKLARATIONEN
    // ------------------------------------------------------------------------
    // Sonstige ---------------------------------------------------------------

    /** DebugLogger fr Debug-Ausgaben */
    private static final Debug debug = Debug.getLogger();

    // Private ----------------------------------------------------------------

    //~ FELDER ================================================================

    /**
     * Tabelle, in der das Verhalten beim Empfang von Attributgruppen eingetragen wird. Jede Tabellenzeile hat folgenden
     * Aufbau:
     * <pre>
     * "Attributgruppe (Pid)", "Aspekt (Pid)", "Fg", "Typ", "Id", "ObjektTyp"
     * z.B.
     * "atg.tlsGloKanalSteuerung", "asp.parameterSoll", "1", "29", "2",
     * "typ.deLve"
     * </pre>
     * Die Auswertung erfolgt dabei wie folgt: <ul> <li> Fr jede Attributguppen/Aspekt Kombination erfolgt eine
     * Empfangsanmeldung als Empfnger.</li> <li> In der Methode {@link #update} wird fr jede zur
     * Attributgruppen/Aspket-Kombination passende Kombination eine Methode aufgerufen, deren Namen aus dem Tabelleneintrag
     * wie folgt gebildet wird (hier fr den Beispieleintrag):<br> <code>Fg001Typ029Id002AtgTlsGloKanalSteuerungAspParameterSoll(ResultData)</code><br>
     * Dabei enthlt <code>ResultData</code> die Daten der zu verarbeitenden ATG/ASP-Kombination.</li> <li> Fr jede
     * Attributguppen/Aspekt Kombination erfolgt eine Empfangsanmeldung als Empfnger. Sind mehrere passende Eintrage
     * vorhanden, werden entsprechend mehrere Methodenaufrufe durchgefhrt. Dadurch knnen mehrere ATG/ASP-Kombinationen in
     * einen De-Block abgebildet werden (z.B. protokollnahe TLS-Attributgruppe und Attributgruppe eines Fachmodells):</li>
     * <li> Ein Eintrag von "255" entspricht dabei einer "Wildcard" (passt immer). </ul>
     */
    private final Object[][] davToTls = new Object[][] {

        // Attributgruppe, Aspekt, Fg, Typ, Id, ObjektTyp                                     //
        { "atg.tlsGloDeBlockAbruf", "asp.tlsAbruf", "255", "255", "255", "typ.deWzg" },           //
        { "atg.tlsGloDeBlockSenden", "asp.tlsVorgabe", "255", "255", "255", "typ.deWzg" },        //
        { "atg.tlsGloKanalSteuerung", "asp.parameterSoll", "255", "29", "2", "typ.deWzg" },       //
        { "atg.tlsGloGeographischeKenndaten", "asp.tlsVorgabe", "255", "36", "3", "typ.deWzg" },  //
        { "atg.tlsWzgBetriebsArt", "asp.parameterSoll", "4", "17", "2", "typ.deWzg" },            //
        { "atg.tlsWzgAbrufPufferInhalt", "asp.tlsAbruf", "4", "20", "2", "typ.deWzg" },           //
        { "atg.tlsWzgAbrufCodeDefinitionen", "asp.tlsAbruf", "4", "21", "3", "typ.deWzg" },       //
        { "atg.tlsWzgWvzGrundEinstellung", "asp.parameterSoll", "4", "32", "3", "typ.deWzg" },    //
        { "atg.tlsWzgGrundEinstellung", "asp.parameterSoll", "4", "33", "3", "typ.deWzg" },       //
        { "atg.tlsWzgGrundProgramm", "asp.parameterSoll", "4", "42", "3", "typ.deWzg" },          //
        { "atg.tlsWzgCodeDefinition", "asp.parameterSoll", "4", "44", "3", "typ.deWzg" },         //
        { "atg.tlsWzgWvzStellZustand", "asp.tlsVorgabe", "4", "48", "5", "typ.deWzg" },           //
        { "atg.tlsWzgHelligkeit", "asp.tlsVorgabe", "4", "49", "5", "typ.deWzg" },                //
        { "atg.tlsWzgWechselText", "asp.tlsVorgabe", "4", "50", "5", "typ.deWzg" },               //
        { "atg.tlsWzgStellZustand", "asp.tlsVorgabe", "4", "55", "5", "typ.deWzg" },              //
        { "atg.tlsWzgStellProgramm", "asp.tlsVorgabe", "4", "58", "5", "typ.deWzg" },             //
        { "atg.tlsWzgAbrufBildDefinition", "asp.tlsAbruf", "4", "22", "3", "typ.deWzg" },         //
        { "atg.tlsWzgBildDefinition", "asp.tlsVorgabe", "4", "45", "3", "typ.deWzg" }             //
    };

    /**
     * Tabelle, in der das Verhalten beim Empfang von TLS De-Blcken eingetragen wird. Jede Tabellenzeile hat folgenden
     * Aufbau:
     * <pre>
     * "Attributgruppe (Pid)", "Aspekt (Pid)", "Fg", "Typ", "Id", "ObjektTyp"
     * z.B.
     * "atg.tlsGloKanalSteuerung", "asp.tlsAntwort", "1", "29", "2", "typ.deLve"
     * </pre>
     * Die Auswertung erfolgt dabei wie folgt: <ul> <li> Fr jede Attributguppen/Aspekt Kombination erfolgt eine
     * Sendeanmeldung als Quelle.</li> <li> In der Methode {@link #processDeBlock} wird fr jede zum De-Block passende
     * Fg/Id/Typ-Kombination eine Methode aufgerufen, deren Namen aus dem Tabelleneintrag wie folgt gebildet wird (hier fr
     * den Beispieleintrag):<br> <code>Fg001Typ029Id002AtgTlsGloKanalSteuerungAspTlsAntwort(DeBlock, Data,
     * DataDescription)</code><br> Dabei enthlt der <code>deBlock</code> die Daten des zu verarbeitenden De-Blocks und
     * <code>data</code> ein Datenobjekt der passenden Attributgruppen/Aspekt-Kombination, welchem direkt die Inhalte des
     * De-Blocks zugewiesen werden knnen.<code>DataDescription</code> enthlt die passende {@link
     * de.bsvrz.dav.daf.main.DataDescription} fr die zu sendenden Daten.</li> <li> Fr jede Attributguppen/Aspekt
     * Kombination erfolgt eine Sendeanmeldung als Quelle. Sind mehrere passende Eintrage vorhanden, werden entsprechend
     * mehrere Methodenaufrufe durchgefhrt. Dadurch kann ein De-Block mehrfach ausgewert und an verschiedene
     * Attributgruppen weitergerleitet werden (z.B. protokollnahe TLS-Attributgruppe und Attributgruppe eine
     * Fachmodells):</li> <li> Ein Eintrag von "255" entspricht dabei einer "Wildcard" (passt immer). </ul>
     */
    private final Object[][] tlsToDav = new Object[][] {
        { "atg.tlsGloDeFehler", "asp.tlsAntwort", "4", "1", "1", "typ.deWzg" },                     //
        { "atg.tlsGloDeFehler", "asp.tlsAntwort", "4", "1", "33", "typ.deWzg" },                    //
        { "atg.tlsGloKanalSteuerung", "asp.parameterIst", "255", "29", "2", "typ.deWzg" },          //
        { "atg.tlsGloKanalSteuerung", "asp.parameterIst", "255", "29", "34", "typ.deWzg" },         //
        { "atg.tlsGloKanalSteuerung", "asp.parameterIst", "4", "29", "34", "typ.deWzg" },           //
        { "atg.tlsGloGeographischeKenndaten", "asp.tlsZustand", "255", "36", "3", "typ.deWzg" },    //
        { "atg.tlsGloGeographischeKenndaten", "asp.tlsZustand", "255", "36", "35", "typ.deWzg" },   //
        { "atg.tlsWzgErgnzendeDeFehlerMeldung", "asp.tlsAntwort", "4", "14", "1", "typ.deWzg" },   //
        { "atg.tlsWzgErgnzendeDeFehlerMeldung", "asp.tlsAntwort", "4", "14", "33", "typ.deWzg" },  //
        { "atg.tlsWzgNichtDarstellbareWVZ", "asp.tlsAntwort", "4", "2", "1", "typ.deWzg" },         //
        { "atg.tlsWzgNichtDarstellbareWVZ", "asp.tlsAntwort", "4", "2", "33", "typ.deWzg" },        //
        { "atg.tlsWzgDefekteLampen", "asp.tlsAntwort", "4", "3", "1", "typ.deWzg" },                //
        { "atg.tlsWzgDefekteLampen", "asp.tlsAntwort", "4", "3", "33", "typ.deWzg" },               //
        { "atg.tlsWzgGestrteTextPositionen", "asp.tlsAntwort", "4", "4", "1", "typ.deWzg" },       //
        { "atg.tlsWzgGestrteTextPositionen", "asp.tlsAntwort", "4", "4", "33", "typ.deWzg" },      //
        { "atg.tlsWzgDefekteLEDKetten", "asp.tlsAntwort", "4", "5", "1", "typ.deWzg" },       //
        { "atg.tlsWzgDefekteLEDKetten", "asp.tlsAntwort", "4", "5", "33", "typ.deWzg" },      //
        { "atg.tlsWzgNegativeQuittung", "asp.tlsAntwort", "4", "16", "2", "typ.deWzg" },            //
        { "atg.tlsWzgNegativeQuittung", "asp.tlsAntwort", "4", "16", "34", "typ.deWzg" },           //
        { "atg.tlsWzgBetriebsArt", "asp.parameterIst", "4", "17", "2", "typ.deWzg" },               //
        { "atg.tlsWzgBetriebsArt", "asp.parameterIst", "4", "17", "34", "typ.deWzg" },              //
        { "atg.tlsWzgWvzGrundEinstellung", "asp.parameterIst", "4", "32", "3", "typ.deWzg" },       //
        { "atg.tlsWzgWvzGrundEinstellung", "asp.parameterIst", "4", "32", "35", "typ.deWzg" },      //
        { "atg.tlsWzgGrundEinstellung", "asp.parameterIst", "4", "33", "3", "typ.deWzg" },          //
        { "atg.tlsWzgGrundEinstellung", "asp.parameterIst", "4", "33", "35", "typ.deWzg" },         //
        { "atg.tlsWzgGrundProgramm", "asp.parameterIst", "4", "42", "3", "typ.deWzg" },             //
        { "atg.tlsWzgGrundProgramm", "asp.parameterIst", "4", "42", "35", "typ.deWzg" },            //
        { "atg.tlsWzgCodeListe", "asp.tlsAntwort", "4", "43", "3", "typ.deWzg" },                   //
        { "atg.tlsWzgCodeListe", "asp.tlsAntwort", "4", "43", "35", "typ.deWzg" },                  //
        { "atg.tlsWzgCodeDefinition", "asp.parameterIst", "4", "44", "3", "typ.deWzg" },            //
        { "atg.tlsWzgCodeDefinition", "asp.parameterIst", "4", "44", "35", "typ.deWzg" },           //
        { "atg.tlsWzgWvzStellZustand", "asp.tlsZustand", "4", "48", "5", "typ.deWzg" },             //
        { "atg.tlsWzgWvzStellZustand", "asp.tlsZustand", "4", "48", "37", "typ.deWzg" },            //
        { "atg.tlsWzgHelligkeit", "asp.tlsZustand", "4", "49", "5", "typ.deWzg" },                  //
        { "atg.tlsWzgHelligkeit", "asp.tlsZustand", "4", "49", "37", "typ.deWzg" },                 //
        { "atg.tlsWzgWechselText", "asp.tlsZustand", "4", "50", "5", "typ.deWzg" },                 //
        { "atg.tlsWzgWechselText", "asp.tlsZustand", "4", "50", "37", "typ.deWzg" },                //
        { "atg.tlsWzgStellZustand", "asp.tlsZustand", "4", "55", "5", "typ.deWzg" },                //
        { "atg.tlsWzgStellZustand", "asp.tlsZustand", "4", "55", "37", "typ.deWzg" },               //
        { "atg.tlsWzgStellProgramm", "asp.tlsZustand", "4", "58", "5", "typ.deWzg" },               //
        { "atg.tlsWzgStellProgramm", "asp.tlsZustand", "4", "58", "37", "typ.deWzg" },              //
        { "atg.tlsWzgBildDefinition", "asp.tlsZustand", "4", "45", "3", "typ.deWzg" },              //
        { "atg.tlsWzgBildDefinition", "asp.tlsZustand", "4", "45", "35", "typ.deWzg" }              //
    };

    /**
     * Gltiger WvzCode, der beim Einschalten oder als Rckmeldung verwendet wurde. Da sich eine Reihe von
     * Softwareentwicklern fr die SM/EAK-Software bei den Firmen auf eine etwas seltsame Interpretation der TLS im Falle
     * des Ausschaltens von WVZ-Anzeigen versteift (1), wird bis zur verbindlichen Festlegung einer vernnftigen
     * Interpretation ein WVZ beim Ausschalten jeweils mit dem letzten verschickten oder empfangenen WVZ-Code einer
     * Einschaltsequenz ausgeschaltet. (1) Das Schalten einer WVZ-Anzeige erfolgt ber zwei Steuerbyte: dem eigentlichen
     * Befehl ob ein- oder auszuschalten ist und dem darzustellenden WVZ-Code. Letzterer macht logischerweise nur Sinn beim
     * Einschalten. Einige Entwickler versteifen sich aber darauf, dass auch beim Ausschalten ein gltiger Code verwendet
     * werden muss und schalten das WZG ansonsten nicht aus (obwohl kein Mensch erklren kann, warum hier so ein unntiger
     * Programmieraufwand getrieben wird, der zudem einen sicheren Betrieb erheblich erschwert...Im Zweifelsfall kann
     * nmlich eine Fehlanzeige nicht ohne weiteres ber eine Mnuelle Schaltung ausgeschaltet werden, weil die Liste der
     * gltigen Anzeigecode nicht bekannt ist!!).
     */
    private int _validWvzCode = 0;  // initial auf 0 setzen

    //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==============

    // ------------------------------------------------------------------------
    // KONSTRUKTOREN  (und vom Konstruktor verwende Methoden)
    // ------------------------------------------------------------------------

    /** Erzeugt ein Objekt vom Typ Fg001Default */
    public Fg004Default(ClientDavInterface daf, Eak eak, De de) {
        super(daf, eak, de);
        subscribeAllData(tlsToDav, davToTls);
    }

    //~ METHODEN ==============================================================

    // ------------------------------------------------------------------------
    // SONSTIGE METHODEN
    // ------------------------------------------------------------------------

    /**
     * FG 4 : Typ: 1 Id: 1 Name: DE-Fehler (Antwort). Ergnzt globale Behandlung De-Fehler in {@link
     * #Fg255Typ001Id001AtgTlsGloDeFehlerAspTlsAntwort}. Beim Empfang einer De-Gutmeldung wird die De initialisiert, indem
     * <ul> <li>WZG-kanalsteuerung {@link #Fg255Typ029Id002AtgTlsGloKanalSteuerungAspParameterSoll}</li> TBD: Folgende
     * Einstellung bei FG 4 nach DE-Gutmeldung versenden (aber wie werden die verschiedenen Bauformaen, TLS-Stnde
     * konfiguriert ??? {"atg.tlsWzgBetriebsArt", "asp.parameterSoll", "4", "17", "2", "typ.deWzg"},
     * {"atg.tlsWzgWvzGrundEinstellung", "asp.parameterSoll", "4", "32", "3", "typ.deWzg"}, {"atg.tlsWzgGrundEinstellung",
     * "asp.parameterSoll", "4", "33", "3", "typ.deWzg"}, {"atg.tlsWzgGrundProgramm", "asp.parameterSoll", "4", "42", "3",
     * "typ.deWzg"}, {"atg.tlsWzgCodeListe", "asp.parameterSoll", "4", "43", "3", "typ.deWzg"}, <ul> versendet werden.
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz fr die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung fr die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg004Typ001Id001AtgTlsGloDeFehlerAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
        ResultData        resultData     = super.Fg255Typ001Id001AtgTlsGloDeFehlerAspTlsAntwort(deBlock, data, dataDescription);
        IntegerValueState DEFehlerStatus = data.getUnscaledValue("DEFehlerStatus").getState();

        if (DEFehlerStatus != null) {
            debug.fine("DEFehlerStatus: " + DEFehlerStatus.getName());

            if (DEFehlerStatus.getName().equals("ok")) {
                sendInitialDeBlock(deBlock.getKey(), 29, 2);  // Wzg-Kanalsteuerung senden

                // sendInitialDeBlock(deBlock.getKey(), 17, 2); // Wzg-tlsWzgBetriebsArt senden
                // sendInitialDeBlock(deBlock.getKey(), 32, 3); // Wzg-tlsWzgWvzGrundEinstellung senden
                // sendInitialDeBlock(deBlock.getKey(), 33, 3); // Wzg-tlsWzgGrundEinstellung senden
                // sendInitialDeBlock(deBlock.getKey(), 42, 3); // Wzg-tlsWzgGrundProgramm senden
                // sendInitialDeBlock(deBlock.getKey(), 43, 3); // Wzg-tlsWzgCodeListe senden
            } else {
                List deList = new ArrayList();

                deList.add(_de);
                TlsApplicationLayer.writeNoDataToUnreachableDe(deList, _eakConversionModul.getTimeStamp(), -1, "atg.tlsGloDeFehler");
            }
        }

        return resultData;
    }

    public ResultData Fg004Typ001Id033AtgTlsGloDeFehlerAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {

        // Standardverarbeitung wie bei Id 1, aber:
        // 1. als "nachgelieferte" Daten an den DaV bergeben.
        // 2. bei nachgelieferten Daten werden keine Sequenzen verschickt.
        ResultData resultData = super.Fg255Typ001Id001AtgTlsGloDeFehlerAspTlsAntwort(deBlock, data, dataDescription);

        return new ResultData(_de.getDe(), dataDescription, resultData.getDataTime(), resultData.getData(), true);
    }

    /**
     * FG 4 : Typ: 2 Id: 1 Name: Nicht darstellbare WVZ (Antwort). TLS-Datenbeschreibung:<br>
     * <pre>
     * Position         Bezeichnung                                 Erluterung
     * Byte 1           Lnge DE-Block                              Lnge des
     * folgenden DE-Blocks [3 + n ]
     * Byte 2           Daten-Endgerte-Kanal                       [ 1.. 254 ]
     * Byte 3           Typ der DE-Daten                            [ 2 ]
     * Byte 4           Anzahl Code-Bytes                           Anzahl nicht
     * mehr darstellbarer WVZ
     * Byte 5           1. WVZ-Code                                 1. nicht mehr
     * darstellb. Zeichen (lt. Anh. 7, Kap.1)
     * Byte .. n.       WVZ-Code                                    n. nicht mehr
     * darstellb. Zeichen (lt. Anh. 7, Kap. 1)
     * </pre>
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz fr die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung fr die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg004Typ002Id001AtgTlsWzgNichtDarstellbareWVZAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
        ResultData resultData = null;

        // WVZ-Codes aus Telegramm -----------------------------------------------------------------------
        int   count     = deBlock.getDeData().readUnsignedByteAsInt();
        int[] _WvzCode_ = new int[count];

        for (int i = 0; i < count; i++) {
            _WvzCode_[i] = deBlock.getDeData().readUnsignedByteAsInt();
        }

        Data.NumberArray _NichtDarstellbareWVZ_ = data.getUnscaledArray("NichtDarstellbareWVZ");

        _NichtDarstellbareWVZ_.setLength(count);

        for (int i = 0; i < count; i++) {
            _NichtDarstellbareWVZ_.getValue(i).set(_WvzCode_[i]);
        }

        // Ergebnisdatensatz erzeugen...
        resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);

        return resultData;
    }

    /**
     * FG 4 : Typ: 2 Id: 33 Name: Nicht darstellbare WVZ (Antwort). TLS-Datenbeschreibung:<br> Standardverarbeitung wie bei
     * Id 1, aber als "nachgelieferte" Daten an den DaV bergeben.
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz fr die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung fr die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg004Typ002Id033AtgTlsWzgNichtDarstellbareWVZAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {

        // Standardverarbeitung wie bei Id 1, aber als "nachgelieferte" Daten an den DaV bergeben.
        ResultData resultData = Fg004Typ002Id001AtgTlsWzgNichtDarstellbareWVZAspTlsAntwort(deBlock, data, dataDescription);

        return new ResultData(_de.getDe(), dataDescription, resultData.getDataTime(), resultData.getData(), true);
    }

    /**
     * FG 4 : Typ: 3 Id: 1 Name: Defekte Lampen (Antwort). TLS-Datenbeschreibung:<br>
     * <pre>
     * Position         Bezeichnung                               Erluterung
     * Byte 1           Lnge DE-Block                            Lnge des
     * folgenden DE-Blocks [3 + n ]
     * Byte 2           Daten-Endgerte-Kanal (DE)                [ 1..254 ]
     * Byte 3           Typ der DE-Daten                          [ 3 ]
     * Byte 4           Anzahl Lampenpaare
     * Byte 5           1. Lampen-Statusbyte                      Enthlt die
     * Gut/Schlecht-Aussage von 8 Lampen
     * Byte .. n.       Lampen-Statusbyte                         Enthlt die
     * Gut/Schlecht-Aussage von 8 Lampen
     * </pre>
     * Inhalt 1. Lampen-Statusbyte
     * <pre>
     *    7        6        5        4        3        2        1        0
     *    NL 4     HL 4     NL 3     HL 3     NL 2     HL 2     NL 1     HL 1
     * </pre>
     * Bits 0,2,4,6 :    Hauptlampen 1 bis 4,    (0 = OK, 1 = defekt)<br> Bits 1,3,5,7 :    Nebenlampen 1 bis 4,    (0 =
     * OK, 1 = defekt)<br> <br> Die weiteren Lampen-Statusbytes enthalten die Lampenpaare in entsprechend aufsteigender
     * Folge.<br>
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz fr die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung fr die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg004Typ003Id001AtgTlsWzgDefekteLampenAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
        ResultData resultData = null;

        // Lampen-Paare (je 2 Lampen)  aus Telegramm -----------------------------------------------------------------------
        int countLightPairs = deBlock.getDeData().readUnsignedByteAsInt();
        int count;

        if ((countLightPairs % 4) == 0) {
            count = countLightPairs / 4;
        } else {
            count = (countLightPairs / 4) + 1;
        }

        int[] _LampenStatus_ = new int[count];

        for (int i = 0; i < count; i++) {
            _LampenStatus_[i] = deBlock.getDeData().readUnsignedByteAsInt();
        }

        Data.NumberArray _ZustandLampenPaar_ = data.getUnscaledArray("ZustandLampenPaar");

        _ZustandLampenPaar_.setLength(countLightPairs);

        int lightState;

        for (int i = 0; i < countLightPairs; i++) {
            lightState = (_LampenStatus_[i / 4] >>> (i % 4) * 2) & 0x03;
            _ZustandLampenPaar_.getValue(i).set(lightState);
        }

        // Ergebnisdatensatz erzeugen...
        resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);

        return resultData;
    }

    /**
     * Entspricht {@link Fg004Default#Fg004Typ003Id001AtgTlsWzgDefekteLampenAspTlsAntwort(de.bsvrz.kex.tls.osi7.DeBlock,
     * de.bsvrz.dav.daf.main.Data, de.bsvrz.dav.daf.main.DataDescription)}, jedoch werden die Daten als nachgeliefert gekennzeichnet.
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz fr die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung fr die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg004Typ003Id033AtgTlsWzgDefekteLampenAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {

        // Generierte Methode: Standardverarbeitung wie bei Id 1, aber als "nachgelieferte" Daten an den DaV bergeben.
        ResultData resultData = Fg004Typ003Id001AtgTlsWzgDefekteLampenAspTlsAntwort(deBlock, data, dataDescription);

        return new ResultData(_de.getDe(), dataDescription, resultData.getDataTime(), resultData.getData(), true);
    }

    /**
     * FG 4 : Typ: 4 Id: 1 Name: Gestrte Textpositionen (Antwort). TLS-Datenbeschreibung:<br>
     * <pre>
     * Position         Bezeichnung
     * Erluterung
     * Byte 1           Lnge DE-Block                                      Lnge
     * des folgenden DE-Blocks [3 + n ]
     * Byte 2           Daten-Endgerte-Kanal (DE)                          [
     * 1..254 ]
     * Byte 3           Typ der DE-Daten                                    [ 4]
     * Byte 4           Anzahl ausgefallener Textpositionen                 [
     * 0..213 ]
     * Byte 5           1. gestrte Textposition Position der Textanzeige   [
     * 1..213 ]
     * Byte ..          n. gestrte Textposition Position der Textanzeige   [
     * 1..213 ]
     * </pre>
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz fr die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung fr die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg004Typ004Id001AtgTlsWzgGestrteTextPositionenAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
        ResultData resultData = null;

        // Gestrte Textpositionen aus Telegramm -----------------------------------------------------------------------
        int   count            = deBlock.getDeData().readUnsignedByteAsInt();
        int[] _TextPositionen_ = new int[count];

        for (int i = 0; i < count; i++) {
            _TextPositionen_[i] = deBlock.getDeData().readUnsignedByteAsInt();
        }

        Data.NumberArray _GestrteTextPosition_ = data.getUnscaledArray("GestrteTextPosition");

        _GestrteTextPosition_.setLength(count);

        for (int i = 0; i < count; i++) {
            _GestrteTextPosition_.getValue(i).set(_TextPositionen_[i]);
        }

        // Ergebnisdatensatz erzeugen...
        resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);

        return resultData;
    }

    /**
     * FG 4 : Typ: 4 Id: 33 Name: Gestrte Textpositionen (Antwort). TLS-Datenbeschreibung:<br> Standardverarbeitung wie
     * bei Id 1, aber als "nachgelieferte" Daten an den DaV bergeben.
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz fr die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung fr die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg004Typ004Id033AtgTlsWzgGestrteTextPositionenAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {

        // Standardverarbeitung wie bei Id 1, aber als "nachgelieferte" Daten an den DaV bergeben.
        ResultData resultData = Fg004Typ004Id001AtgTlsWzgGestrteTextPositionenAspTlsAntwort(deBlock, data, dataDescription);

        return new ResultData(_de.getDe(), dataDescription, resultData.getDataTime(), resultData.getData(), true);
    }

    /**
     * FG 4 : Typ: 5 Id: 1 Name: Defekte LED-Ketten (Antwort). TLS-Datenbeschreibung:<br>
     * <pre>
     * Position         Bezeichnung                               Erluterung
     * Byte 1           Lnge DE-Block                            Lnge des folgenden DE-Blocks [3 + n ]
     * Byte 2           Daten-Endgerte-Kanal (DE)                [ 1..254 ]
     * Byte 3           Typ der DE-Daten                          [ 5 ]
     * Byte 4           Anzeigeprinzip
     * Byte 5           Schwere des Ausfalls
     * Byte 6           Anzahl vorhandene Ketten (Low)
     * Byte 7           Anzahl vorhandene Ketten (High)
     * Byte 8           Anzahl bertragene Ketten (Low)
     * Byte 9           Anzahl bertragene Ketten (High)
     * Byte 10          1. Ketten-Statusbyte                      Enthlt die Gut/Schlecht-Aussage der ersten 8 Ketten
     * Byte ..          n. Ketten-Statusbyte                      Enthlt die Gut/Schlecht-Aussage von 8 jeweils aufsteigend folgenden Ketten
     * </pre>
     * Inhalt 1. Ketten-Statusbyte
     * <pre>
     *    7        6        5        4        3        2        1        0
     *    Kette 8  Kette 7  Kette 6  Kette 5  Kette 4  Kette 3  Kette 2  Kette 1
     * </pre>
     * Bits 0,1,2,3,4,5,6,7 :    Kette 1 bis 8,    (0 = Keine Strung, 1 = Strung)
     * <br><br>
     * Die weiteren Ketten-Statusbytes enthalten die Ketten in entsprechend aufsteigender Folge.<br><br>
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz fr die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung fr die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg004Typ005Id001AtgTlsWzgDefekteLEDKettenAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
        ResultData resultData = null;

        // Anzeige-Prinzip
        int anzeigePrinzip = deBlock.getDeData().readUnsignedByteAsInt();

        data.getUnscaledValue("AnzeigePrinzip").set(anzeigePrinzip);

        // Schwere des Ausfalls
        int schwereDesAusfalls = deBlock.getDeData().readUnsignedByteAsInt();

        data.getUnscaledValue("SchwereDesAusfalls").set(schwereDesAusfalls);

        // Anzahl vorhandene Ketten
        int anzahlVorhandeneKetten = deBlock.getDeData().readUnsignedLowHighByteAsInt();

        data.getUnscaledValue("AnzahlVorhandeneLEDKetten").set(anzahlVorhandeneKetten);

        // Anzahl bertragene Ketten
        int anzahlUebertrageneKetten = deBlock.getDeData().readUnsignedLowHighByteAsInt();

        // Anzahl Ketten-Status-Bytes bestimmen
        int anzahlUebertrageneBytes = ((anzahlUebertrageneKetten+7)/8);

        Vector<Integer> vectorDefekteKetten = new Vector<Integer>();

        int kette = 1;

        int kettenStatusByte = 0;

        for (int i = 0; i < anzahlUebertrageneBytes; i++)
        {
          // x. Ketten-Status-Byte aus Telegramm holen
          kettenStatusByte = deBlock.getDeData().readUnsignedByteAsInt();

          for (int j = 1; j < 256; j*=2)
          {
            if ((kettenStatusByte & j) > 0)
            {
              if (kette <= anzahlUebertrageneKetten)
              {
                vectorDefekteKetten.add(kette);
              }
            }

            kette++;
          }
        }

        data.getUnscaledArray("DefekteLEDKetten").setLength(vectorDefekteKetten.size());

        for (int i = 0; i < vectorDefekteKetten.size(); i++)
        {
          data.getUnscaledArray("DefekteLEDKetten").getValue(i).set((int)vectorDefekteKetten.get(i));
        }

        // Ergebnisdatensatz erzeugen...
        resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);

        return resultData;
    }

    /**
     * Entspricht {@link Fg004Default#Fg004Typ005Id001AtgTlsWzgDefekteLEDKettenAspTlsAntwort(de.bsvrz.kex.tls.osi7.DeBlock,
     * de.bsvrz.dav.daf.main.Data, de.bsvrz.dav.daf.main.DataDescription)}, jedoch werden die Daten als nachgeliefert gekennzeichnet.
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz fr die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung fr die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg004Typ005Id033AtgTlsWzgDefekteLEDKettenAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {

        // Generierte Methode: Standardverarbeitung wie bei Id 1, aber als "nachgelieferte" Daten an den DaV bergeben.
        ResultData resultData = Fg004Typ005Id001AtgTlsWzgDefekteLEDKettenAspTlsAntwort(deBlock, data, dataDescription);

        return new ResultData(_de.getDe(), dataDescription, resultData.getDataTime(), resultData.getData(), true);
    }

    /*
     * Ende Erweiterung STS 2013.02
     */

    /**
     * FG 4 : Typ: 14 Id: 1  Name: Ergnzende DE-Fehlermeldung (Antwort). Ergnzt globale Behandlung Kanal-Steuerung in
     * {@link #Fg004Typ029Id034AtgTlsGloKanalSteuerungAspParameterIst}.
     * <pre>
     * Position         Bezeichnung                             Erluterung
     * Byte 1           Lnge DE-Block                          [ 9..29 ]
     * Byte 2           Daten-Endgerte-Kanal                   [ 1..254 ]
     * Byte 3           Typ der DE-Daten                        [ 14 ]
     * Byte 4           Herstellercode                          (s. Anh. 7, Kap.
     * 2)
     * Byte 5           Anzahl TLS - Fehlerbytes                [ 4 ]
     * Byte 6           TLS - Fehlerbyte 1                      Codierung siehe
     * unten
     * Byte 7           TLS - Fehlerbyte 2                      Codierung siehe
     * unten
     * Byte 8           TLS - Fehlerbyte 3                      Codierung siehe
     * unten
     * Byte 9           TLS - Fehlerbyte 4                      Codierung siehe
     * unten
     * Byte 10          Anzahl Hersteller Fehlerbytes           [ 0..20 ]
     * Byte 11          Hersteller-Fehlerbyte 1                 herstellerspez.
     * Codierung
     * Byte..           Hersteller-Fehlerbyte n                 herstellerspez.
     * Codierung
     * </pre>
     * <br> Inhalt TLS-Fehlerbytes:<br> Die Fehlerbytes sind bitweise codiert. Jedes Bit definiert einen bestimmten Fehler,
     * wobei das Bit gleich 1 gesetzt ist.<br> Es knnen auch mehrere Bit gleichzeitig gesetzt werden. Der Inhalt der
     * Fehlerbytes ist optimal. D.h. bei einer Realisierung dieses Telegrammtyps mssen vom Hersteller nicht alle in der
     * Tabelle definierten Fehlercodes erfllt werden. Fr sptere Definitionen reservierte Bits bzw. nicht verwendete Bits
     * sind gleich Null zu setzen.<br>
     * <pre>
     *         Bit 7      Bit 6       Bit 5       Bit 4          Bit 3         Bit
     * 2       Bit 1       Bit 0
     * Byte 1  Reserve    Reserve     RNR am      Lokalbus       Reserve
     * Reserve     Reserve     Teilstrung
     *                                Lokalbus1   ausgefallen
     * Byte 2  Reserve    Reserve     Fehler in   Fehler in      WZG Ver-      WZG
     *        RNR am      Subbus
     *                                Wechseltext-Prismenan-     sorgungs-
     * defekt      Subbus      ausgefallen
     *                                steuerung   steuerung      spannung
     *                                                           unter/ber-
     *                                                           schritten
     * Byte 3  Reserve    Reserve     Reserve     Reserve        Reserve
     * Reserve     Reserve     Reserve
     * Byte 4  Reserve    Reserve     Reserve     Reserve        Reserve
     * Reserve     Reserve     Reserve
     * 1) nur in Verbindung mit St/SM=1 im Byte 4 der DE-Block-Struktur im Typ 1
     * </pre>
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz fr die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung fr die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg004Typ014Id001AtgTlsWzgErgnzendeDeFehlerMeldungAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
        ResultData resultData = null;

        // Herstellercode-Byte aus Telegramm ---------------------------------------------------------------------------
        int _Herstellercode_ = deBlock.getDeData().readUnsignedByteAsInt();

        data.getUnscaledValue("Hersteller").set(_Herstellercode_);

        // Fehler-Bytes aus Telegramm -----------------------------------------------------------------------------------
        deBlock.getDeData().readUnsignedByteAsInt();  // Anzahl der folgenden Bytes berlesen, da immer == 4 !

        int              _TlsFehler1_ = deBlock.getDeData().readUnsignedByteAsInt();
        int              _TlsFehler2_ = deBlock.getDeData().readUnsignedByteAsInt();
        int              _TlsFehler3_ = deBlock.getDeData().readUnsignedByteAsInt();
        int              _TlsFehler4_ = deBlock.getDeData().readUnsignedByteAsInt();
        Data.NumberArray _TlsFehler_  = data.getItem("FehlercodesTLS").getUnscaledArray("Fehler");

        _TlsFehler_.setLength(4);
        _TlsFehler_.getValue(0).set(_TlsFehler1_);
        _TlsFehler_.getValue(1).set(_TlsFehler2_);
        _TlsFehler_.getValue(2).set(_TlsFehler3_);
        _TlsFehler_.getValue(3).set(_TlsFehler4_);

        // Hersteller Fehler-Bytes aus Telegramm -----------------------------------------------------------------------
        int   count              = deBlock.getDeData().readUnsignedByteAsInt();
        int[] _HerstellerFehler_ = new int[count];

        for (int i = 0; i < count; i++) {
            _HerstellerFehler_[i] = deBlock.getDeData().readUnsignedByteAsInt();
        }

        Data.NumberArray _Fehler_ = data.getItem("FehlercodesHersteller").getUnscaledArray("Fehler");

        _Fehler_.setLength(count);

        for (int i = 0; i < count; i++) {
            _Fehler_.getValue(i).set(_HerstellerFehler_[i]);
        }

        // Ergebnisdatensatz erzeugen...
        resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);

        return resultData;
    }

    /**
     * FG 4 : Typ: 14 Id: 33 Name: Ergnzende DE-Fehlermeldung (Antwort). Ergnzt globale Behandlung Kanal-Steuerung in
     * {@link #Fg004Typ029Id034AtgTlsGloKanalSteuerungAspParameterIst}. Standardverarbeitung wie bei Id 2, aber als
     * "nachgelieferte" Daten an den DaV bergeben.
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz fr die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung fr die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg004Typ014Id033AtgTlsWzgErgnzendeDeFehlerMeldungAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {

        // Standardverarbeitung wie bei Id 2, aber als "nachgelieferte" Daten an den DaV bergeben.
        ResultData resultData = Fg004Typ014Id001AtgTlsWzgErgnzendeDeFehlerMeldungAspTlsAntwort(deBlock, data, dataDescription);

        return new ResultData(_de.getDe(), dataDescription, resultData.getDataTime(), resultData.getData(), true);
    }

    /**
     * FG 4 : Typ: 16 Id: 2 Name: Negative Quittung (Antwort). TLS-Datenbeschreibung:<br>
     * <pre>
     * Position         Bezeichnung                             Erluterung
     * Byte 1           Lnge DE-Block                          Lnge des folgenden
     * DE-Blocks [ 4 ]
     * Byte 2           Daten-Endgerte-Kanal                   [ 1..254 ]
     * Byte 3           Typ der DE-Daten                        [ 16 ]
     * Byte 4           Fehlerursache                           siehe unten
     * Byte 5           Herstellercode                          siehe Liste in
     * Anhang 7, Kap. 2
     * </pre>
     * Inhalt Byte: Fehlerursache unter FG 4:
     * <pre>
     *        0:   sonstige Fehlerursache
     *        1:   unbekannte oder nicht auswertbare ID
     *        2:   unbekannter oder nicht auswertbarer Typ
     *        3:   reserviert fr sptere Definition
     *        4:   Stellcode auf diesem WZG oder AQ nicht vorhanden
     *        5:   Stellcode wegen defekter Anzeige (Lampen, Prisma oder Rollo)
     * nicht schaltbar
     *        6:   Stellcode wegen sonstiger Hardwarestrungen nicht schaltbar
     *        7:   Helligkeitswert nicht einstellbar
     *        8:   Keine automatische Helligkeitssteuerung mglich
     *        9:   Betriebsart unbekannt bzw. nicht einstellbar
     *       10:   Befehl bzw. Programm in dieser Betriebsart nicht ausfhrbar
     *       11:   Nachricht nicht bearbeitbar, da Projektierungsdaten fehlerhaft
     * sind
     *       12:   Pufferinhalt nicht verfgbar
     *       13:   Prismencode nicht zulssig (z.B. Stellung 4 fr Dreikantprisma
     * oder Blinkercode fr Prisma)
     *       14:   Funktionsbyte unzulssig (z.B. Blinkfunktion nicht vorhanden)
     *       15:   Kanal passiviert
     *       16:   Verriegelungsmatrix verletzt (fr Stellzustnde eines AQ oder
     * Programm einer KBA)
     *       17:   Anzahl Prismen/Anzeigen oder DEs im Telegramm stimmt nicht mit
     * vorhandener Anzahl berein
     *       18:   Kein freier Speicherplatz zum Ablegen von Programmdaten
     * vorhanden
     *       19:   Wechseltext nicht darstellbar, da zu lang
     *       20:   Wechseltext enthlt nicht darstellbare Zeichen
     *       21:   falsches Anzeigeprinzip
     *  22 - 31:   reserviert fr sptere Definitionen
     *       32:   vorheriges Programm noch nicht abgeschlossen
     *       33:   Betriebsartenwechsel nicht abgeschlossen
     *       34:   Betriebsart nicht eindeutig feststellbar
     *       35:   Helligkeitsumschaltung noch nicht abgeschlossen
     *       36:   Helligkeit nicht eindeutig feststellbar
     *       37:   Keine Programmdaten verfgbar
     *       38:   bergang zum Signalprogramm nicht definiert
     *       39:   bergang zum Grundprogramm nicht definiert
     *       40:   Programmdaten unvollstndig
     *       41:   Stellcode-Vorrat nicht nderbar
     *       42:   Stellcode-Definition nicht nderbar
     *       43:   Unzulssiger Stellcode
     *       44:   Kurzversion mit diesem Stellcode nicht zulssig
     *       45:   Stellcode bei Steuerungsprinzip 1 nicht zulssig
     *       46:   Langversion nicht zulssig
     *       47:   Lschen benutzter Stellcodes nicht zulssig
     *       48:   Programmdefinition enthlt undefinierten Stellcode
     *  49..127:   reserviert fr sptere Definitionen
     * 128..255:   frei fr herstellerdefinierte Ursachencodes
     * </pre>
     * Inhalt Byte Herstellercode<br> Der Herstellercode ist ein fr jeden Hersteller eindeutig zugeordneter Code, der der
     * Zentrale die Zuordnung der Nachrichten erleichtern soll.<br>
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz fr die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung fr die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg004Typ016Id002AtgTlsWzgNegativeQuittungAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
        return super.Fg255Typ016Id002AtgTlsGloNegativeQuittungAspTlsAntwort(deBlock, data, dataDescription);
    }

    /**
     * Entspricht {@link Fg004Default#Fg004Typ016Id002AtgTlsWzgNegativeQuittungAspTlsAntwort(de.bsvrz.kex.tls.osi7.DeBlock,
     * de.bsvrz.dav.daf.main.Data, de.bsvrz.dav.daf.main.DataDescription)}, jedoch werden die Daten als nachgeliefert gekennzeichnet.
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz fr die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung fr die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg004Typ016Id034AtgTlsWzgNegativeQuittungAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {

        // Generierte Methode: Standardverarbeitung wie bei Id 2, aber als "nachgelieferte" Daten an den DaV bergeben.
        ResultData resultData = Fg004Typ016Id002AtgTlsWzgNegativeQuittungAspTlsAntwort(deBlock, data, dataDescription);

        return new ResultData(_de.getDe(), dataDescription, resultData.getDataTime(), resultData.getData(), true);
    }

    /**
     * FG 4 : Typ: 17 Id: 2 Name: Betriebsart (Antwort). TLS-Datenbeschreibung:<br>
     * <pre>
     * Position         Bezeichnung                                    Erluterung
     * Byte 1           Lnge DE-Block                                 Lnge des
     * folgenden DE-Blocks [ 3 ]
     * Byte 2           Daten-Endgerte-Kanal (DE)                     [193..222,
     * 223, (255)1]
     * Byte 3           Typ der DE-Daten                               [ 17 ]
     * Byte 4           Betriebsart                                    siehe unten
     * </pre>
     * 1) siehe Seite A6.1.1 - 3, Kapitel 1.4<br> Inhalt Byte: Betriebsart:<br>
     * <pre>
     *        0: frei
     *        1: Normalbetrieb
     *        2: Blindbetrieb
     *        3: Handbetrieb
     *        4: Autarker Betrieb
     *        5: Testbetrieb
     *        6: Notbetrieb
     *   7..255: noch nicht definiert
     * </pre>
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz fr die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung fr die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg004Typ017Id002AtgTlsWzgBetriebsArtAspParameterIst(DeBlock deBlock, Data data, DataDescription dataDescription) {
        ResultData resultData = null;

        // Betriebsart aus Telegramm -----------------------------------------------------------------------
        int _BetriebsArt_ = deBlock.getDeData().readUnsignedByteAsInt();

        data.getUnscaledValue("BetriebsArt").set(_BetriebsArt_);

        // Ergebnisdatensatz erzeugen...
        resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);

        return resultData;
    }

    /**
     * FG 4 : Typ: 17 Id: 2 Name: Betriebsart (Antwort). TLS-Datenbeschreibung:<br>
     * <pre>
     * Position         Bezeichnung                                    Erluterung
     * Byte 1           Lnge DE-Block                                 Lnge des
     * folgenden DE-Blocks [ 3 ]
     * Byte 2           Daten-Endgerte-Kanal (DE)                     [193..222,
     * 223, (255)1]
     * Byte 3           Typ der DE-Daten                               [ 17 ]
     * Byte 4           Betriebsart                                    siehe unten
     * </pre>
     * 1) siehe Seite A6.1.1 - 3, Kapitel 1.4<br> Inhalt Byte: Betriebsart:<br>
     * <pre>
     *        0: frei
     *        1: Normalbetrieb
     *        2: Blindbetrieb
     *        3: Handbetrieb
     *        4: Autarker Betrieb
     *        5: Testbetrieb
     *        6: Notbetrieb
     *   7..255: noch nicht definiert
     * </pre>
     *
     * @param result Der vom Datenverteiler empgangene Datensatz, der in einen De-Block konvertiert wird.
     *
     * @return De-Block mit den konvertierten Daten.
     */
    public DeBlock Fg004Typ017Id002AtgTlsWzgBetriebsArtAspParameterSoll(ResultData result) {
        DeBlock deBlock = null;

        // Datenblock beseht aus folgenden Inhalten:
        int _BetriebsArt_;

        // Umsetzung
        Data data = result.getData();  // Datensatz holen

        _BetriebsArt_ = data.getUnscaledValue("BetriebsArt").intValue();

        // DeBlock als TlsData aufbauen...
        TlsData deApplicationData = new TlsData(1);

        deApplicationData.writeIntAsByte(_BetriebsArt_);

        // DeBlock erzeugen
        try {
            deBlock = new DeBlock(deApplicationData, _de.getEak().getSm().getOsi7Adress(), _de.getFg(), 17, 2, _de.getDeChannel());
            debug.finest("TLS-DE-Block: " + HexDumper.toString(deBlock.getTlsDeDataBlock()));
        }
        catch (IllegalOsi7TelegramStructureException e) {
            debug.warning("Fehler beim Erzeugen eines De-Blocks zum Versand: " + e);
        }

        return deBlock;
    }

    /**
     * FG 4 : Typ: 17 Id: 34 Name: Betriebsart (Antwort). TLS-Datenbeschreibung:<br> Standardverarbeitung wie bei Id 2,
     * aber als "nachgelieferte" Daten an den DaV bergeben.
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz fr die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung fr die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg004Typ017Id034AtgTlsWzgBetriebsArtAspParameterIst(DeBlock deBlock, Data data, DataDescription dataDescription) {

        // Standardverarbeitung wie bei Id 1, aber als "nachgelieferte" Daten an den DaV bergeben.
        ResultData resultData = Fg004Typ017Id002AtgTlsWzgBetriebsArtAspParameterIst(deBlock, data, dataDescription);

        return new ResultData(_de.getDe(), dataDescription, resultData.getDataTime(), resultData.getData(), true);
    }

    /**
     * FG 4 : Typ: 20 Id: 2 Name: Abruf Pufferinhalt (Abruf). TLS-Datenbeschreibung:<br>
     * <pre>
     * Position         Bezeichnung                                    Erluterung
     * Byte 1           Lnge DE-Block                                 [ 5 ]
     * Byte 2           Daten-Endgerte-Kanal (DE)                     [ 193..222,
     * 223,(2551) ]
     * Byte 3           Typ der DE-Daten                               [ 20 ]
     * Byte 4           Erste Folgenummer (low Byte)                   [ 0,
     * 1...65535 ]
     * Byte 5           Erste Folgenummer (high Byte)
     * Byte 6           Anzahl angeforderter Nachrichten               [ 1...255 ]
     * </pre>
     * 1) siehe Seite A6.1.1 - 3, Kapitel 1.4<br> Inhalt Byte Anzahl angeforderter Nachrichten:<br> Gibt die Anzahl der zu
     * bertragenden Nachrichten an. Dabei wird ab der durch die Folgenummer festgelegten Nachricht in Richtung der
     * aktuellen Nachricht gezhlt.<br> Die lteste der geforderten Nachrichten wird auch zuerst bertragen. Ist die
     * Folgenummer = 0, wird die entsprechende Anzahl der aktuellsten Nachrichten bertragen. Stehen zum Beispiel die
     * Telegramm 1..35 im Speicher, wobei 35 die aktuelle Nachricht ist, werden beim Abruf Folgenummer = 0, Anzahl
     * geforderter Nachrichten = 3 die Nachrichten 33 bis 35 bertragen.<br>
     *
     * @param result Der vom Datenverteiler empgangene Datensatz, der in einen De-Block konvertiert wird.
     *
     * @return De-Block mit den konvertierten Daten.
     */
    public DeBlock Fg004Typ020Id002AtgTlsWzgAbrufPufferInhaltAspTlsAbruf(ResultData result) {
        DeBlock deBlock = null;

        // Datenblock beseht aus folgenden Inhalten:
        int _StartFolgeNummer_;
        int _Anzahl1_;

        // Umsetzung
        Data data = result.getData();  // Datensatz holen

        _StartFolgeNummer_ = data.getUnscaledValue("StartFolgeNummer").intValue();
        _Anzahl1_          = data.getUnscaledValue("Anzahl1").intValue();

        // DeBlock als TlsData aufbauen...
        TlsData deApplicationData = new TlsData(3);

        deApplicationData.writeIntAsLowHighByte(_StartFolgeNummer_);
        deApplicationData.writeIntAsByte(_Anzahl1_);

        // DeBlock erzeugen
        try {
            deBlock = new DeBlock(deApplicationData, _de.getEak().getSm().getOsi7Adress(), _de.getFg(), 20, 2, _de.getDeChannel());
            debug.finest("TLS-DE-Block: " + HexDumper.toString(deBlock.getTlsDeDataBlock()));
        }
        catch (IllegalOsi7TelegramStructureException e) {
            debug.warning("Fehler beim Erzeugen eines De-Blocks zum Versand: " + e);
        }

        return deBlock;
    }

    /**
     * FG 4 : Typ: 21 Id: 3 Name: Abruf Codedefinition (Abruf). TLS-Datenbeschreibung:<br>
     * <pre>
     * Position         Bezeichnung                                    Erluterung
     * Byte 1           Lnge DE-Block                                 [ 3 ]
     * Byte 2           Daten-Endgerte-Kanal (DE)                     [ 1..254,
     * 255 ]
     * Byte 3           Typ der DE-Daten                               [ 21 ]
     * Byte 4           Stellcode                                      [ 1..254]
     * </pre>
     * Inhalt Byte: Stellcode:<br> Enthlt den Stellcode, dessen Definition angefordert wird.<br>
     *
     * @param result Der vom Datenverteiler empgangene Datensatz, der in einen De-Block konvertiert wird.
     *
     * @return De-Block mit den konvertierten Daten.
     */
    public DeBlock Fg004Typ021Id003AtgTlsWzgAbrufCodeDefinitionenAspTlsAbruf(ResultData result) {
        DeBlock deBlock = null;

        // Datenblock beseht aus folgenden Inhalten:
        int _StellCode_;

        // Umsetzung
        Data data = result.getData();  // Datensatz holen

        _StellCode_ = data.getUnscaledValue("StellCode").intValue();

        // DeBlock als TlsData aufbauen...
        TlsData deApplicationData = new TlsData(1);

        deApplicationData.writeIntAsByte(_StellCode_);

        // DeBlock erzeugen
        try {
            deBlock = new DeBlock(deApplicationData, _de.getEak().getSm().getOsi7Adress(), _de.getFg(), 21, 3, _de.getDeChannel());
            debug.finest("TLS-DE-Block: " + HexDumper.toString(deBlock.getTlsDeDataBlock()));
        }
        catch (IllegalOsi7TelegramStructureException e) {
            debug.warning("Fehler beim Erzeugen eines De-Blocks zum Versand: " + e);
        }

        return deBlock;
    }

    // Gieseler, BitCtrl

    /**
     * FG 4: Typ: 22 Id: 3 Name: Abruf Bilddefinition/Grafiktextkonserve (Abruf). TLS-Datenbeschreibung:<br>
     * <pre>
     * Position         Bezeichnung                                    Erluterung
     * Byte 1           L&auml;nge DE-Block                                 [ 3 ]
     * Byte 2           Daten-Endger&auml;te-Kanal (DE)                     [ 1..254, 255 ]
     * Byte 3           Typ der DE-Daten                               [ 22 ]
     * Byte 4           Stellcode/Grafiktextkonservennummer            [ 1..254]
     * </pre>
     * Inhalt Byte: Stellcode/Grafiktextkonservennummer<br> Enth&auml;lt den Stellcode dessen Bilddefinitionen
     * abgefragt werden soll (Anzeigeprinzip b, Upload-f&auml;hige LED-PixelrasterAnzeigen) bzw. die
     * Grafiktextkonservennummer dessen Grafiktextkonserve abgefragt werden soll
     * (Anzeigeprinzip d, Wechseltextanzeigen mit Grafiktextkonserven).<br>
     *
     * @param result Der vom Datenverteiler empfangene Datensatz, der in einen De-Block konvertiert wird.
     *
     * @return De-Block mit den konvertierten Daten.
     */
    public DeBlock Fg004Typ022Id003AtgTlsWzgAbrufBildDefinitionAspTlsAbruf(ResultData result) {
        DeBlock deBlock = null;

        // Datenblock beseht aus folgenden Inhalten:
        int _StellCode_;

        // Umsetzung
        Data data = result.getData();  // Datensatz holen

        _StellCode_ = data.getUnscaledValue("StellCodeGrafikTextKonservenNummer").intValue();

        // DeBlock als TlsData aufbauen...
        TlsData deApplicationData = new TlsData(1);

        deApplicationData.writeIntAsByte(_StellCode_);

        // DeBlock erzeugen
        try {
            deBlock = new DeBlock(deApplicationData, _de.getEak().getSm().getOsi7Adress(), _de.getFg(), 22, 3, _de.getDeChannel());
            debug.finest("TLS-DE-Block: " + HexDumper.toString(deBlock.getTlsDeDataBlock()));
        }
        catch (IllegalOsi7TelegramStructureException e) {
            debug.warning("Fehler beim Erzeugen eines De-Blocks zum Versand: " + e);
        }

        return deBlock;
    }

    // Ende Gieseler, BitCtrl

    /**
     * FG 4 : Typ: 29 Id: 2/34 Name: Kanal-Steuerung (Antwort/Antwort Puffer). Ergnzt globale Behandlung Kanal-Steuerung
     * in {@link #Fg004Typ029Id034AtgTlsGloKanalSteuerungAspParameterIst}. Standardverarbeitung wie bei Id 2, aber als
     * "nachgelieferte" Daten an den DaV bergeben.
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz fr die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung fr die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg004Typ029Id034AtgTlsGloKanalSteuerungAspParameterIst(DeBlock deBlock, Data data, DataDescription dataDescription) {

        // Standardverarbeitung wie bei Id 2, aber als "nachgelieferte" Daten an den DaV bergeben.
        ResultData resultData = super.Fg255Typ029Id002AtgTlsGloKanalSteuerungAspParameterIst(deBlock, data, dataDescription);

        return new ResultData(_de.getDe(), dataDescription, resultData.getDataTime(), resultData.getData(), true);
    }

    /**
     * FG 4 : Typ: 32 Id: 3 Name: WVZ-Grundeinstellung (Abruf / Antwort). TLS-Datenbeschreibung:<br>
     * <pre>
     * Position         Bezeichnung                                    Erluterung
     * Byte 1           Lnge DE-Block                                 Lnge des
     * folgenden DE-Blocks [ 4 ]
     * Byte 2           Daten-Endgerte-Kanal (DE)                     [ 1..254,
     * 255 ]
     * Byte 3           Typ der DE-Daten                               [ 32 ]
     * Byte 4           Stellzustand: WVZ-Code
     * anzuzeigendes WVZ
     * Byte 5           Funktionsbyte                                   Ein / Aus 
     * Blinken
     * </pre>
     * Inhalt Byte: Stellzustand:<br> Codierung von Verkehrszeichen: (WVZ-Code): Siehe Anhang 7, Kapitel 1.<br> Wird der
     * WVZ-Code 255 ausgewhlt, so bleibt der zuletzt geschaltete Stellzustand im Autarkbetrieb bestehen.<br> <br> Inhalt
     * Funktionsbyte:<br>
     * <pre>
     *    7        6        5        4        3        2        1        0
     *               Blinkzeit                            Funktion
     * Bit 0..3:     Funktion:
     *               0 = ausschalten
     *               1 = einschalten
     *               2 = blinken (Die Einstellung "WVZ blinkend" ist eine optionale
     * Eigenschaft.)
     *               3-15 reserviert
     * <p/>
     * Bit 4..7:     Blinkzeit in 200 ms (1..3 s)
     *               Die Blinkzeit ist die gesamte Periodendauer, d.h. die Summe
     * aus Ein- und Auszeit. Das Verhltnis Ein-
     *               /Auszeit wird nach Vorgabe des Anlagenbetreibers am WZG
     * eingestellt.
     *               Die Koordination der Blinkzeiten verschiedener WVZ ist Aufgabe
     * der Zentrale.
     * </pre>
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz fr die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung fr die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg004Typ032Id003AtgTlsWzgWvzGrundEinstellungAspParameterIst(DeBlock deBlock, Data data, DataDescription dataDescription) {
        ResultData resultData = null;

        // Betriebsart aus Telegramm -----------------------------------------------------------------------
        int _Stellzustand_ = deBlock.getDeData().readUnsignedByteAsInt();

        _validWvzCode = _Stellzustand_;

        int _Funktion_ = deBlock.getDeData().readUnsignedByteAsInt();

        // Wenn Funktion = ausgeschaltet, hat WVZ-Code keine Bedeutung. Wird deshalb fr
        // Protokollausgaben auf -1 = "aus" gesetzt.
        if ((_Funktion_ & 0x0f) == 0) {
            _Stellzustand_ = -1;
        }

        data.getItem("WVZAnzeige").getUnscaledValue("Code").set(_Stellzustand_);

        if ((_Funktion_ & 0x02) == 0) {
            data.getItem("WVZAnzeige").getItem("Blinken").getUnscaledValue("Status").setText("aus");
            data.getItem("WVZAnzeige").getItem("Blinken").getUnscaledValue("Dauer").setText("aus");
        } else {
            data.getItem("WVZAnzeige").getItem("Blinken").getUnscaledValue("Status").setText("ein");

            int _BlinkDauer_ = _Funktion_ >>> 4;

            if ((_BlinkDauer_ < 5) || (_BlinkDauer_ > 15)) {
                debug.warning("Blinkdauer ungltig (5-15) : " + (_Funktion_ >>> 4) + ". Blinkdauerwert wurde auf 5 gesetzt!");
                _BlinkDauer_ = 5;
            }

            data.getItem("WVZAnzeige").getItem("Blinken").getUnscaledValue("Dauer").set(_BlinkDauer_);
        }

        data.getItem("WVZAnzeige").getItem("Status").getUnscaledValue("Fehler").setText("ok");
        data.getItem("WVZAnzeige").getItem("Status").getUnscaledValue("Programm").setText("abgeschlossen");

        // Ergebnisdatensatz erzeugen...
        resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);

        return resultData;
    }

    /**
     * FG 4 : Typ: 32 Id: 3 Name: WVZ-Grundeinstellung (Abruf / Antwort). TLS-Datenbeschreibung:<br>
     * <pre>
     * Position         Bezeichnung                                    Erluterung
     * Byte 1           Lnge DE-Block                                 Lnge des
     * folgenden DE-Blocks [ 4 ]
     * Byte 2           Daten-Endgerte-Kanal (DE)                     [ 1..254,
     * 255 ]
     * Byte 3           Typ der DE-Daten                               [ 32 ]
     * Byte 4           Stellzustand: WVZ-Code
     * anzuzeigendes WVZ
     * Byte 5           Funktionsbyte                                   Ein / Aus 
     * Blinken
     * </pre>
     * Inhalt Byte: Stellzustand:<br> Codierung von Verkehrszeichen: (WVZ-Code): Siehe Anhang 7, Kapitel 1.<br> Wird der
     * WVZ-Code 255 ausgewhlt, so bleibt der zuletzt geschaltete Stellzustand im Autarkbetrieb bestehen.<br> <br> Inhalt
     * Funktionsbyte:<br>
     * <pre>
     *    7        6        5        4        3        2        1        0
     *               Blinkzeit                            Funktion
     * Bit 0..3:     Funktion:
     *               0 = ausschalten
     *               1 = einschalten
     *               2 = blinken (Die Einstellung "WVZ blinkend" ist eine optionale
     * Eigenschaft.)
     *               3-15 reserviert
     * <p/>
     * Bit 4..7:     Blinkzeit in 200 ms (1..3 s)
     *               Die Blinkzeit ist die gesamte Periodendauer, d.h. die Summe
     * aus Ein- und Auszeit. Das Verhltnis Ein-
     *               /Auszeit wird nach Vorgabe des Anlagenbetreibers am WZG
     * eingestellt.
     *               Die Koordination der Blinkzeiten verschiedener WVZ ist Aufgabe
     * der Zentrale.
     * </pre>
     *
     * @param result Der vom Datenverteiler empgangene Datensatz, der in einen De-Block konvertiert wird.
     *
     * @return De-Block mit den konvertierten Daten.
     */
    public DeBlock Fg004Typ032Id003AtgTlsWzgWvzGrundEinstellungAspParameterSoll(ResultData result) {
        DeBlock deBlock = null;

        // Datenblock beseht aus folgenden Inhalten:
        int _Stellzustand_;
        int _Funktion_ = 0;

        // Umsetzung
        Data data = result.getData();  // Datensatz holen

        _Stellzustand_ = data.getItem("WVZAnzeige").getUnscaledValue("Code").intValue();

        // Wenn _Stellzustand_ = -1, muss Funktionsbyte auf "aus" gesetzt werden ( = 0).
        if (_Stellzustand_ == -1) {
            _Funktion_     = 0;              // ausschalten
            _Stellzustand_ = _validWvzCode;  // letzter als gltig angenommener Code!
        } else {
            if (data.getItem("WVZAnzeige").getItem("Blinken").getTextValue("Status").getText().equals("aus")) {
                _Funktion_ = 1;              // einschalten ohne Blinken
            } else {
                _Funktion_ = 2;              // einschalten mit Blinken

                // Blinkezeit auswerten.
                if (data.getItem("WVZAnzeige").getItem("Blinken").getTextValue("Dauer").getText().equals("aus")) {
                    _Funktion_ &= 0x0f;      // Bit 4-7 = 0 setzten
                } else {
                    _Funktion_ |= data.getItem("WVZAnzeige").getItem("Blinken").getUnscaledValue("Dauer").intValue() << 4;
                }
            }
        }

        // DeBlock als TlsData aufbauen...
        TlsData deApplicationData = new TlsData(2);

        deApplicationData.writeIntAsByte(_Stellzustand_);
        deApplicationData.writeIntAsByte(_Funktion_);

        // DeBlock erzeugen
        try {
            deBlock = new DeBlock(deApplicationData, _de.getEak().getSm().getOsi7Adress(), _de.getFg(), 32, 3, _de.getDeChannel());
            debug.finest("TLS-DE-Block: " + HexDumper.toString(deBlock.getTlsDeDataBlock()));
        }
        catch (IllegalOsi7TelegramStructureException e) {
            debug.warning("Fehler beim Erzeugen eines De-Blocks zum Versand: " + e);
        }

        return deBlock;
    }

    /**
     * Entspricht {@link Fg004Default#Fg004Typ032Id003AtgTlsWzgWvzGrundEinstellungAspParameterIst(de.bsvrz.kex.tls.osi7.DeBlock,
     * de.bsvrz.dav.daf.main.Data, de.bsvrz.dav.daf.main.DataDescription)}, jedoch werden die Daten als nachgeliefert gekennzeichnet.
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz fr die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung fr die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg004Typ032Id035AtgTlsWzgWvzGrundEinstellungAspParameterIst(DeBlock deBlock, Data data, DataDescription dataDescription) {

        // Generierte Methode: Standardverarbeitung wie bei Id 3, aber als "nachgelieferte" Daten an den DaV bergeben.
        ResultData resultData = Fg004Typ032Id003AtgTlsWzgWvzGrundEinstellungAspParameterIst(deBlock, data, dataDescription);

        return new ResultData(_de.getDe(), dataDescription, resultData.getDataTime(), resultData.getData(), true);
    }

    /**
     * FG 4 : Typ: 33 Id: 3 und Id: 31 Name: Grundeinstellung (Abruf / Antwort). TLS-Datenbeschreibung:<br>
     * <pre>
     * Position         Bezeichnung                              Erluterung
     * Byte 1           Lnge DE-Block                           Lnge des
     * folgenden DE-Blocks [5 |6+f*n]
     * Byte 2           Daten-Endgerte-Kanal (DE)               [ 1..254 ]
     * Byte 3           Typ der DE-Daten                         [ 33 ]
     * Byte 4           Anzeigeprinzip                           siehe unten
     * Byte 5           Stellcode der Grundeinstellung           [0, 1..254,
     * 255]siehe unten
     * Byte 6           Funktionsbyte                            siehe unten, vgl
     * Kap. 6.2.18
     * Byte 7           n : Anzahl Komponenten 1             [213 | 106] siehe
     * unten
     * f Bytes...       Grundeinstellung der 1.Komponente        siehe unten
     * f Bytes          Grundeinstellung der n.Komponente
     * </pre>
     * Inhalt Byte : Anzeigeprinzip<br> Das Byte gibt das Anzeigeprinzip des adressierten/sendenden DEs an und ermglicht
     * damit die genaue Interpretation des folgenden Telegramminhalts. Es ist folgendermaen codiert:<br>
     * <pre>
     * Anzeigeprinzip      Wert
     *              a      0
     *              b      1
     *              c      2
     *              d      3
     *              e      4
     *        Cluster      8
     * </pre>
     * Erhlt ein DE in Abrufrichtung eine Nachricht mit einem Wert, der nicht seinem Anzeigeprinzip entspricht, so wird
     * die Nachricht mit Code 21 "falsches Anzeigeprinzip" negativ quittiert.<br> Inhalt Byte: Stellcode der
     * Grundeinstellung:<br> Der Inhalt des Bytes ist abhngig davon, ob das adressierte DE ein Clusterkanal ist bzw. vom
     * Anzeigeprinzip des DEs. Diese Information ist in Byte 4 "Anzeigeprinzip" enthalten.<br>
     * <pre>
     * Clusterkanle :   Nummer des Programms, das im Autarkbetrieb eingenommen
     * werden soll. Ist der Wert 0,
     *                   so nehmen alle DEs dann den Zustand ein, der ihrer
     * jeweiligen individuellen
     *                   Grundeinstellung entspricht. Ist der Wert 255, so behalten
     * alle DEs den zuletzt eingestellten
     *                   Zustand bei. Ist als Wert eine Programmnummer zwischen 1
     * und 254 angegeben, so werden
     *                   alle DEs im Autarkbetrieb entsprechend der Definition
     * dieses Programms eingestellt. Die
     *                   Codedefinition wird fr alle DEs bernommen und von diesen
     * spontan gemeldet.
     *                   Ausnahmen hiervon bilden lediglich DEs, fr welche die
     * entsprechende
     *                   Programmdefinition den Wert 255 dont care enthlt.
     * Diese DEs nehmen im
     *                   Autarkbetrieb den Stellcode ihrer individuellen
     * Grundeinstellung an.
     * Anzeigeprinzip a und b :  WVZ-Code entsprechend Anhang 7, Kapitel 1. Gibt
     * den WVZ-Code an, den das
     *                           DE im Autarkbetrieb darstellen soll, falls die
     * Grundeinstellungs-Programmnummer des
     *                           Clusterkanals den Wert 0 enthlt oder die
     * Definition des Grundeinstellungsprogramms fr
     *                           dieses DE den Wert 255 "dont care" angibt. In
     * allen anderen Fllen ist die individuelle
     *                           Grundeinstellung nicht relevant. Ist der Wert 255,
     * so behlt das WZG seinen letzten
     *                           Stellzustand bei. Der Wert 0 ist fr DEs dieser
     * Anzeigeprinzipien nicht zulssig.
     * Anzeigeprinzip c und d :  Stellcode des vordefinierten Textes, den das DE im
     * Autarkbetrieb darstellen soll,
     *                           falls die Grundeinstellungs-Programmnummer des
     * Clusterkanals den Wert 0 enthlt oder
     *                           die Definition des Grundeinstellungsprogramms fr
     * dieses DE den Wert 255 "dont care"
     *                           angibt. In allen anderen Fllen ist die
     * individuelle Grundeinstellung nicht relevant. Ist
     * der
     *                           Wert 255, so behlt das WZG seinen zuletzt
     * dargestellten Text bei. Ist der Wert hingegen
     *                           0, so wird das Telegramm in der Langversion
     * bertragen und enthlt den darzustellenden
     *                           Text als ASCII-String.
     * Anzeigeprinzip e :        Stellcode der vordefinierten Gruppendefinition,
     * die das DE im Autarkbetrieb
     *                           darstellen soll, falls die
     * Grundeinstellungs-Programmnummer des Clusterkanals den Wert 0
     *                           enthlt oder die Definition des
     * Grundeinstellungsprogramms fr dieses DE den Wert 255
     *                           "dont care" angibt. In allen anderen Fllen ist
     * die individuelle Grundeinstellung nicht
     *                           relevant. Ist der Wert 255, so behlt die Gruppe
     * ihren zuletzt dargestellten Inhalt bei.
     * Ist der
     *                           Wert hingegen 0, so wird das Telegramm in der
     * Langversion bertragen und enthlt fr
     *                           jede Komponente den entsprechenden Stellzustand.
     * </pre>
     * Zulssige Werte :
     * <pre>
     *                            Cluster                a,b               c,d,e
     * Steuerungsprinzip 1        0, 255                 --                0, 255
     * Steuerungsprinzip 2        1-254, 255             Anhang 7, Kap. 1
     * 1-254,255
     * Steuerungsprinzip 3        0,1-254,255            --
     * 0,1-254,255
     * </pre>
     * Inhalt Funktionsbyte :<br> Der Inhalt des Funktionsbytes entspricht der verallgemeinerten Definition des
     * Funktionsbytes in Kapitel 6.2.18.<br> <br> Inhalt Grundeinstellung einer Komponenten :<br> Dieser Teil des
     * Telegramms wird genauso wie die Anzahl der Komponenten im Byte 7 nur in der Langversion des Telegramms bertragen,
     * d.h. insbesondere, dass der Stellcode in Byte 5 den Wert 0 haben muss. Wird die Langversion mit einem Stellcode
     * ungleich 0 empfangen, so wird mit Code 46 Langversion nicht zulssig negativ quittiert. Inhalt und Lnge der
     * wiederholten Komponentenblcke sind abhngig vom Anzeigeprinzip des adressierten DE:<br>
     * <pre>
     * Clusterkanle : Die Langversion ist fr Clusterkanle nicht zulssig. Die
     * individuellen Grundeinstellungen
     *                 werden an die einzelnen DEs direkt adressiert. Negative
     * Quittung Code 46.
     * Anzeigeprinzip a und b : Die Langversion ist inhaltsgem fr diese Anzeigen
     * nicht zulssig. Negative
     *                 Quittung Code 46.
     * Anzeigeprinzip c und d : Die Lnge eines Komponentenblocks (f) ist 1, so
     * dass die Langversion eine DEBlock-
     *                 Lnge von 6 + n hat. Die Anzahl Komponenten (Textzeichen)
     * ist deswegen auf
     *                 maximal 213 beschrnkt. Jeder Komponentenblock ist
     * folgendermaen aufgebaut:
     * <p/>
     *                 Byte 1           Textzeichen in ASCII-Code        siehe
     * Anhang 7, Kapitel 3
     * <p/>
     * Anzeigeprinzip e : Die Lnge eines Komponentenblocks (f) ist 2, so dass die
     * Langversion eine DE-Block-
     *                 Lnge von 6 + 2*n hat. Die Anzahl Komponenten ist deswegen
     * auf maximal 106
     *                 beschrnkt. Jeder Komponentenblock ist folgendermaen
     * aufgebaut:
     * <p/>
     *                 Byte 1           WVZ-Code                         siehe
     * Anhang 7, Kapitel 1
     *                 Byte 2           Funktionsbyte
     * <p/>
     *                 Der Inhalt des Funktionsbytes entspricht der
     * verallgemeinerten Definition des
     *                 Funktionsbytes in Kapitel 6.2.18.
     * </pre>
     * Die folgende Tabelle stellt bersichtlich alle Varianten des Telegramms zusammen, erlutert deren Funktion und gibt
     * an, fr welche Kombination aus Anzeige- und Steuerungsprinzip die jeweilige Verwendung zulssig ist. Dabei sind
     * Felder mit --  hinterlegt, wenn sie im betreffenden Kontext nicht relevant sind. Mit - hinterlegte Felder in der
     * Spalte "Steuerungsprinzip" bedeuten, dass die jeweiligen Stellbefehle negativ quittiert werden, die Nummer in
     * Klammern gibt den zu verwendenden Fehlercode an. Felder mit einem (zul) kennzeichnen die Zulssigkeit des
     * entsprechenden Telegramms fr das jeweilige DE. Eine hinter zul angefgte Nummer gekennzeichnetes Feld verweist auf
     * einen Telegrammtyp aus frheren Versionen der TLS, die bei Altlagen im jeweiligen Kontext noch zulssig sind.<br>
     * <pre>
     * Steuerungsprinzip
     * ID   Typ   Lnge  Stellcode  n      Funktion                       Anzeige
     *    f     Steuerungsprinzip
     *                                                                    prinzip
     * <p/>
     *          1        2       3
     * 3R   33    5      0          --     Grundeinstellung pro DE        a,b
     *    --    --       -(43)   --
     *                              --     individuell festlegen          c-e
     *    --    -(44)    -(43)   -(44)
     *                              --                                    Cl
     *    --    zul      -(43)   zul
     *                   1-254      --     Grundeinstellung festlegen     a,b
     *    --    --       zul 32  --
     *                              --     mit Codenummer                 c,d
     *    --    -(45)    zul     zul
     *                              --                                    e,Cl
     *    --    -(45)    zul 42  zul 42
     *                   255        --     Grundeinstellung "son't care"  alle
     *    --    zul      zul 42  zul 42
     *            6+n*f  0          1 --   Grundeinstellung festlegen     a,b
     *    --    --       -(43)   --
     *                              1 213  mit freier Definition          c,d
     *    1     zul      -(43)   zul
     *                              1 106                                 e
     *    2     zul      -(43)   zul
     *                              1 --                                  Cl
     *    --    --       -(43)   --
     * 3R         5      0          --     Grundeinstellung pro DE        a,b
     *    --    --       --      --
     *                              --     individuell festlegen          Cl
     *    --    zul      --      zul
     *                   1-254      --     Grundeinstellung melden        a,b
     *    --    --       zul 32  --
     *                              --     mit Codenummer                 c,d
     *    --    --       zul     zul
     *                              --                                    e,Cl
     *    --    --       zul 42  zul 42
     *                   255        --     Grundeinstellung "son't care"  alle
     *    --    zul      zul 42  zul 42
     *            6+n*f  0          1 --   Grundeinstellung melden        a,b
     *    --    --       --   --
     *                              1 213  mit freier Definition          c,d
     *    1     zul      --      zul
     *                              1 106                                 e
     *    2     zul      --      zul
     *                              1 --                                  Cl
     *    --    --       --      --
     * 19R        2      --         --     Abruf Grundeinstellung         alle
     *    --    zul      zul     zul
     * </pre>
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz fr die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung fr die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg004Typ033Id003AtgTlsWzgGrundEinstellungAspParameterIst(DeBlock deBlock, Data data, DataDescription dataDescription) {
        ResultData resultData = null;

        // AnzeigePrinzip aus Telegramm --------------------------------------------------------------------------------
        int _AnzeigePrinzip_ = deBlock.getDeData().readUnsignedByteAsInt();

        data.getUnscaledValue("AnzeigePrinzip").set(_AnzeigePrinzip_);

        // Stellcode und Funktion aus Telegramm ------------------------------------------------------------------------
        int _Stellcode_ = deBlock.getDeData().readUnsignedByteAsInt();

        _validWvzCode = _Stellcode_;

        int _Funktion_ = deBlock.getDeData().readUnsignedByteAsInt();

        // Alle dynamischen Arrays auf Lnge 0, werden abhngig von der Semantik nachfolgende auf richtige Gre gesetzt.
        data.getArray("WVZAnzeige").setLength(0);
        data.getArray("StellCode").setLength(0);
        data.getArray("TextDefinition").setLength(0);
        data.getArray("GruppenDefinition").setLength(0);

        // Abhngig vom AnzeigePrinzip Stellcode und Funktion zuordnen
        switch (_AnzeigePrinzip_) {
        case 0 :
        case 1 : {
            data.getArray("WVZAnzeige").setLength(1);

            // Wenn Funktion = ausgeschaltet, hat WVZ-Code keine Bedeutung. Wird deshalb fr
            // Protokollausgaben auf -1 = "aus" gesetzt.
            if ((_Funktion_ & 0x0f) == 0) {
                _Stellcode_ = -1;
            }

            data.getArray("WVZAnzeige").getItem(0).getUnscaledValue("Code").set(_Stellcode_);

            if ((_Funktion_ & 0x02) == 0) {
                data.getArray("WVZAnzeige").getItem(0).getItem("Blinken").getUnscaledValue("Status").setText("aus");
                data.getArray("WVZAnzeige").getItem(0).getItem("Blinken").getUnscaledValue("Dauer").setText("aus");
            } else {
                data.getArray("WVZAnzeige").getItem(0).getItem("Blinken").getUnscaledValue("Status").setText("ein");

                int _BlinkDauer_ = _Funktion_ >>> 4;

                if ((_BlinkDauer_ < 5) || (_BlinkDauer_ > 15)) {
                    debug.warning("Blinkdauer ungltig (5-15) : " + (_Funktion_ >>> 4) + ". Blinkdauerwert wurde auf 5 gesetzt!");
                    _BlinkDauer_ = 5;
                }

                data.getArray("WVZAnzeige").getItem(0).getItem("Blinken").getUnscaledValue("Dauer").set(_BlinkDauer_);
            }

            if ((_Funktion_ & 0x04) == 0) {
                data.getArray("WVZAnzeige").getItem(0).getItem("Status").getUnscaledValue("Fehler").setText("ok");
            } else {
                data.getArray("WVZAnzeige").getItem(0).getItem("Status").getUnscaledValue("Fehler").setText("Fehler");
            }

            if ((_Funktion_ & 0x08) == 0) {
                data.getArray("WVZAnzeige").getItem(0).getItem("Status").getUnscaledValue("Programm").setText("abgeschlossen");
            } else {
                data.getArray("WVZAnzeige").getItem(0).getItem("Status").getUnscaledValue("Programm").setText("in Arbeit");
            }
        }

        break;

        case 2 :
        case 3 :
        case 4 :
        case 8 : {
            data.getArray("StellCode").setLength(1);

            // Wenn Funktion = ausgeschaltet, hat Stell-Code keine Bedeutung. Wird deshalb fr
            // Protokollausgaben auf -1 = "aus" gesetzt.
            if ((_Funktion_ & 0x0f) == 0) {
                _Stellcode_ = -1;
            }

            data.getArray("StellCode").getItem(0).getUnscaledValue("Code").set(_Stellcode_);

            if ((_Funktion_ & 0x02) == 0) {
                data.getArray("StellCode").getItem(0).getItem("Blinken").getUnscaledValue("Status").setText("aus");
                data.getArray("StellCode").getItem(0).getItem("Blinken").getUnscaledValue("Dauer").setText("aus");
            } else {
                data.getArray("StellCode").getItem(0).getItem("Blinken").getUnscaledValue("Status").setText("ein");

                int _BlinkDauer_ = _Funktion_ >>> 4;

                if ((_BlinkDauer_ < 5) || (_BlinkDauer_ > 15)) {
                    debug.warning("Blinkdauer ungltig (5-15) : " + (_Funktion_ >>> 4) + ". Blinkdauerwert wurde auf 5 gesetzt!");
                    _BlinkDauer_ = 5;
                }

                data.getArray("StellCode").getItem(0).getItem("Blinken").getUnscaledValue("Dauer").set(_BlinkDauer_);
            }

            if ((_Funktion_ & 0x04) == 0) {
                data.getArray("StellCode").getItem(0).getItem("Status").getUnscaledValue("Fehler").setText("ok");
            } else {
                data.getArray("StellCode").getItem(0).getItem("Status").getUnscaledValue("Fehler").setText("Fehler");
            }

            if ((_Funktion_ & 0x08) == 0) {
                data.getArray("StellCode").getItem(0).getItem("Status").getUnscaledValue("Programm").setText("abgeschlossen");
            } else {
                data.getArray("StellCode").getItem(0).getItem("Status").getUnscaledValue("Programm").setText("in Arbeit");
            }
        }

        break;

        default : {
            debug.warning("Anzeigeprinzip ungltig (0-4,8) : " + _AnzeigePrinzip_ + Debug.NEWLINE + "De-Block (Osi7Adresse-DeNummer-FG-Typ-Id): " + deBlock.getLongKey() + " verworfen:" + Debug.NEWLINE + deBlock.toString());

            return resultData;
        }
        }

        // Ermitteln, ob Langversion vorliegt...

        if (deBlock.getDeData().length() > 3) {
            int count = deBlock.getDeData().readUnsignedByteAsInt();

            // Abhngig vom AnzeigePrinzip Stellcode und Funktion zuordnen
            switch (_AnzeigePrinzip_) {
            case 2 :
            case 3 : {
                data.getArray("TextDefinition").setLength(count);

                char text;

                for (int i = 0; i < count; ++i) {
                    text = (char) deBlock.getDeData().readUnsignedByteAsInt();
                    data.getArray("TextDefinition").getItem(i).getTextValue("Text").setText("" + text);
                }
            }

            break;

            case 4 : {
                data.getArray("GruppenDefinition").setLength(count / 2);

                for (int i = 0; i < count / 2; ++i) {
                    _Stellcode_   = deBlock.getDeData().readUnsignedByteAsInt();
                    _validWvzCode = _Stellcode_;
                    _Funktion_    = deBlock.getDeData().readUnsignedByteAsInt();

                    // Wenn Funktion = ausgeschaltet, hat Stell-Code keine Bedeutung. Wird deshalb fr
                    // Protokollausgaben auf -1 = "aus" gesetzt.
                    if ((_Funktion_ & 0x0f) == 0) {
                        _Stellcode_ = -1;
                    }

                    data.getArray("GruppenDefinition").getItem(i).getUnscaledValue("Code").set(_Stellcode_);

                    if ((_Funktion_ & 0x02) == 0) {
                        data.getArray("GruppenDefinition").getItem(i).getItem("Blinken").getUnscaledValue("Status").setText("aus");
                        data.getArray("GruppenDefinition").getItem(i).getItem("Blinken").getUnscaledValue("Dauer").setText("aus");
                    } else {
                        data.getArray("GruppenDefinition").getItem(i).getItem("Blinken").getUnscaledValue("Status").setText("ein");

                        int _BlinkDauer_ = _Funktion_ >>> 4;

                        if ((_BlinkDauer_ < 5) || (_BlinkDauer_ > 15)) {
                            debug.warning("Blinkdauer ungltig (5-15) : " + (_Funktion_ >>> 4) + ". Blinkdauerwert wurde auf 5 gesetzt!");
                            _BlinkDauer_ = 5;
                        }

                        data.getArray("GruppenDefinition").getItem(i).getItem("Blinken").getUnscaledValue("Dauer").set(_BlinkDauer_);
                    }

                    if ((_Funktion_ & 0x04) == 0) {
                        data.getArray("GruppenDefinition").getItem(i).getItem("Status").getUnscaledValue("Fehler").setText("ok");
                    } else {
                        data.getArray("GruppenDefinition").getItem(i).getItem("Status").getUnscaledValue("Fehler").setText("Fehler");
                    }

                    if ((_Funktion_ & 0x08) == 0) {
                        data.getArray("GruppenDefinition").getItem(i).getItem("Status").getUnscaledValue("Programm").setText("abgeschlossen");
                    } else {
                        data.getArray("GruppenDefinition").getItem(i).getItem("Status").getUnscaledValue("Programm").setText("in Arbeit");
                    }
                }
            }

            break;

            default : {
                debug.warning("Anzeigeprinzip ungltig (0-4,8) : " + _AnzeigePrinzip_ + Debug.NEWLINE + "De-Block (Osi7Adresse-DeNummer-FG-Typ-Id): " + deBlock.getLongKey() + " verworfen:" + Debug.NEWLINE + deBlock.toString());

                return resultData;
            }
            }
        }

        resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);

        return resultData;
    }

    /**
     * FG 4 : Typ: 33 Id: 3 und Id: 31 Name: Grundeinstellung (Abruf / Antwort). TLS-Datenbeschreibung:<br>
     * <pre>
     * Position         Bezeichnung                              Erluterung
     * Byte 1           Lnge DE-Block                           Lnge des
     * folgenden DE-Blocks [5 |6+f*n]
     * Byte 2           Daten-Endgerte-Kanal (DE)               [ 1..254 ]
     * Byte 3           Typ der DE-Daten                         [ 33 ]
     * Byte 4           Anzeigeprinzip                           siehe unten
     * Byte 5           Stellcode der Grundeinstellung           [0, 1..254,
     * 255]siehe unten
     * Byte 6           Funktionsbyte                            siehe unten, vgl
     * Kap. 6.2.18
     * Byte 7           n : Anzahl Komponenten 1             [213 | 106] siehe
     * unten
     * f Bytes...       Grundeinstellung der 1.Komponente        siehe unten
     * f Bytes          Grundeinstellung der n.Komponente
     * </pre>
     * Inhalt Byte : Anzeigeprinzip<br> Das Byte gibt das Anzeigeprinzip des adressierten/sendenden DEs an und ermglicht
     * damit die genaue Interpretation des folgenden Telegramminhalts. Es ist folgendermaen codiert:<br>
     * <pre>
     * Anzeigeprinzip      Wert
     *              a      0
     *              b      1
     *              c      2
     *              d      3
     *              e      4
     *        Cluster      8
     * </pre>
     * Erhlt ein DE in Abrufrichtung eine Nachricht mit einem Wert, der nicht seinem Anzeigeprinzip entspricht, so wird
     * die Nachricht mit Code 21 "falsches Anzeigeprinzip" negativ quittiert.<br> Inhalt Byte: Stellcode der
     * Grundeinstellung:<br> Der Inhalt des Bytes ist abhngig davon, ob das adressierte DE ein Clusterkanal ist bzw. vom
     * Anzeigeprinzip des DEs. Diese Information ist in Byte 4 "Anzeigeprinzip" enthalten.<br>
     * <pre>
     * Clusterkanle :   Nummer des Programms, das im Autarkbetrieb eingenommen
     * werden soll. Ist der Wert 0,
     *                   so nehmen alle DEs dann den Zustand ein, der ihrer
     * jeweiligen individuellen
     *                   Grundeinstellung entspricht. Ist der Wert 255, so behalten
     * alle DEs den zuletzt eingestellten
     *                   Zustand bei. Ist als Wert eine Programmnummer zwischen 1
     * und 254 angegeben, so werden
     *                   alle DEs im Autarkbetrieb entsprechend der Definition
     * dieses Programms eingestellt. Die
     *                   Codedefinition wird fr alle DEs bernommen und von diesen
     * spontan gemeldet.
     *                   Ausnahmen hiervon bilden lediglich DEs, fr welche die
     * entsprechende
     *                   Programmdefinition den Wert 255 dont care enthlt.
     * Diese DEs nehmen im
     *                   Autarkbetrieb den Stellcode ihrer individuellen
     * Grundeinstellung an.
     * Anzeigeprinzip a und b :  WVZ-Code entsprechend Anhang 7, Kapitel 1. Gibt
     * den WVZ-Code an, den das
     *                           DE im Autarkbetrieb darstellen soll, falls die
     * Grundeinstellungs-Programmnummer des
     *                           Clusterkanals den Wert 0 enthlt oder die
     * Definition des Grundeinstellungsprogramms fr
     *                           dieses DE den Wert 255 "dont care" angibt. In
     * allen anderen Fllen ist die individuelle
     *                           Grundeinstellung nicht relevant. Ist der Wert 255,
     * so behlt das WZG seinen letzten
     *                           Stellzustand bei. Der Wert 0 ist fr DEs dieser
     * Anzeigeprinzipien nicht zulssig.
     * Anzeigeprinzip c und d :  Stellcode des vordefinierten Textes, den das DE im
     * Autarkbetrieb darstellen soll,
     *                           falls die Grundeinstellungs-Programmnummer des
     * Clusterkanals den Wert 0 enthlt oder
     *                           die Definition des Grundeinstellungsprogramms fr
     * dieses DE den Wert 255 "dont care"
     *                           angibt. In allen anderen Fllen ist die
     * individuelle Grundeinstellung nicht relevant. Ist
     * der
     *                           Wert 255, so behlt das WZG seinen zuletzt
     * dargestellten Text bei. Ist der Wert hingegen
     *                           0, so wird das Telegramm in der Langversion
     * bertragen und enthlt den darzustellenden
     *                           Text als ASCII-String.
     * Anzeigeprinzip e :        Stellcode der vordefinierten Gruppendefinition,
     * die das DE im Autarkbetrieb
     *                           darstellen soll, falls die
     * Grundeinstellungs-Programmnummer des Clusterkanals den Wert 0
     *                           enthlt oder die Definition des
     * Grundeinstellungsprogramms fr dieses DE den Wert 255
     *                           "dont care" angibt. In allen anderen Fllen ist
     * die individuelle Grundeinstellung nicht
     *                           relevant. Ist der Wert 255, so behlt die Gruppe
     * ihren zuletzt dargestellten Inhalt bei.
     * Ist der
     *                           Wert hingegen 0, so wird das Telegramm in der
     * Langversion bertragen und enthlt fr
     *                           jede Komponente den entsprechenden Stellzustand.
     * </pre>
     * Zulssige Werte :
     * <pre>
     *                            Cluster                a,b               c,d,e
     * Steuerungsprinzip 1        0, 255                 --                0, 255
     * Steuerungsprinzip 2        1-254, 255             Anhang 7, Kap. 1
     * 1-254,255
     * Steuerungsprinzip 3        0,1-254,255            --
     * 0,1-254,255
     * </pre>
     * Inhalt Funktionsbyte :<br> Der Inhalt des Funktionsbytes entspricht der verallgemeinerten Definition des
     * Funktionsbytes in Kapitel 6.2.18.<br> <br> Inhalt Grundeinstellung einer Komponenten :<br> Dieser Teil des
     * Telegramms wird genauso wie die Anzahl der Komponenten im Byte 7 nur in der Langversion des Telegramms bertragen,
     * d.h. insbesondere, dass der Stellcode in Byte 5 den Wert 0 haben muss. Wird die Langversion mit einem Stellcode
     * ungleich 0 empfangen, so wird mit Code 46 Langversion nicht zulssig negativ quittiert. Inhalt und Lnge der
     * wiederholten Komponentenblcke sind abhngig vom Anzeigeprinzip des adressierten DE:<br>
     * <pre>
     * Clusterkanle : Die Langversion ist fr Clusterkanle nicht zulssig. Die
     * individuellen Grundeinstellungen
     *                 werden an die einzelnen DEs direkt adressiert. Negative
     * Quittung Code 46.
     * Anzeigeprinzip a und b : Die Langversion ist inhaltsgem fr diese Anzeigen
     * nicht zulssig. Negative
     *                 Quittung Code 46.
     * Anzeigeprinzip c und d : Die Lnge eines Komponentenblocks (f) ist 1, so
     * dass die Langversion eine DEBlock-
     *                 Lnge von 6 + n hat. Die Anzahl Komponenten (Textzeichen)
     * ist deswegen auf
     *                 maximal 213 beschrnkt. Jeder Komponentenblock ist
     * folgendermaen aufgebaut:
     * <p/>
     *                 Byte 1           Textzeichen in ASCII-Code        siehe
     * Anhang 7, Kapitel 3
     * <p/>
     * Anzeigeprinzip e : Die Lnge eines Komponentenblocks (f) ist 2, so dass die
     * Langversion eine DE-Block-
     *                 Lnge von 6 + 2*n hat. Die Anzahl Komponenten ist deswegen
     * auf maximal 106
     *                 beschrnkt. Jeder Komponentenblock ist folgendermaen
     * aufgebaut:
     * <p/>
     *                 Byte 1           WVZ-Code                         siehe
     * Anhang 7, Kapitel 1
     *                 Byte 2           Funktionsbyte
     * <p/>
     *                 Der Inhalt des Funktionsbytes entspricht der
     * verallgemeinerten Definition des
     *                 Funktionsbytes in Kapitel 6.2.18.
     * </pre>
     * Die folgende Tabelle stellt bersichtlich alle Varianten des Telegramms zusammen, erlutert deren Funktion und gibt
     * an, fr welche Kombination aus Anzeige- und Steuerungsprinzip die jeweilige Verwendung zulssig ist. Dabei sind
     * Felder mit --  hinterlegt, wenn sie im betreffenden Kontext nicht relevant sind. Mit - hinterlegte Felder in der
     * Spalte "Steuerungsprinzip" bedeuten, dass die jeweiligen Stellbefehle negativ quittiert werden, die Nummer in
     * Klammern gibt den zu verwendenden Fehlercode an. Felder mit einem (zul) kennzeichnen die Zulssigkeit des
     * entsprechenden Telegramms fr das jeweilige DE. Eine hinter zul angefgte Nummer gekennzeichnetes Feld verweist auf
     * einen Telegrammtyp aus frheren Versionen der TLS, die bei Altlagen im jeweiligen Kontext noch zulssig sind.<br>
     * <pre>
     * Steuerungsprinzip
     * ID   Typ   Lnge  Stellcode  n      Funktion                       Anzeige
     *    f     Steuerungsprinzip
     *                                                                    prinzip
     * <p/>
     *          1        2       3
     * 3R   33    5      0          --     Grundeinstellung pro DE        a,b
     *    --    --       -(43)   --
     *                              --     individuell festlegen          c-e
     *    --    -(44)    -(43)   -(44)
     *                              --                                    Cl
     *    --    zul      -(43)   zul
     *                   1-254      --     Grundeinstellung festlegen     a,b
     *    --    --       zul 32  --
     *                              --     mit Codenummer                 c,d
     *    --    -(45)    zul     zul
     *                              --                                    e,Cl
     *    --    -(45)    zul 42  zul 42
     *                   255        --     Grundeinstellung "son't care"  alle
     *    --    zul      zul 42  zul 42
     *            6+n*f  0          1 --   Grundeinstellung festlegen     a,b
     *    --    --       -(43)   --
     *                              1 213  mit freier Definition          c,d
     *    1     zul      -(43)   zul
     *                              1 106                                 e
     *    2     zul      -(43)   zul
     *                              1 --                                  Cl
     *    --    --       -(43)   --
     * 3R         5      0          --     Grundeinstellung pro DE        a,b
     *    --    --       --      --
     *                              --     individuell festlegen          Cl
     *    --    zul      --      zul
     *                   1-254      --     Grundeinstellung melden        a,b
     *    --    --       zul 32  --
     *                              --     mit Codenummer                 c,d
     *    --    --       zul     zul
     *                              --                                    e,Cl
     *    --    --       zul 42  zul 42
     *                   255        --     Grundeinstellung "son't care"  alle
     *    --    zul      zul 42  zul 42
     *            6+n*f  0          1 --   Grundeinstellung melden        a,b
     *    --    --       --   --
     *                              1 213  mit freier Definition          c,d
     *    1     zul      --      zul
     *                              1 106                                 e
     *    2     zul      --      zul
     *                              1 --                                  Cl
     *    --    --       --      --
     * 19R        2      --         --     Abruf Grundeinstellung         alle
     *    --    zul      zul     zul
     * </pre>
     *
     * @param result Der vom Datenverteiler empgangene Datensatz, der in einen De-Block konvertiert wird.
     *
     * @return De-Block mit den konvertierten Daten.
     */
    public DeBlock Fg004Typ033Id003AtgTlsWzgGrundEinstellungAspParameterSoll(ResultData result) {
        DeBlock deBlock = null;

        // Datenblock beseht aus folgenden Inhalten:
        int     _AnzeigePrinzip_;
        int     _Stellcode_       = 0;
        int     _Funktion_        = 0;
        TlsData deApplicationData = null;

        // Umsetzung
        Data data = result.getData();  // Datensatz holen

        _AnzeigePrinzip_ = data.getUnscaledValue("AnzeigePrinzip").intValue();

        // Abhngig vom AnzeigePrinzip Stellcode und Funktion zuordnen
        switch (_AnzeigePrinzip_) {
        case 0 :
        case 1 : {
            _Stellcode_ = data.getArray("WVZAnzeige").getItem(0).getUnscaledValue("Code").intValue();

            // Wenn _Stellzustand_ = -1, muss Funktionsbyte auf "aus" gesetzt werden ( = 0).
            if (_Stellcode_ == -1) {
                _Stellcode_ = 0;              // ausschalten
                _Stellcode_ = _validWvzCode;  // letzter als gltig angenommener Code!
            } else {
                if (data.getArray("WVZAnzeige").getItem(0).getItem("Blinken").getTextValue("Status").getText().equals("aus")) {
                    _Funktion_ = 1;           // einschalten ohne Blinken
                } else {
                    _Funktion_ = 2;           // einschalten mit Blinken

                    // Blinkezeit auswerten.
                    if (data.getArray("WVZAnzeige").getItem(0).getItem("Blinken").getTextValue("Dauer").getText().equals("aus")) {
                        _Funktion_ &= 0x0f;   // Bit 4-7 = 0 setzten
                    } else {
                        _Funktion_ |= data.getArray("WVZAnzeige").getItem(0).getItem("Blinken").getUnscaledValue("Dauer").intValue() << 4;
                    }
                }
            }
        }

        break;

        case 2 :
        case 3 :
        case 4 :
        case 8 : {
            _Stellcode_ = data.getArray("StellCode").getItem(0).getUnscaledValue("Code").intValue();

            // Wenn _Stellzustand_ = -1, muss Funktionsbyte auf "aus" gesetzt werden ( = 0).
            if (_Stellcode_ == -1) {
                _Funktion_  = 0;              // ausschalten
                _Stellcode_ = _validWvzCode;  // letzter als gltig angenommener Code!
            } else {
                if (data.getArray("StellCode").getItem(0).getItem("Blinken").getTextValue("Status").getText().equals("aus")) {
                    _Funktion_ = 1;           // einschalten ohne Blinken
                } else {
                    _Funktion_ = 2;           // einschalten mit Blinken

                    // Blinkezeit auswerten.
                    if (data.getArray("StellCode").getItem(0).getItem("Blinken").getTextValue("Dauer").getText().equals("aus")) {
                        _Funktion_ &= 0x0f;   // Bit 4-7 = 0 setzten
                    } else {
                        _Funktion_ |= data.getArray("StellCode").getItem(0).getItem("Blinken").getUnscaledValue("Dauer").intValue() << 4;
                    }
                }
            }
        }

        break;

        default : {
            debug.warning("Anzeigeprinzip ungltig (0-4,8) : " + _AnzeigePrinzip_ + Debug.NEWLINE + "De-Block konnte nicht erzeugt werden:" + Debug.NEWLINE + data.toString());
        }
        }

        // Ermitteln, ob Langversion vorliegt...
        if ((data.getArray("TextDefinition").getLength() == 0) && (data.getArray("GruppenDefinition").getLength() == 0)) {

            // DeBlock als TlsData aufbauen...
            deApplicationData = new TlsData(3);
            deApplicationData.writeIntAsByte(_AnzeigePrinzip_);
            deApplicationData.writeIntAsByte(_Stellcode_);
            deApplicationData.writeIntAsByte(_Funktion_);
        } else {
            int count = 0;

            // Abhngig vom AnzeigePrinzip Stellcode und Funktion zuordnen
            switch (_AnzeigePrinzip_) {
            case 2 :
            case 3 : {
                count = data.getArray("TextDefinition").getLength();

                // DeBlock als TlsData aufbauen...
                deApplicationData = new TlsData(4 + count);
                deApplicationData.writeIntAsByte(_AnzeigePrinzip_);
                deApplicationData.writeIntAsByte(_Stellcode_);
                deApplicationData.writeIntAsByte(_Funktion_);
                deApplicationData.writeIntAsByte(count);

                int text;

                for (int i = 0; i < count; i++) {
                    text = data.getArray("TextDefinition").getItem(i).getTextValue("Text").getText().charAt(0);
                    deApplicationData.writeIntAsByte(text);
                }
            }

            break;

            case 4 : {
                count = data.getArray("GruppenDefinition").getLength();

                // DeBlock als TlsData aufbauen...
                deApplicationData = new TlsData(4 + (count * 2));
                deApplicationData.writeIntAsByte(_AnzeigePrinzip_);
                deApplicationData.writeIntAsByte(_Stellcode_);
                deApplicationData.writeIntAsByte(_Funktion_);
                deApplicationData.writeIntAsByte(count);

                for (int i = 0; i < count; i++) {
                    _Stellcode_ = data.getArray("GruppenDefinition").getItem(i).getUnscaledValue("Code").intValue();

                    // Wenn _Stellzustand_ = -1, muss Funktionsbyte auf "aus" gesetzt werden ( = 0).
                    if (_Stellcode_ == -1) {
                        _Funktion_  = 0;              // ausschalten
                        _Stellcode_ = _validWvzCode;  // letzter als gltig angenommener Code!
                    } else {
                        if (data.getArray("GruppenDefinition").getItem(i).getItem("Blinken").getTextValue("Status").getText().equals("aus")) {
                            _Funktion_ = 1;           // einschalten ohne Blinken
                        } else {
                            _Funktion_ = 2;           // einschalten mit Blinken

                            // Blinkezeit auswerten.
                            if (data.getArray("GruppenDefinition").getItem(0).getItem("Blinken").getTextValue("Dauer").getText().equals("aus")) {
                                _Funktion_ &= 0x0f;   // Bit 4-7 = 0 setzten
                            } else {
                                _Funktion_ |= data.getArray("GruppenDefinition").getItem(0).getItem("Blinken").getUnscaledValue("Dauer").intValue() << 4;
                            }
                        }
                    }

                    deApplicationData.writeIntAsByte(_Stellcode_);
                    deApplicationData.writeIntAsByte(_Funktion_);
                }
            }

            break;

            default : {
                debug.warning("Anzeigeprinzip ungltig (0-4,8) : " + _AnzeigePrinzip_ + Debug.NEWLINE + "De-Block konnte nicht erzeugt werden:" + Debug.NEWLINE + data.toString());
            }
            }
        }

        // DeBlock erzeugen
        try {
            deBlock = new DeBlock(deApplicationData, _de.getEak().getSm().getOsi7Adress(), _de.getFg(), 33, 3, _de.getDeChannel());
            debug.finest("TLS-DE-Block: " + HexDumper.toString(deBlock.getTlsDeDataBlock()));
        }
        catch (IllegalOsi7TelegramStructureException e) {
            debug.warning("Fehler beim Erzeugen eines De-Blocks zum Versand: " + e);
        }

        return deBlock;
    }

    /**
     * Entspricht {@link Fg004Default#Fg004Typ033Id003AtgTlsWzgGrundEinstellungAspParameterIst(de.bsvrz.kex.tls.osi7.DeBlock,
     * de.bsvrz.dav.daf.main.Data, de.bsvrz.dav.daf.main.DataDescription)}, jedoch werden die Daten als nachgeliefert gekennzeichnet.
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz fr die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung fr die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg004Typ033Id035AtgTlsWzgGrundEinstellungAspParameterIst(DeBlock deBlock, Data data, DataDescription dataDescription) {

        // Generierte Methode: Standardverarbeitung wie bei Id 3, aber als "nachgelieferte" Daten an den DaV bergeben.
        ResultData resultData = Fg004Typ033Id003AtgTlsWzgGrundEinstellungAspParameterIst(deBlock, data, dataDescription);

        return new ResultData(_de.getDe(), dataDescription, resultData.getDataTime(), resultData.getData(), true);
    }

    /**
     * FG 4 : Typ: 42 Id: 3 Name: Grundprogramm (Abruf / Antwort). TLS-Datenbeschreibung:<br>
     * <pre>
     * Position         Bezeichnung                             Erluterung
     * Byte 1           Lnge DE-Block                          [ 3 ]
     * Byte 2           Daten-Endgerte-Kanal (DE)              [ 193..222, 223,
     * (255), (evtl. 1..254,255)1 ]
     * Byte 3           Typ der DE-Daten                        [ 42 ]
     * Byte 4           Grundprogramm-Nummer                    [ 1 - 254, 255 ]
     * 1) es wird der Clusterkanal gem der Definition Seite A6.1.1  3, Kapitel
     * 1.4 angewendet, lediglich bei Anlagen,
     * die
     * nach dem Prinzip der alten FG5 (TLS Ausgabe 1993) arbeiten, knnen die
     * E/A-Kanle direkt adressiert werden
     * </pre>
     * Inhalt Byte Programm-Nummer:<br> Nummer des Programms, in das bei Autarkbetrieb umgeschaltet werden soll. Wird die
     * Programmnummer 255 ausgewhlt, so bleibt das vor Beginn des Autarkbetrieb geschaltete Programm im Autarkbetrieb
     * bestehen.<br>
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz fr die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung fr die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg004Typ042Id003AtgTlsWzgGrundProgrammAspParameterIst(DeBlock deBlock, Data data, DataDescription dataDescription) {
        ResultData resultData = null;

        // Grundprogramm aus Telegramm --------------------------------------------------------------------------------
        int _GrundProgramm_ = deBlock.getDeData().readUnsignedByteAsInt();

        data.getUnscaledValue("GrundProgramm").set(_GrundProgramm_);

        // Ergebnisdatensatz erzeugen...
        resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);

        return resultData;
    }

    /**
     * FG 4 : Typ: 42 Id: 3 Name: Grundprogramm (Abruf / Antwort). TLS-Datenbeschreibung:<br>
     * <pre>
     * Position         Bezeichnung                             Erluterung
     * Byte 1           Lnge DE-Block                          [ 3 ]
     * Byte 2           Daten-Endgerte-Kanal (DE)              [ 193..222, 223,
     * (255), (evtl. 1..254,255)1 ]
     * Byte 3           Typ der DE-Daten                        [ 42 ]
     * Byte 4           Grundprogramm-Nummer                    [ 1 - 254, 255 ]
     * 1) es wird der Clusterkanal gem der Definition Seite A6.1.1  3, Kapitel
     * 1.4 angewendet, lediglich bei Anlagen,
     * die
     * nach dem Prinzip der alten FG5 (TLS Ausgabe 1993) arbeiten, knnen die
     * E/A-Kanle direkt adressiert werden
     * </pre>
     * Inhalt Byte Programm-Nummer:<br> Nummer des Programms, in das bei Autarkbetrieb umgeschaltet werden soll. Wird die
     * Programmnummer 255 ausgewhlt, so bleibt das vor Beginn des Autarkbetrieb geschaltete Programm im Autarkbetrieb
     * bestehen.<br>
     *
     * @param result Der vom Datenverteiler empgangene Datensatz, der in einen De-Block konvertiert wird.
     *
     * @return De-Block mit den konvertierten Daten.
     */
    public DeBlock Fg004Typ042Id003AtgTlsWzgGrundProgrammAspParameterSoll(ResultData result) {
        DeBlock deBlock = null;

        // Datenblock beseht aus folgenden Inhalten:
        int _GrundProgramm_;

        // Umsetzung
        Data data = result.getData();  // Datensatz holen

        _GrundProgramm_ = data.getUnscaledValue("GrundProgramm").intValue();

        // DeBlock als TlsData aufbauen...
        TlsData deApplicationData = new TlsData(1);

        deApplicationData.writeIntAsByte(_GrundProgramm_);

        // DeBlock erzeugen
        try {
            deBlock = new DeBlock(deApplicationData, _de.getEak().getSm().getOsi7Adress(), _de.getFg(), 42, 3, _de.getDeChannel());
            debug.finest("TLS-DE-Block: " + HexDumper.toString(deBlock.getTlsDeDataBlock()));
        }
        catch (IllegalOsi7TelegramStructureException e) {
            debug.warning("Fehler beim Erzeugen eines De-Blocks zum Versand: " + e);
        }

        return deBlock;
    }

    /**
     * Entspricht {@link Fg004Default#Fg004Typ042Id003AtgTlsWzgGrundProgrammAspParameterIst(de.bsvrz.kex.tls.osi7.DeBlock,
     * de.bsvrz.dav.daf.main.Data, de.bsvrz.dav.daf.main.DataDescription)}, jedoch werden die Daten als nachgeliefert gekennzeichnet.
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz fr die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung fr die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg004Typ042Id035AtgTlsWzgGrundProgrammAspParameterIst(DeBlock deBlock, Data data, DataDescription dataDescription) {

        // Generierte Methode: Standardverarbeitung wie bei Id 3, aber als "nachgelieferte" Daten an den DaV bergeben.
        ResultData resultData = Fg004Typ042Id003AtgTlsWzgGrundProgrammAspParameterIst(deBlock, data, dataDescription);

        return new ResultData(_de.getDe(), dataDescription, resultData.getDataTime(), resultData.getData(), true);
    }

    /**
     * FG 4 : Typ: 43 Id: 3 Name: Codeliste (Antwort). TLS-Datenbeschreibung:<br>
     * <pre>
     * Position         Bezeichnung                             Erluterung
     * Byte 1           Lnge DE-Block                          [ 3 ]
     * Byte 2           Daten-Endgerte-Kanal (DE)              [ 1..254,255 ]
     * Byte 3           Typ der DE-Daten                        [ 43 ]
     * Byte 4           Grundprogramm-Nummer                    [ 1 - 254, 255 ]
     * 1) es wird der Clusterkanal gem der Definition Seite A6.1.1  3, Kapitel
     * 1.4 angewendet, lediglich bei Anlagen,
     * die
     * nach dem Prinzip der alten FG5 (TLS Ausgabe 1993) arbeiten, knnen die
     * E/A-Kanle direkt adressiert werden
     * </pre>
     * Inhalt Byte Programm-Nummer:<br> Nummer des Programms, in das bei Autarkbetrieb umgeschaltet werden soll. Wird die
     * Programmnummer 255 ausgewhlt, so bleibt das vor Beginn des Autarkbetrieb geschaltete Programm im Autarkbetrieb
     * bestehen.<br>
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz fr die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung fr die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg004Typ043Id003AtgTlsWzgCodeListeAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
        ResultData resultData = null;

        // Anzeigeprinzip aus Telegramm --------------------------------------------------------------------------------
        int _AnzeigePrinzip_ = deBlock.getDeData().readUnsignedByteAsInt();

        data.getUnscaledValue("AnzeigePrinzip").set(_AnzeigePrinzip_);

        // Stellcodes auslesen...
        int _Stellcode_;
        int count = deBlock.getDeData().readUnsignedByteAsInt();

        data.getUnscaledArray("StellCode").setLength(count);

        for (int i = 0; i < count; ++i) {
            _Stellcode_   = deBlock.getDeData().readUnsignedByteAsInt();
            _validWvzCode = _Stellcode_;
            data.getUnscaledArray("StellCode").getValue(i).set(_Stellcode_);
        }

        // Ergebnisdatensatz erzeugen...
        resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);

        return resultData;
    }

    /**
     * Entspricht {@link Fg004Default#Fg004Typ043Id003AtgTlsWzgCodeListeAspTlsAntwort(de.bsvrz.kex.tls.osi7.DeBlock,
     * de.bsvrz.dav.daf.main.Data, de.bsvrz.dav.daf.main.DataDescription)}, jedoch werden die Daten als nachgeliefert gekennzeichnet.
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz fr die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung fr die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg004Typ043Id035AtgTlsWzgCodeListeAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {

        // Generierte Methode: Standardverarbeitung wie bei Id 3, aber als "nachgelieferte" Daten an den DaV bergeben.
        ResultData resultData = Fg004Typ043Id003AtgTlsWzgCodeListeAspTlsAntwort(deBlock, data, dataDescription);

        return new ResultData(_de.getDe(), dataDescription, resultData.getDataTime(), resultData.getData(), true);
    }

    /**
     * FG 4 : Typ: 44 Id: 3 Name: Codedefinition (Abruf / Antwort). TLS-Datenbeschreibung:<br>
     * <pre>
     * Position         Bezeichnung                                    Erluterung
     * Byte 1           Lnge DE-Block                                 Lnge des
     * folgenden DE-Blocks [5+f*n]
     * Byte 2           Daten-Endgerte-Kanal (DE)                     [ 1..254 ]
     * Byte 3           Typ der DE-Daten                               [ 44 ]
     * Byte 4           Anzeigeprinzip                                 siehe unten
     * Byte 5           Stellcode                                      [1..254,
     * 255]siehe unten
     * Byte 6           n : Anzahl Komponenten                         0, 1 - [213
     * | 106 | 53] siehe unten
     * f Bytes...       Definition der 1.Komponente                    siehe unten
     * f Bytes          Definition der n.Komponente
     * </pre>
     * Inhalt Byte : Anzeigeprinzip<br> Das Byte gibt das Anzeigeprinzip des adressierten/sendenden DEs an und ermglicht
     * damit die genaue Interpretation des folgenden Telegramminhalts. Es ist folgendermaen codiert:<br>
     * <pre>
     * Anzeigeprinzip        Wert
     *              a        0
     *              b        1
     *              c        2
     *              d        3
     *              e        4
     *        Cluster        8
     * </pre>
     * Erhlt ein DE in Abrufrichtung eine Nachricht mit einem Wert, der nicht seinem Anzeigeprinzip entspricht, so wird
     * die Nachricht mit Code 21 "falsches Anzeigeprinzip" negativ quittiert.<br> <br> Inhalt Byte: Stellcode:<br> Gibt den
     * Stellcode an, dessen Definition neu angelegt, gendert oder gelscht werden soll. Der Wert 255 ist nur gltig im
     * Zusammenhang mit einer Anzahl Komponenten von 0. In diesem Fall dient die Nachricht zum Lschen aller Definitionen
     * aus dem Speicher des Gertes, andernfalls wird mit Code 43 "unzulssiger Stellcode" negativ quittiert.<br> <br>
     * Inhalt Byte : Anzahl Komponenten:<br> Fr WZG vom Anzeigetyp a) ist das Telegramm nicht zulssig. Die Nachricht wird
     * von solchen DEs mit Code 41 "Stellcodevorrat nicht nderbar" negativ quittiert.<br> Fr WZG vom Anzeigeprinzip b)
     * gibt das Byte in Antwortrichtung als Antwort auf Abruf mit Typ 21 optional die Anzahl der an der Darstellung des
     * erfragten WVZ-Codes beteiligten Lampen an. In Abrufrichtung ist das Telegramm fr WZG vom Anzeigeprinzip b) nicht
     * zulssig und wird ebenfalls mit Code 41 negativ quittiert.<br> Fr WZG vom Anzeigeprinzip c) und d) (Wechseltext)
     * gibt das Byte die Lnge des folgenden Textes an. Die Angabe der Textlnge ist unabhngig von der Anzahl der
     * tatschlich vorhandenen Textzeichen. Wenn eine geringere Anzahl Zeichen bergeben wird, wird der Rest ausgeschaltet.
     * Wird eine grere Anzahl bergeben, wird der Befehl mit Code 19 "Wechseltext nicht darstellbar, da zu lang" negativ
     * quittiert.<br> Fr WZG vom Anzeigeprinzip e) (Gruppen) gibt das Byte die Anzahl der Anzeigen/Prismen dieses DEs an.
     * Als Zuweisung sind nur DE-Blcke zulssig, die genau fr jede/jedes an diesem DE vorhandene Anzeige/Prisma eine
     * Definition enthalten. Andere DE-Blcke werden mit Code 17 negativ quittiert.<br> Fr Clusterkanle gibt das Byte die
     * Anzahl der DEs der FG 4 an diesem EAK an. Als Zuweisung sind nur DEBlcke zulssig, die genau fr jedes an diesem
     * EAK vorhandene DE eine Definition enthalten. Andere DEBlcke werden mit Code 17 negativ quittiert.<br> Der Wert 0
     * ist zulssig fr WZGs c)-e) und Cluster, um Definitionen von Stellcodes zu lschen, wenn dies nicht generell
     * ausgeschlossen wurde (Anhang 9), ansonsten werden solche Nachrichten mit Code 41 "Stellcode-Vorrat nicht nderbar"
     * negativ quittiert.<br> <br> Inhalt : Definition einer Komponente:<br> Fr WZG vom Anzeigeprinzip a) und b) (feste
     * Zeicheninhalte) ist das Telegramm in Abrufrichtung nicht zulssig. Fr aktiv leuchtende WZG kann das Telegramm in
     * Antwortrichtung (auf Abruf mit Typ 21) allerdings optional dazu genutzt werden, der Zentrale die an der Darstellung
     * eines WVZ beteiligten Lampen zu benennen. Mit dieser Information kann die Zentrale die oft geforderte berwachung
     * von Lampenbrenndauern implementieren, ohne die Zuordnung von Lampen zu Zeicheninhalten separat parametrieren zu
     * mssen. Die Definition einer Komponente hat dann je ein Byte und ist wie folgt aufgebaut:<br>
     * <pre>
     * Byte 1           Nummer der beteiligten Lampe          Nummerierung wie bei
     * Typ 3
     * </pre>
     * Fr WZG vom Anzeigeprinzip c) und d) (Wechseltext) besteht die Definition einer Komponente aus jeweils nur einem
     * Byte und ist wie folgt aufgebaut :<br>
     * <pre>
     * Byte 1           Textzeichen in ASCII-Code             siehe Anhang 7,
     * Kapitel 3
     * </pre>
     * Fr WZG vom Anzeigeprinzip e) (Gruppen) besteht die Definition einer Komponente aus jeweils 2 Bytes und ist wie
     * folgt aufgebaut:<br>
     * <pre>
     * Byte 1           WVZ-Code                              siehe Anhang 7,
     * Kapitel 1
     * Byte 2           Funktionsbyte                         siehe Definition in
     * Kapitel 6.2.18
     * </pre>
     * Der Inhalt des Funktionsbytes entspricht der verallgemeinerten Definition des Funktionsbytes in Kapitel 6.2.18.<br>
     * Fr Clusterkanle besteht die Definition einer Komponente aus jeweils 4 Bytes. Jedes DE der FG4 an dem EAK bildet
     * dabei eine Komponente. (Es ist darauf zu achten, dass sich fr EAKs, die im Programmmodus gesteuert werden knnen,
     * die Anzahl der maximal zulssigen DEs auf 53 reduziert gegenber den 56 per DEKonfiguration mglichen). Der Aufbau
     * ist wie folgt:<br>
     * <pre>
     * Byte 1           DE-Nummer der Komponente
     * Byte 2           Anzeigeprinzip dieses DEs              siehe unten
     * Byte 3           Stellcode fr die Komponente           siehe Anhang 7,
     * Kapitel 1 bzw. [ 1  254, 255 ]
     * Byte 4           Funktionsbyte                          siehe Definition in
     * Kapitel 6.2.18
     * </pre>
     * Das Byte 2 codiert das Anzeigeprinzip des DEs und ermglicht so eine eindeutige Interpretation der folgenden
     * Definitionskomponente. Es werden die gleiche Wert wie oben verwendet, nur der Wert 8 fr Clusterkanle ist hier
     * natrlich nicht zulssig.<br> Das Byte 3 hat dabei die jedem DE entsprechende Bedeutung entweder als WVZ-Code oder
     * als Stellcode einer Text- oder einer Gruppendefinition. Hier ist auch der Wert 255 zulssig, er bedeutet unabhngig
     * vom Anzeigeprinzip des DEs, dass dieses DE individuell angesteuert werden kann, auch wenn der Cluster im
     * Programmmodus gesteuert wird.<br> Der Inhalt des Funktionsbytes entspricht der verallgemeinerten Definition des
     * Funktionsbytes in Kapitel 6.2.18.<br>
     * <pre>
     * Hinweis :    Es wird empfohlen, die Definition der Codes im EAK intern
     * E/A-kanalbezogen abzulegen, um zu
     *              verhindern, dass bei genderter DE-Zuordnung alle
     * Programmdefinitionen gelscht werden
     *              mssen. Ansonsten, oder wenn die projektierten E/A-Kanle nicht
     * gleichgeblieben sind, werden
     *              bei Zuweisung einer genderten DE-Zuordnung ber die Nachricht
     * Typ 33 der
     *              Funktionsgruppe 254 alle Programmdefinitionen fr die
     * betroffenen EAKs gelscht. Der
     *              Clusterkanal meldet dann spontan mit Typ 43, dass fr ihn jetzt
     * keine Programme mehr definiert
     *              sind. Enthlt die Zuweisung der DE-Zuordnung allerdings bzgl.
     * der vorigen Zuordnungstabelle
     *              nur nderungen der DE-Nummern einer gleichbleibenden Menge von
     * E/A-Kanlen oder nur
     *              nderungen bzgl. anderer EAKs, so bleiben die
     * Programm-Definitionen erhalten.
     * </pre>
     * Es ist dabei zu beachten, dass die Stellcodes der einzelnen DEs nach Anzeigeprinzip c)-e) genauso wie trivialerweise
     * die WVZ-Codes fr Anzeigen vom Prinzip a) oder b) von den Programmnummern des EAKs vollstndig unabhngig sind.<br>
     * <br> Erhlt ein DE ein Telegramm, dessen Lnge nicht den obigen Festlegungen entspricht, so wird die Nachricht
     * negativ quittiert.<br> Die folgende Tabelle stellt bersichtlich alle Varianten des Telegramms zusammen, erlutert
     * deren Funktion und gibt an, fr welche Kombination aus Anzeige- und Steuerungsprinzip die jeweilige Verwendung
     * zulssig ist. Dabei sind Felder mit --  hinterlegt, wenn sie im betreffenden Kontext nicht relevant sind. Mit -
     * hinterlegte Felder in der Spalte "Steuerungsprinzip" bedeuten, dass die jeweiligen Stellbefehle negativ quittiert
     * werden, die Nummer in Klammern gibt den zu verwendenden Fehlercode an. Felder mit einem (zul) kennzeichnen die
     * Zulssigkeit des entsprechenden Telegramms fr das jeweilige DE.<br>
     * <pre>
     * ID   Typ   Lnge  Stellcode  n      Funktion                       Anzeige
     *    f     Steuerungsprinzip
     *                                                                    prinzip
     * <p/>
     *          1        2       3
     * 3R   44    5+n*f  1-254      0      Code-Definition lschen        a,b
     *    --    --       -(41)   --
     *                                                                    c-e, Cl
     *    --    -(45)    zul     zul
     * <p/>
     *                              1 --   Code-Definition anlegen        a,b
     *    --    --       -(41)    --
     *                              1 213  oder ndern                    c,d
     *    1     -(45)    zul     zul
     *                              1 106                                 e
     *    2     -(45)    zul     zul
     *                              1  53                                 Cl
     *    4     -(45)    zul     zul
     *                   255        0      Alle Code-Definitionen         a,b
     *    --    --       -(41)   --
     *                                     lschen                        c.e, Cl
     *    --    -(45)    zul     zul
     * 3A                1-254      0      Meldung Code gelscht          a,b
     *    --    --       --      --
     *                                     (nicht auf Abfrage)            c-e, Cl
     *    --    --       zul     zul
     *                              1 --   --                             a
     *    --    --       --      --
     *                              1 213  Meldung beteiligter Lampen     b
     *    1     --       zul     --
     *                              1 213  Meldung Code-Definiton         c,d
     *    1     --       zul     zul
     *                              1 106                                 e
     *    2     --       zul     zul
     *                              1  53                                 Cl
     *    4     --       zul     zul
     *                   255        0      Meldung alle Codes gelscht    a,b
     *    --    --       --      --
     *                                     (nicht auf Abfrage)            c.e, Cl
     *    --    --       zul     zul
     * </pre>
     * Lschen und ndern von Codedefinitionen<br> Lschen und ndern nicht aktuell oder als Grundeinstellung verwendeter
     * Codes des Clusterkanals ist zulssig und kann ohne weiteres durchgefhrt werden. Lschen oder ndern aktuell
     * anstehender Codes fhren dazu, dass der Clusterkanal in der Folge spontan vom Programmmodus in den Komponentenmodus
     * bergeht und dies spontan mit einer Nachricht Typ 55 meldet. Damit bleibt der aktuelle Anzeigezustand erhalten.
     * Entsprechendes gilt, wenn der genderte oder gelschte Code als Grundprogramm am Cluster definiert ist: In der Folge
     * gilt fr den Cluster, dass die Grundeinstellung im Komponentenmodus definiert ist. Der Cluster meldet dies spontan
     * mit einer Nachricht vom Typ 33.<br> Ein Stellcode fr ein einzelnes DE darf nicht gelscht werden, solange er in der
     * Definition eines oder mehrerer Programme seines Clusterkanals noch verwendet wird. Wenn der EAK feststellt, dass der
     * zu lschende Stellcode noch in der Definition eines seiner Programme verwendet wird, so wird der Lschbefehl mit
     * Code 47 "Lschen benutzter Stellcodes unzulssig" negativ quittiert.<br>
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz fr die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung fr die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     *
     */
    public ResultData Fg004Typ044Id003AtgTlsWzgCodeDefinitionAspParameterIst(DeBlock deBlock, Data data, DataDescription dataDescription) {
        ResultData resultData = null;

        // AnzeigePrinzip aus Telegramm --------------------------------------------------------------------------------
        int _AnzeigePrinzip_ = deBlock.getDeData().readUnsignedByteAsInt();

        data.getUnscaledValue("AnzeigePrinzip").set(_AnzeigePrinzip_);

        // Stellcode und Funktion aus Telegramm ------------------------------------------------------------------------
        int _Stellcode_ = deBlock.getDeData().readUnsignedByteAsInt();

        _validWvzCode = _Stellcode_;
        data.getUnscaledValue("StellCode").set(_Stellcode_);

        // Alle dynamischen Arrays auf Lnge 0, werden abhngig von der Semantik nachfolgende auf richtige Gre gesetzt.
        data.getArray("LampenDefinition").setLength(0);
        data.getArray("TextDefinition").setLength(0);
        data.getArray("GruppenDefinition").setLength(0);
        data.getArray("ClusterDefinition").setLength(0);

        // Anzahl der Komponenten aus Telegramm ------------------------------------------------------------------------
        int count = deBlock.getDeData().readUnsignedByteAsInt();

        // Abhngig vom AnzeigePrinzip Stellcode und Funktion zuordnen
        switch (_AnzeigePrinzip_) {
        case 1 :  // Rckmeldung der vorhanden Lampen
        {

            // Lampen-Paare (je 2 Lampen)  aus Telegramm -----------------------------------------------------------------------
            int   countLightPairs = count;
            int   countByte       = (countLightPairs / 4) + 1;
            int[] _LampenStatus_  = new int[countByte];

            for (int i = 0; i < countByte; i++) {
                _LampenStatus_[i] = deBlock.getDeData().readUnsignedByteAsInt();
            }

            data.getArray("LampenDefinition").setLength(countLightPairs);

            int lightState;

            for (int i = 0; i < countLightPairs; i++) {
                lightState = (_LampenStatus_[i / 4] >>> (i % 4) * 2) & 0x03;
                data.getArray("LampenDefinition").getItem(i).getUnscaledValue("Lampen").set(lightState);
            }
        }

        break;

        case 2 :  // Wechseltext
        case 3 : {
            data.getArray("TextDefinition").setLength(count);

            char text;

            for (int i = 0; i < count; ++i) {
                text = (char) deBlock.getDeData().readUnsignedByteAsInt();
                data.getArray("TextDefinition").getItem(i).getTextValue("Text").setText("" + text);
            }
        }

        break;

        case 4 :  // Gruppen
        {
            data.getArray("GruppenDefinition").setLength(count);

            int _Funktion_;

            for (int i = 0; i < count; ++i) {
                _Stellcode_   = deBlock.getDeData().readUnsignedByteAsInt();
                _validWvzCode = _Stellcode_;
                _Funktion_    = deBlock.getDeData().readUnsignedByteAsInt();

                // Wenn Funktion = ausgeschaltet, hat Stell-Code keine Bedeutung. Wird deshalb fr
                // Protokollausgaben auf -1 = "aus" gesetzt.
                if ((_Funktion_ & 0x0f) == 0) {
                    _Stellcode_ = -1;
                }

                data.getArray("StellCode").getItem(i).getUnscaledValue("Code").set(_Stellcode_);

                if ((_Funktion_ & 0x02) == 0) {
                    data.getArray("StellCode").getItem(i).getItem("Blinken").getUnscaledValue("Status").setText("aus");
                    data.getArray("StellCode").getItem(i).getItem("Blinken").getUnscaledValue("Dauer").setText("aus");
                } else {
                    data.getArray("StellCode").getItem(i).getItem("Blinken").getUnscaledValue("Status").setText("ein");

                    int _BlinkDauer_ = _Funktion_ >>> 4;

                    if ((_BlinkDauer_ < 5) || (_BlinkDauer_ > 15)) {
                        debug.warning("Blinkdauer ungltig (5-15) : " + (_Funktion_ >>> 4) + ". Blinkdauerwert wurde auf 5 gesetzt!");
                        _BlinkDauer_ = 5;
                    }

                    data.getArray("StellCode").getItem(i).getItem("Blinken").getUnscaledValue("Dauer").set(_BlinkDauer_);
                }

                if ((_Funktion_ & 0x04) == 0) {
                    data.getArray("StellCode").getItem(i).getItem("Status").getUnscaledValue("Fehler").setText("ok");
                } else {
                    data.getArray("StellCode").getItem(i).getItem("Status").getUnscaledValue("Fehler").setText("Fehler");
                }

                if ((_Funktion_ & 0x08) == 0) {
                    data.getArray("StellCode").getItem(i).getItem("Status").getUnscaledValue("Programm").setText("abgeschlossen");
                } else {
                    data.getArray("StellCode").getItem(i).getItem("Status").getUnscaledValue("Programm").setText("in Arbeit");
                }
            }
        }

        break;

        case 8 :  // Cluster
        {
            data.getArray("ClusterDefinition").setLength(count);

            int _De_;
            int _Funktion_;

            for (int i = 0; i < count; ++i) {
                _De_ = deBlock.getDeData().readUnsignedByteAsInt();
                data.getArray("ClusterDefinition").getItem(i).getUnscaledValue("DeNummer").set(_De_);
                _AnzeigePrinzip_ = deBlock.getDeData().readUnsignedByteAsInt();
                data.getArray("ClusterDefinition").getItem(i).getUnscaledValue("AnzeigePrinzip").set(_AnzeigePrinzip_);
                _Stellcode_   = deBlock.getDeData().readUnsignedByteAsInt();
                _validWvzCode = _Stellcode_;
                _Funktion_    = deBlock.getDeData().readUnsignedByteAsInt();

                // Wenn Funktion = ausgeschaltet, hat Stell-Code keine Bedeutung. Wird deshalb fr
                // Protokollausgaben auf -1 = "aus" gesetzt.
                if ((_Funktion_ & 0x0f) == 0) {
                    _Stellcode_ = -1;
                }

                data.getArray("ClusterDefinition").getItem(i).getItem("Anzeige").getUnscaledValue("Code").set(_Stellcode_);

                if ((_Funktion_ & 0x02) == 0) {
                    data.getArray("ClusterDefinition").getItem(i).getItem("Anzeige").getItem("Blinken").getUnscaledValue("Status").setText("aus");
                    data.getArray("ClusterDefinition").getItem(i).getItem("Anzeige").getItem("Blinken").getUnscaledValue("Dauer").setText("aus");
                } else {
                    data.getArray("ClusterDefinition").getItem(i).getItem("Anzeige").getItem("Blinken").getUnscaledValue("Status").setText("ein");

                    int _BlinkDauer_ = _Funktion_ >>> 4;

                    if ((_BlinkDauer_ < 5) || (_BlinkDauer_ > 15)) {
                        debug.warning("Blinkdauer ungltig (5-15) : " + (_Funktion_ >>> 4) + ". Blinkdauerwert wurde auf 5 gesetzt!");
                        _BlinkDauer_ = 5;
                    }

                    data.getArray("ClusterDefinition").getItem(i).getItem("Anzeige").getItem("Blinken").getUnscaledValue("Dauer").set(_BlinkDauer_);
                }

                if ((_Funktion_ & 0x04) == 0) {
                    data.getArray("ClusterDefinition").getItem(i).getItem("Anzeige").getItem("Status").getUnscaledValue("Fehler").setText("ok");
                } else {
                    data.getArray("ClusterDefinition").getItem(i).getItem("Anzeige").getItem("Status").getUnscaledValue("Fehler").setText("Fehler");
                }

                if ((_Funktion_ & 0x08) == 0) {
                    data.getArray("ClusterDefinition").getItem(i).getItem("Anzeige").getItem("Status").getUnscaledValue("Programm").setText("abgeschlossen");
                } else {
                    data.getArray("ClusterDefinition").getItem(i).getItem("Anzeige").getItem("Status").getUnscaledValue("Programm").setText("in Arbeit");
                }
            }
        }

        break;

        default : {
            debug.warning("Anzeigeprinzip ungltig (0-4,8) : " + _AnzeigePrinzip_ + Debug.NEWLINE + "De-Block (Osi7Adresse-DeNummer-FG-Typ-Id): " + deBlock.getLongKey() + " verworfen:" + Debug.NEWLINE + deBlock.toString());

            return resultData;
        }
        }

        resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);

        return resultData;
    }

    /**
     * FG 4 : Typ: 44 Id: 3 Name: Codedefinition (Abruf / Antwort). TLS-Datenbeschreibung:<br>
     * <pre>
     * Position         Bezeichnung                                    Erluterung
     * Byte 1           Lnge DE-Block                                 Lnge des
     * folgenden DE-Blocks [5+f*n]
     * Byte 2           Daten-Endgerte-Kanal (DE)                     [ 1..254 ]
     * Byte 3           Typ der DE-Daten                               [ 44 ]
     * Byte 4           Anzeigeprinzip                                 siehe unten
     * Byte 5           Stellcode                                      [1..254,
     * 255]siehe unten
     * Byte 6           n : Anzahl Komponenten                         0, 1 - [213
     * | 106 | 53] siehe unten
     * f Bytes...       Definition der 1.Komponente                    siehe unten
     * f Bytes          Definition der n.Komponente
     * </pre>
     * Inhalt Byte : Anzeigeprinzip<br> Das Byte gibt das Anzeigeprinzip des adressierten/sendenden DEs an und ermglicht
     * damit die genaue Interpretation des folgenden Telegramminhalts. Es ist folgendermaen codiert:<br>
     * <pre>
     * Anzeigeprinzip        Wert
     *              a        0
     *              b        1
     *              c        2
     *              d        3
     *              e        4
     *        Cluster        8
     * </pre>
     * Erhlt ein DE in Abrufrichtung eine Nachricht mit einem Wert, der nicht seinem Anzeigeprinzip entspricht, so wird
     * die Nachricht mit Code 21 "falsches Anzeigeprinzip" negativ quittiert.<br> <br> Inhalt Byte: Stellcode:<br> Gibt den
     * Stellcode an, dessen Definition neu angelegt, gendert oder gelscht werden soll. Der Wert 255 ist nur gltig im
     * Zusammenhang mit einer Anzahl Komponenten von 0. In diesem Fall dient die Nachricht zum Lschen aller Definitionen
     * aus dem Speicher des Gertes, andernfalls wird mit Code 43 "unzulssiger Stellcode" negativ quittiert.<br> <br>
     * Inhalt Byte : Anzahl Komponenten:<br> Fr WZG vom Anzeigetyp a) ist das Telegramm nicht zulssig. Die Nachricht wird
     * von solchen DEs mit Code 41 "Stellcodevorrat nicht nderbar" negativ quittiert.<br> Fr WZG vom Anzeigeprinzip b)
     * gibt das Byte in Antwortrichtung als Antwort auf Abruf mit Typ 21 optional die Anzahl der an der Darstellung des
     * erfragten WVZ-Codes beteiligten Lampen an. In Abrufrichtung ist das Telegramm fr WZG vom Anzeigeprinzip b) nicht
     * zulssig und wird ebenfalls mit Code 41 negativ quittiert.<br> Fr WZG vom Anzeigeprinzip c) und d) (Wechseltext)
     * gibt das Byte die Lnge des folgenden Textes an. Die Angabe der Textlnge ist unabhngig von der Anzahl der
     * tatschlich vorhandenen Textzeichen. Wenn eine geringere Anzahl Zeichen bergeben wird, wird der Rest ausgeschaltet.
     * Wird eine grere Anzahl bergeben, wird der Befehl mit Code 19 "Wechseltext nicht darstellbar, da zu lang" negativ
     * quittiert.<br> Fr WZG vom Anzeigeprinzip e) (Gruppen) gibt das Byte die Anzahl der Anzeigen/Prismen dieses DEs an.
     * Als Zuweisung sind nur DE-Blcke zulssig, die genau fr jede/jedes an diesem DE vorhandene Anzeige/Prisma eine
     * Definition enthalten. Andere DE-Blcke werden mit Code 17 negativ quittiert.<br> Fr Clusterkanle gibt das Byte die
     * Anzahl der DEs der FG 4 an diesem EAK an. Als Zuweisung sind nur DEBlcke zulssig, die genau fr jedes an diesem
     * EAK vorhandene DE eine Definition enthalten. Andere DEBlcke werden mit Code 17 negativ quittiert.<br> Der Wert 0
     * ist zulssig fr WZGs c)-e) und Cluster, um Definitionen von Stellcodes zu lschen, wenn dies nicht generell
     * ausgeschlossen wurde (Anhang 9), ansonsten werden solche Nachrichten mit Code 41 "Stellcode-Vorrat nicht nderbar"
     * negativ quittiert.<br> <br> Inhalt : Definition einer Komponente:<br> Fr WZG vom Anzeigeprinzip a) und b) (feste
     * Zeicheninhalte) ist das Telegramm in Abrufrichtung nicht zulssig. Fr aktiv leuchtende WZG kann das Telegramm in
     * Antwortrichtung (auf Abruf mit Typ 21) allerdings optional dazu genutzt werden, der Zentrale die an der Darstellung
     * eines WVZ beteiligten Lampen zu benennen. Mit dieser Information kann die Zentrale die oft geforderte berwachung
     * von Lampenbrenndauern implementieren, ohne die Zuordnung von Lampen zu Zeicheninhalten separat parametrieren zu
     * mssen. Die Definition einer Komponente hat dann je ein Byte und ist wie folgt aufgebaut:<br>
     * <pre>
     * Byte 1           Nummer der beteiligten Lampe          Nummerierung wie bei
     * Typ 3
     * </pre>
     * Fr WZG vom Anzeigeprinzip c) und d) (Wechseltext) besteht die Definition einer Komponente aus jeweils nur einem
     * Byte und ist wie folgt aufgebaut :<br>
     * <pre>
     * Byte 1           Textzeichen in ASCII-Code             siehe Anhang 7,
     * Kapitel 3
     * </pre>
     * Fr WZG vom Anzeigeprinzip e) (Gruppen) besteht die Definition einer Komponente aus jeweils 2 Bytes und ist wie
     * folgt aufgebaut:<br>
     * <pre>
     * Byte 1           WVZ-Code                              siehe Anhang 7,
     * Kapitel 1
     * Byte 2           Funktionsbyte                         siehe Definition in
     * Kapitel 6.2.18
     * </pre>
     * Der Inhalt des Funktionsbytes entspricht der verallgemeinerten Definition des Funktionsbytes in Kapitel 6.2.18.<br>
     * Fr Clusterkanle besteht die Definition einer Komponente aus jeweils 4 Bytes. Jedes DE der FG4 an dem EAK bildet
     * dabei eine Komponente. (Es ist darauf zu achten, dass sich fr EAKs, die im Programmmodus gesteuert werden knnen,
     * die Anzahl der maximal zulssigen DEs auf 53 reduziert gegenber den 56 per DEKonfiguration mglichen). Der Aufbau
     * ist wie folgt:<br>
     * <pre>
     * Byte 1           DE-Nummer der Komponente
     * Byte 2           Anzeigeprinzip dieses DEs              siehe unten
     * Byte 3           Stellcode fr die Komponente           siehe Anhang 7,
     * Kapitel 1 bzw. [ 1  254, 255 ]
     * Byte 4           Funktionsbyte                          siehe Definition in
     * Kapitel 6.2.18
     * </pre>
     * Das Byte 2 codiert das Anzeigeprinzip des DEs und ermglicht so eine eindeutige Interpretation der folgenden
     * Definitionskomponente. Es werden die gleiche Wert wie oben verwendet, nur der Wert 8 fr Clusterkanle ist hier
     * natrlich nicht zulssig.<br> Das Byte 3 hat dabei die jedem DE entsprechende Bedeutung entweder als WVZ-Code oder
     * als Stellcode einer Text- oder einer Gruppendefinition. Hier ist auch der Wert 255 zulssig, er bedeutet unabhngig
     * vom Anzeigeprinzip des DEs, dass dieses DE individuell angesteuert werden kann, auch wenn der Cluster im
     * Programmmodus gesteuert wird.<br> Der Inhalt des Funktionsbytes entspricht der verallgemeinerten Definition des
     * Funktionsbytes in Kapitel 6.2.18.<br>
     * <pre>
     * Hinweis :    Es wird empfohlen, die Definition der Codes im EAK intern
     * E/A-kanalbezogen abzulegen, um zu
     *              verhindern, dass bei genderter DE-Zuordnung alle
     * Programmdefinitionen gelscht werden
     *              mssen. Ansonsten, oder wenn die projektierten E/A-Kanle nicht
     * gleichgeblieben sind, werden
     *              bei Zuweisung einer genderten DE-Zuordnung ber die Nachricht
     * Typ 33 der
     *              Funktionsgruppe 254 alle Programmdefinitionen fr die
     * betroffenen EAKs gelscht. Der
     *              Clusterkanal meldet dann spontan mit Typ 43, dass fr ihn jetzt
     * keine Programme mehr definiert
     *              sind. Enthlt die Zuweisung der DE-Zuordnung allerdings bzgl.
     * der vorigen Zuordnungstabelle
     *              nur nderungen der DE-Nummern einer gleichbleibenden Menge von
     * E/A-Kanlen oder nur
     *              nderungen bzgl. anderer EAKs, so bleiben die
     * Programm-Definitionen erhalten.
     * </pre>
     * Es ist dabei zu beachten, dass die Stellcodes der einzelnen DEs nach Anzeigeprinzip c)-e) genauso wie trivialerweise
     * die WVZ-Codes fr Anzeigen vom Prinzip a) oder b) von den Programmnummern des EAKs vollstndig unabhngig sind.<br>
     * <br> Erhlt ein DE ein Telegramm, dessen Lnge nicht den obigen Festlegungen entspricht, so wird die Nachricht
     * negativ quittiert.<br> Die folgende Tabelle stellt bersichtlich alle Varianten des Telegramms zusammen, erlutert
     * deren Funktion und gibt an, fr welche Kombination aus Anzeige- und Steuerungsprinzip die jeweilige Verwendung
     * zulssig ist. Dabei sind Felder mit --  hinterlegt, wenn sie im betreffenden Kontext nicht relevant sind. Mit -
     * hinterlegte Felder in der Spalte "Steuerungsprinzip" bedeuten, dass die jeweiligen Stellbefehle negativ quittiert
     * werden, die Nummer in Klammern gibt den zu verwendenden Fehlercode an. Felder mit einem (zul) kennzeichnen die
     * Zulssigkeit des entsprechenden Telegramms fr das jeweilige DE.<br>
     * <pre>
     * ID   Typ   Lnge  Stellcode  n      Funktion                       Anzeige
     *    f     Steuerungsprinzip
     *                                                                    prinzip
     * <p/>
     *          1        2       3
     * 3R   44    5+n*f  1-254      0      Code-Definition lschen        a,b
     *    --    --       -(41)   --
     *                                                                    c-e, Cl
     *    --    -(45)    zul     zul
     * <p/>
     *                              1 --   Code-Definition anlegen        a,b
     *    --    --       -(41)    --
     *                              1 213  oder ndern                    c,d
     *    1     -(45)    zul     zul
     *                              1 106                                 e
     *    2     -(45)    zul     zul
     *                              1  53                                 Cl
     *    4     -(45)    zul     zul
     *                   255        0      Alle Code-Definitionen         a,b
     *    --    --       -(41)   --
     *                                     lschen                        c.e, Cl
     *    --    -(45)    zul     zul
     * 3A                1-254      0      Meldung Code gelscht          a,b
     *    --    --       --      --
     *                                     (nicht auf Abfrage)            c-e, Cl
     *    --    --       zul     zul
     *                              1 --   --                             a
     *    --    --       --      --
     *                              1 213  Meldung beteiligter Lampen     b
     *    1     --       zul     --
     *                              1 213  Meldung Code-Definiton         c,d
     *    1     --       zul     zul
     *                              1 106                                 e
     *    2     --       zul     zul
     *                              1  53                                 Cl
     *    4     --       zul     zul
     *                   255        0      Meldung alle Codes gelscht    a,b
     *    --    --       --      --
     *                                     (nicht auf Abfrage)            c.e, Cl
     *    --    --       zul     zul
     * </pre>
     * Lschen und ndern von Codedefinitionen<br> Lschen und ndern nicht aktuell oder als Grundeinstellung verwendeter
     * Codes des Clusterkanals ist zulssig und kann ohne weiteres durchgefhrt werden. Lschen oder ndern aktuell
     * anstehender Codes fhren dazu, dass der Clusterkanal in der Folge spontan vom Programmmodus in den Komponentenmodus
     * bergeht und dies spontan mit einer Nachricht Typ 55 meldet. Damit bleibt der aktuelle Anzeigezustand erhalten.
     * Entsprechendes gilt, wenn der genderte oder gelschte Code als Grundprogramm am Cluster definiert ist: In der Folge
     * gilt fr den Cluster, dass die Grundeinstellung im Komponentenmodus definiert ist. Der Cluster meldet dies spontan
     * mit einer Nachricht vom Typ 33.<br> Ein Stellcode fr ein einzelnes DE darf nicht gelscht werden, solange er in der
     * Definition eines oder mehrerer Programme seines Clusterkanals noch verwendet wird. Wenn der EAK feststellt, dass der
     * zu lschende Stellcode noch in der Definition eines seiner Programme verwendet wird, so wird der Lschbefehl mit
     * Code 47 "Lschen benutzter Stellcodes unzulssig" negativ quittiert.<br>
     *
     * @param result Der vom Datenverteiler empgangene Datensatz, der in einen De-Block konvertiert wird.
     *
     * @return De-Block mit den konvertierten Daten.
     */
    public DeBlock Fg004Typ044Id003AtgTlsWzgCodeDefinitionAspParameterSoll(ResultData result) {
        DeBlock deBlock = null;

        // Datenblock beseht aus folgenden Inhalten:
        int     _AnzeigePrinzip_;
        int     _Stellcode_       = 0;
        int     _Funktion_        = 0;
        int     _DeNummer_        = 0;
        int     count             = 0;
        TlsData deApplicationData = null;

        // Umsetzung
        Data data = result.getData();  // Datensatz holen

        _AnzeigePrinzip_ = data.getUnscaledValue("AnzeigePrinzip").intValue();
        _Stellcode_      = data.getUnscaledValue("StellCode").intValue();

        // Ermitteln, ob Langversion vorliegt...
        if ((data.getArray("TextDefinition").getLength() == 0) && (data.getArray("GruppenDefinition").getLength() == 0) && (data.getArray("ClusterDefinition").getLength() == 0)) {

            // DeBlock als TlsData aufbauen...
            deApplicationData = new TlsData(3);
            deApplicationData.writeIntAsByte(_AnzeigePrinzip_);
            deApplicationData.writeIntAsByte(_Stellcode_);
            deApplicationData.writeIntAsByte(0);
        } else {

            // Abhngig vom AnzeigePrinzip Stellcode und Funktion zuordnen
            switch (_AnzeigePrinzip_) {
            case 2 :
            case 3 : {
                count = data.getArray("TextDefinition").getLength();

                // DeBlock als TlsData aufbauen...
                deApplicationData = new TlsData(3 + count);
                deApplicationData.writeIntAsByte(_AnzeigePrinzip_);
                deApplicationData.writeIntAsByte(_Stellcode_);
                deApplicationData.writeIntAsByte(count);

                int text;

                for (int i = 0; i < count; i++) {
                    text = data.getArray("TextDefinition").getItem(i).getTextValue("Text").getText().charAt(0);
                    deApplicationData.writeIntAsByte(text);
                }
            }

            break;

            case 4 : {
                count = data.getArray("GruppenDefinition").getLength();

                // DeBlock als TlsData aufbauen...
                deApplicationData = new TlsData(3 + (count * 2));
                deApplicationData.writeIntAsByte(_AnzeigePrinzip_);
                deApplicationData.writeIntAsByte(_Stellcode_);
                deApplicationData.writeIntAsByte(count);

                for (int i = 0; i < count; i++) {
                    _Stellcode_ = data.getArray("GruppenDefinition").getItem(i).getUnscaledValue("Code").intValue();

                    // Wenn _Stellzustand_ = -1, muss Funktionsbyte auf "aus" gesetzt werden ( =0).
                    if (_Stellcode_ == -1) {
                        _Funktion_  = 0;              // ausschalten
                        _Stellcode_ = _validWvzCode;  // letzter als gltig angenommener Code!
                    } else {
                        if (data.getArray("GruppenDefinition").getItem(i).getItem("Blinken").getTextValue("Status").getText().equals("aus")) {
                            _Funktion_ = 1;           // einschalten ohne Blinken
                        } else {
                            _Funktion_ = 2;           // einschalten mit Blinken

                            // Blinkezeit auswerten.
                            if (data.getArray("GruppenDefinition").getItem(0).getItem("Blinken").getTextValue("Dauer").getText().equals("aus")) {
                                _Funktion_ &= 0x0f;   // Bit 4-7 = 0 setzten
                            } else {
                                _Funktion_ |= data.getArray("GruppenDefinition").getItem(0).getItem("Blinken").getUnscaledValue("Dauer").intValue() << 4;
                            }
                        }
                    }

                    deApplicationData.writeIntAsByte(_Stellcode_);
                    deApplicationData.writeIntAsByte(_Funktion_);
                }
            }

            case 8 : {
                count = data.getArray("ClusterDefinition").getLength();

                // DeBlock als TlsData aufbauen...
                deApplicationData = new TlsData(3 + (count * 4));
                deApplicationData.writeIntAsByte(_AnzeigePrinzip_);
                deApplicationData.writeIntAsByte(_Stellcode_);
                deApplicationData.writeIntAsByte(count);

                for (int i = 0; i < count; i++) {
                    _DeNummer_       = data.getArray("ClusterDefinition").getItem(i).getUnscaledValue("DeNummer").intValue();
                    _AnzeigePrinzip_ = data.getArray("ClusterDefinition").getItem(i).getUnscaledValue("AnzeigePrinzip").intValue();
                    _Stellcode_      = data.getArray("ClusterDefinition").getItem(i).getItem("Anzeige").getUnscaledValue("Code").intValue();

                    // Wenn _Stellzustand_ = -1, muss Funktionsbyte auf "aus" gesetzt werden ( = 0).
                    if (_Stellcode_ == -1) {
                        _Funktion_  = 0;              // ausschalten
                        _Stellcode_ = _validWvzCode;  // letzter als gltig angenommener Code!
                    } else {
                        if (data.getArray("ClusterDefinition").getItem(i).getItem("Anzeige").getItem("Blinken").getTextValue("Status").getText().equals("aus")) {
                            _Funktion_ = 1;           // einschalten ohne Blinken
                        } else {
                            _Funktion_ = 2;           // einschalten mit Blinken

                            // Blinkezeit auswerten.
                            if (data.getArray("ClusterDefinition").getItem(i).getItem("Anzeige").getItem("Blinken").getTextValue("Dauer").getText().equals("aus")) {
                                _Funktion_ &= 0x0f;   // Bit 4-7 = 0 setzten
                            } else {
                                _Funktion_ |= data.getArray("ClusterDefinition").getItem(i).getItem("Anzeige").getItem("Blinken").getUnscaledValue("Dauer").intValue() << 4;
                            }
                        }
                    }

                    deApplicationData.writeIntAsByte(_DeNummer_);
                    deApplicationData.writeIntAsByte(_AnzeigePrinzip_);
                    deApplicationData.writeIntAsByte(_Stellcode_);
                    deApplicationData.writeIntAsByte(_Funktion_);
                }
            }

            break;

            default : {
                debug.warning("Anzeigeprinzip ungltig (0-4,8) : " + _AnzeigePrinzip_ + Debug.NEWLINE + "De-Block konnte nicht erzeugt werden:" + Debug.NEWLINE + data.toString());
            }
            }
        }

        // DeBlock erzeugen
        try {
            deBlock = new DeBlock(deApplicationData, _de.getEak().getSm().getOsi7Adress(), _de.getFg(), 44, 3, _de.getDeChannel());
            debug.finest("TLS-DE-Block: " + HexDumper.toString(deBlock.getTlsDeDataBlock()));
        }
        catch (IllegalOsi7TelegramStructureException e) {
            debug.warning("Fehler beim Erzeugen eines De-Blocks zum Versand: " + e);
        }

        return deBlock;
    }

    /**
     * Entspricht {@link Fg004Default#Fg004Typ044Id003AtgTlsWzgCodeDefinitionAspParameterIst(de.bsvrz.kex.tls.osi7.DeBlock,
     * de.bsvrz.dav.daf.main.Data, de.bsvrz.dav.daf.main.DataDescription)}, jedoch werden die Daten als nachgeliefert gekennzeichnet.
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz fr die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung fr die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg004Typ044Id035AtgTlsWzgCodeDefinitionAspParameterIst(DeBlock deBlock, Data data, DataDescription dataDescription) {

        // Generierte Methode: Standardverarbeitung wie bei Id 3, aber als "nachgelieferte" Daten an den DaV bergeben.
        ResultData resultData = Fg004Typ044Id003AtgTlsWzgCodeDefinitionAspParameterIst(deBlock, data, dataDescription);

        return new ResultData(_de.getDe(), dataDescription, resultData.getDataTime(), resultData.getData(), true);
    }

    // Gieseler, BitCtrl

    /**
     * FG 4: Typ: 45 Id: 3 Name: Bilddefinition/Grafiktextkonserve (Abruf / Antwort). TLS-Datenbeschreibung:<br>
     * <pre>
     * Position         Bezeichnung                                    Erluterung
     * Byte 1           L&auml;nge DE-Block                                 [ 3 ]
     * Byte 2           Daten-Endger&auml;te-Kanal (DE)                     [ 1..254, 255 ]
     * Byte 3           Typ der DE-Daten                               [ 45 ]
     * Byte 4           Stellcode/Grafiktextkonservennummer            [ 1..254]
     * Byte 5           Lnge des BMP-Grafikdateinamens                [ 0, 4..221 ]
     * Byte 6           Name des BMP-Grafikdatenobjektes  +            ASCII-Zeichensatz
     *                  evtl. Speicherort
     * Byte 7                                                          (z.B. "C:\WVZ_231.BMP")
     * </pre>
     * Inhalt Byte: Stellcode/Grafiktextkonservennummer<br> Enth&auml;lt den Stellcode, der einer Bilddefinition
     * zugewiesen wird (Anzeigeprinzip b, frei Programmierbare LED-Pixel-Anzeigen) bzw. die
     * Grafiktextkonservennummer welche einer Grafiktextkonserve zugewiesen wird (Anzeigeprinzip d,
     * Wechseltextanzeigen mit Grafiktextkonserven).<br>
     * Inhalt Byte: Lnge des BMP-Grafikdateinamens<br>L&auml;nge des folgenden Dateinamens bzw. des
     * datei&auml;hnlichen Datenstroms der BMP-Grafikdatei. Mit der L&auml;nge = 0 wird der Stellcode
     * bzw. die Grafiktextkonservennummer gel&ouml;scht.<br>
     * Inhalt Byte: Name des BMP-Grafikdatenobjektes  + evtl. Speicherort<br> Eindeutige Bezeichnung des
     * BMP-Grafikdatenobjektes, z.B. als Dateiname mit Pfadangabe. <br>
     *
     * @param result Der vom Datenverteiler empfangene Datensatz, der in einen De-Block konvertiert wird.
     *
     * @return De-Block mit den konvertierten Daten.
     */
    public DeBlock Fg004Typ045Id003AtgTlsWzgBildDefinitionAspTlsVorgabe(ResultData result) {
        DeBlock deBlock = null;

        // Datenblock beseht aus folgenden Inhalten:
        int    _StellCode_;
        int    _Laenge_;
        String _Name_;

        // Umsetzung
        Data data = result.getData();  // Datensatz holen

        _StellCode_ = data.getUnscaledValue("StellCodeGrafikTextKonservenNummer").intValue();
        _Laenge_    = data.getUnscaledValue("LngeGrafikObjektName").intValue();
        _Name_      = data.getTextValue("GrafikObjektName").getText();

        // DeBlock als TlsData aufbauen...
        TlsData deApplicationData = new TlsData(1);

        deApplicationData.writeIntAsByte(_StellCode_);

        // falls gewnscht, hier Lnge bestimmen
        if (_Laenge_ == -1) {
            _Laenge_ = _Name_.length();

            if (_Laenge_ > 221) {
                debug.error("Lnge 'GrafikObjektName' zu gross: " + _Laenge_ + "(max. 221)");

                return deBlock;
            }
        }

        deApplicationData.writeIntAsByte(_Laenge_);
        deApplicationData.writeByteArray(_Name_.getBytes());

        // DeBlock erzeugen
        try {
            deBlock = new DeBlock(deApplicationData, _de.getEak().getSm().getOsi7Adress(), _de.getFg(), 45, 3, _de.getDeChannel());
            debug.finest("TLS-DE-Block: " + HexDumper.toString(deBlock.getTlsDeDataBlock()));
        }
        catch (IllegalOsi7TelegramStructureException e) {
            debug.warning("Fehler beim Erzeugen eines De-Blocks zum Versand: " + e);
        }

        return deBlock;
    }

    /**
     * FG 4: Typ: 45 Id: 3 Name: Bilddefinition/Grafiktextkonserve (Abruf / Antwort). TLS-Datenbeschreibung:<br>
     * {@link Fg004Default#Fg004Typ045Id003AtgTlsWzgBildDefinitionAspTlsVorgabe(de.bsvrz.dav.daf.main.ResultData)}
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz fr die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung fr die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg004Typ045Id003AtgTlsWzgBildDefinitionAspTlsZustand(DeBlock deBlock, Data data, DataDescription dataDescription) {
        ResultData resultData  = null;
        int        _StellCode_ = deBlock.getDeData().readUnsignedByteAsInt();

        data.getUnscaledValue("StellCodeGrafikTextKonservenNummer").set(_StellCode_);

        TlsData _TlsData_ = deBlock.getDeData().readNextTlsDataBlock();
        int     _Laenge_  = _TlsData_.length();

        if ((_Laenge_ > 0) && (_Laenge_ < 4)) {
            debug.error("'LngeGrafikObjektName' zu klein: " + _Laenge_ + "(min. 4)");

            return null;
        }

        data.getUnscaledValue("LngeGrafikObjektName").set(_TlsData_.length());
        data.getTextValue("GrafikObjektName").setText(new String(_TlsData_.toBytes()));
        resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);

        return resultData;
    }

    /**
     * FG 4: Typ: 45 Id: 35 Name: Bilddefinition/Grafiktextkonserve (Abruf / Antwort). TLS-Datenbeschreibung:<br> Standardverarbeitung wie bei
     * Id 3, aber als "nachgelieferte" Daten an den DaV bergeben.
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz fr die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung fr die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg004Typ045Id035AtgTlsWzgBildDefinitionAspTlsZustand(DeBlock deBlock, Data data, DataDescription dataDescription) {

        // Standardverarbeitung wie bei Id 3, aber als "nachgelieferte" Daten an den DaV bergeben.
        ResultData resultData = Fg004Typ045Id003AtgTlsWzgBildDefinitionAspTlsZustand(deBlock, data, dataDescription);

        return new ResultData(_de.getDe(), dataDescription, resultData.getDataTime(), resultData.getData(), true);
    }

    // Ende Gieseler, BitCtrl

    /**
     * FG 4 : Typ: 48 Id: 5 und Id 37 Name: WVZ-Stellzustand (Abruf / Antwort). TLS-Datenbeschreibung:<br>
     * <pre>
     * Position         Bezeichnung                             Erluterung
     * Byte 1           Lnge DE-Block                          Lnge des folgenden
     * DE-Blocks [ 4 ]
     * Byte 2           Daten-Endgerte-Kanal (DE)              [ 1..254, 255 ]
     * Byte 3           Typ der DE-Daten                        [ 48 ]
     * Byte 4           Stellzustand: Code                      anzuzeigendes WVZ
     * Byte 5           Funktionsbyte                           Ein / Aus 
     * Blinken
     * </pre>
     * Inhalt Byte Stellzustand:<br> Codierung von Verkehrszeichen: (WVZ-Code): Siehe Anhang 7, Kapitel 1.<br> <br> Inhalt
     * Funktionsbyte:
     * <pre>
     *    7        6        5        4        3        2        1        0
     *               Blinkzeit                            Funktion
     * Bit 0..3:     Funktion:
     *               0 = ausschalten
     *               1 = einschalten
     *               2 = blinken (Die Einstellung "WVZ blinkend" ist eine optionale
     * Eigenschaft.)
     *               3-15 reserviert
     * <p/>
     * Bit 4..7:     Blinkzeit in 200 ms (1..3 s)
     *               Die Blinkzeit ist die gesamte Periodendauer, d.h. die Summe
     * aus Ein- und Auszeit. Das Verhltnis Ein-
     *               /Auszeit wird nach Vorgabe des Anlagenbetreibers am WZG
     * eingestellt.
     *               Eine sinnvolle Abstimmung der Periodendauern fr verschiedene
     * WVZ ist Aufgabe der Zentrale.
     * </pre>
     *
     * @param result Der vom Datenverteiler empgangene Datensatz, der in einen De-Block konvertiert wird.
     *
     * @return De-Block mit den konvertierten Daten.
     */
    public DeBlock Fg004Typ048Id005AtgTlsWzgWvzStellZustandAspTlsVorgabe(ResultData result) {
        DeBlock deBlock = null;

        // Datenblock beseht aus folgenden Inhalten:
        int _Stellzustand_;
        int _Funktion_ = 0;

        // Umsetzung
        Data data = result.getData();  // Datensatz holen

        _Stellzustand_ = data.getUnscaledValue("WvzCode").intValue();

        // Wenn _Stellzustand_ = -1, muss Funktionsbyte auf "aus" gesetzt werden ( = 0).
        if (_Stellzustand_ == -1) {
            _Funktion_     = 0;              // ausschalten
            _Stellzustand_ = _validWvzCode;  // letzter als gltig angenommener Code!
        } else {
            if (data.getItem("Blinken").getTextValue("Status").getText().equals("aus")) {
                _Funktion_ = 1;              // einschalten ohne Blinken
            } else {
                _Funktion_ = 2;              // einschalten mit Blinken

                // Blinkezeit auswerten.
                if (data.getItem("Blinken").getTextValue("Dauer").getText().equals("aus")) {
                    _Funktion_ &= 0x0f;      // Bit 4-7 = 0 setzten
                } else {
                    _Funktion_ |= data.getItem("Blinken").getUnscaledValue("Dauer").intValue() << 4;
                }
            }
        }

        // DeBlock als TlsData aufbauen...
        TlsData deApplicationData = new TlsData(2);

        deApplicationData.writeIntAsByte(_Stellzustand_);
        deApplicationData.writeIntAsByte(_Funktion_);

        // DeBlock erzeugen
        try {
            deBlock = new DeBlock(deApplicationData, _de.getEak().getSm().getOsi7Adress(), _de.getFg(), 48, 5, _de.getDeChannel());
            debug.finest("TLS-DE-Block: " + HexDumper.toString(deBlock.getTlsDeDataBlock()));
        }
        catch (IllegalOsi7TelegramStructureException e) {
            debug.warning("Fehler beim Erzeugen eines De-Blocks zum Versand: " + e);
        }

        return deBlock;
    }

    /**
     * FG 4 : Typ: 48 Id: 5 und Id 37 Name: WVZ-Stellzustand (Abruf / Antwort). TLS-Datenbeschreibung:<br>
     * <pre>
     * Position         Bezeichnung                             Erluterung
     * Byte 1           Lnge DE-Block                          Lnge des folgenden
     * DE-Blocks [ 4 ]
     * Byte 2           Daten-Endgerte-Kanal (DE)              [ 1..254, 255 ]
     * Byte 3           Typ der DE-Daten                        [ 48 ]
     * Byte 4           Stellzustand: Code                      anzuzeigendes WVZ
     * Byte 5           Funktionsbyte                           Ein / Aus 
     * Blinken
     * </pre>
     * Inhalt Byte Stellzustand:<br> Codierung von Verkehrszeichen: (WVZ-Code): Siehe Anhang 7, Kapitel 1.<br> <br> Inhalt
     * Funktionsbyte:
     * <pre>
     *    7        6        5        4        3        2        1        0
     *               Blinkzeit                            Funktion
     * Bit 0..3:     Funktion:
     *               0 = ausschalten
     *               1 = einschalten
     *               2 = blinken (Die Einstellung "WVZ blinkend" ist eine optionale
     * Eigenschaft.)
     *               3-15 reserviert
     * <p/>
     * Bit 4..7:     Blinkzeit in 200 ms (1..3 s)
     *               Die Blinkzeit ist die gesamte Periodendauer, d.h. die Summe
     * aus Ein- und Auszeit. Das Verhltnis Ein-
     *               /Auszeit wird nach Vorgabe des Anlagenbetreibers am WZG
     * eingestellt.
     *               Eine sinnvolle Abstimmung der Periodendauern fr verschiedene
     * WVZ ist Aufgabe der Zentrale.
     * </pre>
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz fr die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung fr die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg004Typ048Id005AtgTlsWzgWvzStellZustandAspTlsZustand(DeBlock deBlock, Data data, DataDescription dataDescription) {
        ResultData resultData = null;

        // Betriebsart aus Telegramm -----------------------------------------------------------------------
        int _Stellzustand_ = deBlock.getDeData().readUnsignedByteAsInt();
        if (_Stellzustand_ != 0) // Stellzustand nicht bernehmen, wenn 0
        {

        _validWvzCode = _Stellzustand_;


        }

        int _Funktion_ = deBlock.getDeData().readUnsignedByteAsInt();

        /*
         * Start Erweiterung STS 2013.02
         */

        // Workaround fr ltere Anlagen, die den Zustand 'dunkel' (ausgeschaltet) mit Code 0 und Funktion 1 melden.
        if ((_Stellzustand_ == 0) && (_Funktion_ == 1))
        {
          _Funktion_ = 0;
        }

        /*
         * Ende Erweiterung STS 2013.02
         */

        // Wenn Funktion = ausgeschaltet, hat WVZ-Code keine Bedeutung. Wird deshalb fr
        // Protokollausgaben auf -1 = "aus" gesetzt.
        if ((_Funktion_ & 0x0f) == 0) {
            _Stellzustand_ = -1;
        }

        data.getUnscaledValue("WvzCode").set(_Stellzustand_);

        if ((_Funktion_ & 0x02) == 0) {
            data.getItem("Blinken").getUnscaledValue("Status").setText("aus");
            data.getItem("Blinken").getUnscaledValue("Dauer").setText("aus");
        } else {
            data.getItem("Blinken").getUnscaledValue("Status").setText("ein");

            int _BlinkDauer_ = _Funktion_ >>> 4;

            if ((_BlinkDauer_ < 5) || (_BlinkDauer_ > 15)) {
                debug.warning("Blinkdauer ungltig (5-15) : " + (_Funktion_ >>> 4) + ". Blinkdauerwert wurde auf 5 gesetzt!");
                _BlinkDauer_ = 5;
            }

            data.getItem("Blinken").getUnscaledValue("Dauer").set(_BlinkDauer_);
        }

        // Ergebnisdatensatz erzeugen...
        resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);

        return resultData;
    }

    /**
     * FG 4 : Typ: 48 Id. 37 Name: WVZ-Stellzustand (Abruf / Antwort). TLS-Datenbeschreibung:<br> Standardverarbeitung wie
     * bei Id %, aber als "nachgelieferte" Daten an den DaV bergeben.
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz fr die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung fr die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg004Typ048Id037AtgTlsWzgWvzStellZustandAspTlsZustand(DeBlock deBlock, Data data, DataDescription dataDescription) {

        // Standardverarbeitung wie bei Id 2, aber als "nachgelieferte" Daten an den DaV bergeben.
        ResultData resultData = Fg004Typ048Id005AtgTlsWzgWvzStellZustandAspTlsZustand(deBlock, data, dataDescription);

        if (resultData != null) {
            return new ResultData(_de.getDe(), dataDescription, resultData.getDataTime(), resultData.getData(), true);
        } else {
            return resultData;
        }
    }

    /**
     * FG 4 : Typ: 49 Id: 5  Name: Helligkeit (Abruf / Antwort). TLS-Datenbeschreibung:<br>
     * <pre>
     * Position         Bezeichnung                                    Erluterung
     * Byte 1           Lnge DE-Block                                 Lnge des
     * folgenden DE-Blocks [ 4 ]
     * Byte 2           Daten-Endgerte-Kanal (DE)                     [193..222,
     * 223, (255)1 ]
     * Byte 3           Typ der DE-Daten                               [ 49 ]
     * Byte 4           Helligkeitswert                                [ 0..100
     * Prozent ]
     * Byte 5           Helligkeitsstatusbyte                          siehe unten
     * 1) siehe Anhang 6, Teil 1, Kapitel 1.4
     * </pre>
     * Inhalt Byte: Helligkeitswert:<br> Der Helligkeitswert fr WZGs wird unabhngig von dem Anzeigeprinzip des WZG als
     * eine Prozentzahl von 0 bis 100 bertragen. WZGs stellen die dem Prozentwert am nchsten liegende Helligkeitsstufe
     * ein. Die hierbei geltenden lichttechnischen Anforderungen werden in den "vorlufigen lichttechnischen Festlegungen
     * fr WVZ und Fahrstreifensignalgeber nach dem Lichtrasterprinzip" der BASt beschrieben.<br> In Nachrichten in
     * Abrufrichtung, die im Helligkeitsstatusbyte die automatische Helligkeitsteuerung einschalten, ist der
     * Helligkeitswert nicht definiert.<br> <br> Inhalt Helligkeitsstatusbyte:<br>
     * <pre>
     *    7        6        5        4        3        2        1        0
     *    X        X        X        X        X        X        Meld.    Autom.
     * <p/>
     * Bit 0:   Automatik:
     *          0 = Helligkeitswert wird durch die Zentrale ferneingestellt
     *          1 = automatische, lokale Helligkeitssteuerung (optional, s.o.)
     * Bit 1:   Meldungen im Automatikbetrieb:
     *          0 = keine spontanen Meldungen des aktuellen Helligkeitswertes
     *          1 = nach jeder nderung des eingestellten Helligkeitswertes
     * spontane Meldung senden
     * <pre>
     *
     * @param result Der vom Datenverteiler empgangene Datensatz, der in einen De-Block konvertiert wird.
     *
     * @return De-Block mit den konvertierten Daten.
     */
    public DeBlock Fg004Typ049Id005AtgTlsWzgHelligkeitAspTlsVorgabe(ResultData result) {
        DeBlock deBlock = null;

        // Datenblock beseht aus folgenden Inhalten:
        int _Helligkeitswert_;
        int _Helligkeitsstatusbyte_;

        // Umsetzung
        Data data = result.getData();  // Datensatz holen

        _Helligkeitswert_       = data.getUnscaledValue("HelligkeitsWert").intValue();
        _Helligkeitsstatusbyte_ = data.getUnscaledValue("Status").intValue();

        // DeBlock als TlsData aufbauen...
        TlsData deApplicationData = new TlsData(2);

        deApplicationData.writeIntAsByte(_Helligkeitswert_);
        deApplicationData.writeIntAsByte(_Helligkeitsstatusbyte_);

        // DeBlock erzeugen
        try {
            deBlock = new DeBlock(deApplicationData, _de.getEak().getSm().getOsi7Adress(), _de.getFg(), 49, 5, _de.getDeChannel());
            debug.finest("TLS-DE-Block: " + HexDumper.toString(deBlock.getTlsDeDataBlock()));
        }
        catch (IllegalOsi7TelegramStructureException e) {
            debug.warning("Fehler beim Erzeugen eines De-Blocks zum Versand: " + e);
        }

        return deBlock;
    }

    /**
     * FG 4 : Typ: 49 Id: 5  Name: Helligkeit (Abruf / Antwort). TLS-Datenbeschreibung:<br>
     * <pre>
     * Position         Bezeichnung                                    Erluterung
     * Byte 1           Lnge DE-Block                                 Lnge des
     * folgenden DE-Blocks [ 4 ]
     * Byte 2           Daten-Endgerte-Kanal (DE)                     [193..222,
     * 223, (255)1 ]
     * Byte 3           Typ der DE-Daten                               [ 49 ]
     * Byte 4           Helligkeitswert                                [ 0..100
     * Prozent ]
     * Byte 5           Helligkeitsstatusbyte                          siehe unten
     * 1) siehe Anhang 6, Teil 1, Kapitel 1.4
     * </pre>
     * Inhalt Byte: Helligkeitswert:<br> Der Helligkeitswert fr WZGs wird unabhngig von dem Anzeigeprinzip des WZG als
     * eine Prozentzahl von 0 bis 100 bertragen. WZGs stellen die dem Prozentwert am nchsten liegende Helligkeitsstufe
     * ein. Die hierbei geltenden lichttechnischen Anforderungen werden in den "vorlufigen lichttechnischen Festlegungen
     * fr WVZ und Fahrstreifensignalgeber nach dem Lichtrasterprinzip" der BASt beschrieben.<br> In Nachrichten in
     * Abrufrichtung, die im Helligkeitsstatusbyte die automatische Helligkeitsteuerung einschalten, ist der
     * Helligkeitswert nicht definiert.<br> <br> Inhalt Helligkeitsstatusbyte:<br>
     * <pre>
     *    7        6        5        4        3        2        1        0
     *    X        X        X        X        X        X        Meld.    Autom.
     * <p/>
     * Bit 0:   Automatik:
     *          0 = Helligkeitswert wird durch die Zentrale ferneingestellt
     *          1 = automatische, lokale Helligkeitssteuerung (optional, s.o.)
     * Bit 1:   Meldungen im Automatikbetrieb:
     *          0 = keine spontanen Meldungen des aktuellen Helligkeitswertes
     *          1 = nach jeder nderung des eingestellten Helligkeitswertes
     * spontane Meldung senden
     * <pre>
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz fr die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung fr die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg004Typ049Id005AtgTlsWzgHelligkeitAspTlsZustand(DeBlock deBlock, Data data, DataDescription dataDescription) {
        ResultData resultData = null;

        // Helligkeitswert aus Telegramm -------------------------------------------------------------------------------
        int _Helligkeitswert_ = deBlock.getDeData().readUnsignedByteAsInt();

        if ((_Helligkeitswert_ >= 0) && (_Helligkeitswert_ <= 100)) {
            data.getUnscaledValue("HelligkeitsWert").set(_Helligkeitswert_);
        } else {
            debug.warning("Helligkeitswert ungltig : " + _Helligkeitswert_ + Debug.NEWLINE + "De-Block (Osi7Adresse-DeNummer-FG-Typ-Id): " + deBlock.getLongKey() + " verworfen:" + Debug.NEWLINE + deBlock.toString());

            return resultData;
        }

        // Helligkeitswert aus Telegramm -------------------------------------------------------------------------------
        int _Helligkeitsstatusbyte_ = deBlock.getDeData().readUnsignedByteAsInt();

        data.getUnscaledValue("Status").set(_Helligkeitsstatusbyte_ & 0x03);

        // Ergebnisdatensatz erzeugen...
        resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);

        return resultData;
    }

    /**
     * FG 4 : Typ: 49 Id: 37 Name: Helligkeit (Abruf / Antwort). TLS-Datenbeschreibung:<br> Standardverarbeitung wie bei Id
     * 5, aber als "nachgelieferte" Daten an den DaV bergeben.
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz fr die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung fr die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg004Typ049Id037AtgTlsWzgHelligkeitAspTlsZustand(DeBlock deBlock, Data data, DataDescription dataDescription) {

        // Standardverarbeitung wie bei Id 2, aber als "nachgelieferte" Daten an den DaV bergeben.
        ResultData resultData = Fg004Typ049Id005AtgTlsWzgHelligkeitAspTlsZustand(deBlock, data, dataDescription);

        if (resultData != null) {
            return new ResultData(_de.getDe(), dataDescription, resultData.getDataTime(), resultData.getData(), true);
        } else {
            return resultData;
        }
    }

    /**
     * FG 4 : Typ: 50 Id: 5 und Id 37 Name: Wechseltext (Abruf / Antwort). TLS-Datenbeschreibung:<br>
     * <pre>
     * Position         Bezeichnung                             Erluterung
     * Byte 1           Lnge DE-Block                          Lnge des folgenden
     * DE-Blocks [ 4 + Textl. ]
     * Byte 2           Daten-Endgerte-Kanal (DE)              [ 1..254, 255 ]
     * Byte 3           Typ der DE-Daten                        [ 50 ]
     * Byte 4           Funktionsbyte                           Ein / Aus 
     * Blinken
     * Byte 5           Anzahl Textzeichen                      [ 0..220 ]
     * Byte 6           1. Textzeichen in ASCII-Code            angezeigter 
     * anzuzeigender Text
     * Byte 7           2. Textzeichen in ASCII-Code            (siehe Anhang 7,
     * Kap.3)
     * ......
     * Byte ..          n. Textzeichen in ASCII-Code
     * </pre>
     * Inhalt Funktionsbyte:<br> Codierung von Verkehrszeichen: (WVZ-Code): Siehe Anhang 7, Kapitel 1.<br>
     * <pre>
     *    7        6        5        4        3        2        1        0
     *               Blinkzeit                            Funktion
     * Bit 0..3:     Funktion:
     *               0 = ausschalten
     *               1 = einschalten
     *               2 = blinken (Die Einstellung "WVZ blinkend" ist eine optionale
     * Eigenschaft.)
     *               3-15 reserviert
     * <p/>
     * Bit 4..7:     Blinkzeit in 200 ms (1..3 s)
     *               Die Blinkzeit ist die gesamte Periodendauer, d.h. die Summe
     * aus Ein- und Auszeit. Das Verhltnis Ein-
     *               /Auszeit wird nach Vorgabe des Anlagenbetreibers am WZG
     * eingestellt.
     *               Eine sinnvolle Abstimmung der Periodendauern fr verschiedene
     * WVZ ist Aufgabe der Zentrale.
     * </pre>
     *
     * @param result Der vom Datenverteiler empgangene Datensatz, der in einen De-Block konvertiert wird.
     *
     * @return De-Block mit den konvertierten Daten.
     */
    public DeBlock Fg004Typ050Id005AtgTlsWzgWechselTextAspTlsVorgabe(ResultData result) {
        DeBlock deBlock = null;

        // Datenblock beseht aus folgenden Inhalten:
        int _Funktion_ = 0;

        // Umsetzung
        Data   data  = result.getData();  // Datensatz holen
        int    count = data.getTextValue("WVZTextZeichen").getText().length();
        String text  = data.getTextValue("WVZTextZeichen").getText();

        // Wenn Lnge = 0, muss Funktionsbyte auf "aus" gesetzt werden
        if (count == 0) {
            _Funktion_ = 0;              // ausschalten
        } else {
            if (data.getItem("Blinken").getTextValue("Status").getText().equals("aus")) {
                _Funktion_ = 1;          // einschalten ohne Blinken
            } else {
                _Funktion_ = 2;          // einschalten mit Blinken

                // Blinkezeit auswerten.
                if (data.getItem("Blinken").getTextValue("Dauer").getText().equals("aus")) {
                    _Funktion_ &= 0x0f;  // Bit 4-7 = 0 setzten
                } else {
                    _Funktion_ |= data.getItem("Blinken").getUnscaledValue("Dauer").intValue() << 4;
                }
            }
        }

        // DeBlock als TlsData aufbauen...
        TlsData deApplicationData = new TlsData(1 + count);

        deApplicationData.writeIntAsByte(_Funktion_);
        deApplicationData.writeIntAsByte(count);
        deApplicationData.writeByteArray(text.getBytes());

        // DeBlock erzeugen
        try {
            deBlock = new DeBlock(deApplicationData, _de.getEak().getSm().getOsi7Adress(), _de.getFg(), 50, 5, _de.getDeChannel());
            debug.finest("TLS-DE-Block: " + HexDumper.toString(deBlock.getTlsDeDataBlock()));
        }
        catch (IllegalOsi7TelegramStructureException e) {
            debug.warning("Fehler beim Erzeugen eines De-Blocks zum Versand: " + e);
        }

        return deBlock;
    }

    /**
     * FG 4 : Typ: 50 Id: 5 und Id 37 Name: Wechseltext (Abruf / Antwort). TLS-Datenbeschreibung:<br>
     * <pre>
     * Position         Bezeichnung                             Erluterung
     * Byte 1           Lnge DE-Block                          Lnge des folgenden
     * DE-Blocks [ 4 + Textl. ]
     * Byte 2           Daten-Endgerte-Kanal (DE)              [ 1..254, 255 ]
     * Byte 3           Typ der DE-Daten                        [ 50 ]
     * Byte 4           Funktionsbyte                           Ein / Aus 
     * Blinken
     * Byte 5           Anzahl Textzeichen                      [ 0..220 ]
     * Byte 6           1. Textzeichen in ASCII-Code            angezeigter 
     * anzuzeigender Text
     * Byte 7           2. Textzeichen in ASCII-Code            (siehe Anhang 7,
     * Kap.3)
     * ......
     * Byte ..          n. Textzeichen in ASCII-Code
     * </pre>
     * Inhalt Funktionsbyte:<br> Codierung von Verkehrszeichen: (WVZ-Code): Siehe Anhang 7, Kapitel 1.<br>
     * <pre>
     *    7        6        5        4        3        2        1        0
     *               Blinkzeit                            Funktion
     * Bit 0..3:     Funktion:
     *               0 = ausschalten
     *               1 = einschalten
     *               2 = blinken (Die Einstellung "WVZ blinkend" ist eine optionale
     * Eigenschaft.)
     *               3-15 reserviert
     * <p/>
     * Bit 4..7:     Blinkzeit in 200 ms (1..3 s)
     *               Die Blinkzeit ist die gesamte Periodendauer, d.h. die Summe
     * aus Ein- und Auszeit. Das Verhltnis Ein-
     *               /Auszeit wird nach Vorgabe des Anlagenbetreibers am WZG
     * eingestellt.
     *               Eine sinnvolle Abstimmung der Periodendauern fr verschiedene
     * WVZ ist Aufgabe der Zentrale.
     * </pre>
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz fr die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung fr die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg004Typ050Id005AtgTlsWzgWechselTextAspTlsZustand(DeBlock deBlock, Data data, DataDescription dataDescription) {
        ResultData resultData = null;

        // Funktionsbyte aus Telegramm ---------------------------------------------------------------------------------
        int _Funktion_ = deBlock.getDeData().readUnsignedByteAsInt();

        if ((_Funktion_ & 0x02) == 0) {
            data.getItem("Blinken").getUnscaledValue("Status").setText("aus");
            data.getItem("Blinken").getUnscaledValue("Dauer").setText("aus");
        } else {
            data.getItem("Blinken").getUnscaledValue("Status").setText("ein");

            int _BlinkDauer_ = _Funktion_ >>> 4;

            if ((_BlinkDauer_ < 5) || (_BlinkDauer_ > 15)) {
                debug.warning("Blinkdauer ungltig (5-15) : " + (_Funktion_ >>> 4) + ". Blinkdauerwert wurde auf 5 gesetzt!");
                _BlinkDauer_ = 5;
            }

            data.getItem("Blinken").getUnscaledValue("Dauer").set(_BlinkDauer_);
        }

        // Wenn Funktion = ausgeschaltet, hat WVZ-Code keine Bedeutung. Wird deshalb fr
        // Protokollausgaben auf leeren String gesetzt.
        data.getTextValue("WVZTextZeichen").setText("");

        if ((_Funktion_ & 0x0f) != 0) {
            int          count         = deBlock.getDeData().readUnsignedByteAsInt();
            StringBuffer _AnzeigeText_ = new StringBuffer();

            for (int i = 0; i < count; i++) {
                _AnzeigeText_.append(String.format("%c", deBlock.getDeData().readUnsignedByteAsInt()));
            }

            data.getTextValue("WVZTextZeichen").setText(_AnzeigeText_.toString());
        }

        // Ergebnisdatensatz erzeugen...
        resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);

        return resultData;
    }

    /**
     * FG 4 : Typ: 50 Id: 37 Name: Wechseltext (Abruf / Antwort). TLS-Datenbeschreibung:<br> Standardverarbeitung wie bei
     * Id 5, aber als "nachgelieferte" Daten an den DaV bergeben.
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz fr die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung fr die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg004Typ050Id037AtgTlsWzgWechselTextAspTlsZustand(DeBlock deBlock, Data data, DataDescription dataDescription) {

        // Standardverarbeitung wie bei Id 5, aber als "nachgelieferte" Daten an den DaV bergeben.
        ResultData resultData = Fg004Typ050Id005AtgTlsWzgWechselTextAspTlsZustand(deBlock, data, dataDescription);

        return new ResultData(_de.getDe(), dataDescription, resultData.getDataTime(), resultData.getData(), true);
    }

    /**
     * FG 4 : Typ: 55 Id: 5 Name: Stellzustand (Abruf / Antwort). TLS-Datenbeschreibung:<br>
     * <pre>
     * Position         Bezeichnung                                    Erluterung
     * Byte 1           Lnge DE-Block                                 [5 | 6+f*n]
     * Byte 2           Daten-Endgerte-Kanal (DE)                     [ 1..254 ]
     * Byte 3           Typ der DE-Daten                               [ 55 ]
     * Byte 4           Anzeigeprinzip                                 siehe unten
     * Byte 5           Stellcode
     * anzuzeigendes WVZ/Programm
     * Byte 6           Funktionsbyte                                  siehe unten
     * Byte 7           n : Anzahl Komponenten                         1 -[213 |
     * 106] siehe unten
     * f Bytes...       Stellzustand der 1.Komponente                  siehe unten
     * f Bytes          Stellzustand der n.Komponente
     * </pre>
     * Inhalt Byte : Anzeigeprinzip<br> Das Byte gibt das Anzeigeprinzip des adressierten/sendenden DEs an und ermglicht
     * damit die genaue Interpretation des folgenden Telegramminhalts. Es ist folgendermaen codiert:<br>
     * <pre>
     * Anzeigeprinzip        Wert
     *              a        0
     *              b        1
     *              c        2
     *              d        3
     *              e        4
     *        Cluster        8
     * </pre>
     * Erhlt ein DE in Abrufrichtung eine Nachricht mit einem Wert, der nicht seinem Anzeigeprinzip entspricht, so wird
     * die Nachricht mit Code 21 falsches Anzeigeprinzip negativ quittiert.<br> <br> Inhalt Byte: Stellcode<br> Der
     * Inhalt des Bytes ist abhngig davon, ob das adressierte DE ein Clusterkanal ist bzw. vom Anzeigeprinzip des
     * DEs:<br>
     * <pre>
     * Clusterkanle :   Nummer des Programms, das eingestellt werden soll. Ist der
     * Wert 0, so behalten alle DEs
     *                   den Zustand , den sie zuletzt hatten. In der Folge
     * akzeptieren diese DEs individuell an sie
     *                   gerichtete Stellbefehle und ndern ihren Zustand
     * entsprechend. Ist als Wert eine
     *                   Programmnummer zwischen 1 und 254 angegeben, so werden
     * alle DEs entsprechend der
     *                   Definition dieses Programms eingestellt. Ausnahmen hiervon
     * bilden lediglich DEs, fr
     *                   welche die entsprechende Programmdefinition der Wert 255
     * "dont care" enthlt. Diese
     *                   DEs behalten ihren Zustand und sind in der Folge durch
     * individuelle Stellbefehle steuerbar.
     *                   Der Wert 255 ist nicht zulssig und wird mit Code 43
     * negativ quittiert.
     * Anzeigeprinzip a und b :   WVZ-Code entsprechend Anhang 7, Kapitel 1. Gibt
     * den WVZ-Code an, den das
     *                   DE darstellen soll. Die Nachricht wird nur akzeptiert,
     * wenn der Stellcode des Clusterkanals
     *                   den Wert 0 enthlt oder die Definition des am Cluster
     * eingestellten Programms fr dieses
     *                   DE den Wert 255 "dont care" angibt. In allen anderen
     * Fllen wird der Stellbefehl mit Code
     *                   10 "Befehl bzw. Programm in dieser Betriebsart nicht
     * ausfhrbar" negativ quittiert. Der
     *                   Wert 0 ist fr DEs dieser Anzeigeprinzipien nicht zulssig
     * und wird ebenfalls negativ
     *                   quittiert.
     * Anzeigeprinzip c und d :   Stellcode des vordefinierten Textes, den das DE
     * darstellen soll. Die Nachricht
     *                   wird nur akzeptiert, wenn der eingestellte Stellcode des
     * Clusterkanals den Wert 0 enthlt
     *                   oder die Definition des am Cluster eingestellten Programms
     * fr dieses DE den Wert 255
     *                   "dont care" angibt. In allen anderen Fllen wird der
     * Stellbefehl mit Code 10 "Befehl bzw.
     *                   Programm in dieser Betriebsart nicht ausfhrbar" negativ
     * quittiert. Ist der Wert 0, so wird
     *                   das Telegramm in der Langversion bertragen und enthlt
     * den darzustellenden Text als
     *                   ASCII-String.
     * Anzeigeprinzip e :   Stellcode der vordefinierten Gruppendefinition, die das
     * DE darstellen soll. Die
     *                   Nachricht wird nur akzeptiert, wenn der eingestellte
     * Stellcode des Clusterkanals den Wert 0
     *                   enthlt oder die Definition des am Cluster eingestellten
     * Programms fr dieses DE den Wert
     *                   255 "dont care" angibt. In allen anderen Fllen wird der
     * Stellbefehl mit Code 10 "Befehl
     *                   bzw. Programm in dieser Betriebsart nicht ausfhrbar"
     * negativ quittiert. Ist der Wert 0, so
     *                   wird das Telegramm in der Langversion bertragen und
     * enthlt fr jede Komponente den
     *                   entsprechenden Stellzustand.
     * </pre>
     * Zulssige Werte:
     * <pre>
     *                       Cluster                      a,b
     * c,d,e
     * Steuerungsprinzip 1   0                            --
     * 0
     * Steuerungsprinzip 2   1-254                        Anhang 7, Kap. 1
     * 1-254
     * Steuerungsprinzip 3   0,1-254                      --
     * 0,1-254
     * </pre>
     * Inhalt Funktionsbyte:<br>
     * <pre>
     *    7        6        5        4        3        2        1        0
     *               Blinkzeit                      Status       Funktion
     * </pre>
     * Funktion
     * <pre>
     * Bit    Wert   Bedeutung                           Kommentar
     * 0..1   0      ausschalten                         nur fr DEs nach
     * Anzeigeprinzip b und d
     *        1      einschalten                         sowie Blinker nach
     * Anzeigeprinzip a
     *        2      blinken                             nur bei DEs nach
     * Anzeigeprinzip b und d, optional
     *        3      reserviert
     * DEs, die in Abrufrichtung ein Funktionsbyte mit einem Wert erhalten, der
     * gem ihrem Anzeigeprinzip
     * nicht zulssig ist, antworten mit einer negativen Quittung, Code 14
     * "Funktionsbyte unzulssig".
     * </pre>
     * Status (nur in Antwortrichtung):
     * <pre>
     * Bit    Wert   Bedeutung                           Kommentar
     * 2      0      kein Fehler aufgetreten
     *        1      Fehler aufgetreten
     * 3      2      Programm abgeschlossen              wird nur verwendet von
     * Clusterkanlen und DEs, die
     *        3      Programm noch nicht abgeschlossen   Programmschaltungen
     * ausfhren
     * Blinkzeit<br>
     * <pre>
     * Bit    Wert   Bedeutung                           Kommentar
     * 4..7   5-15   Blinkzeit in 200ms (1..3 s)         nur fr DEs nach
     * Anzeigeprinzip b und d
     *        0                                          wenn Funktion ungleich
     * "blinken"
     * Die Blinkzeit ist die gesamte Periodendauer, d.h. die Summe aus Ein- und
     * Auszeit. Das Verhltnis Ein-
     * /Auszeit wird nach Vorgabe des Anlagenbetreibers am WZG eingestellt.
     * Eine sinnvolle Abstimmung der Periodendauern fr verschiedene WVZ ist
     * Aufgabe der Zentrale.
     * </pre>
     * Inhalt Anzahl Komponenten :
     * Wird die Nachricht in der Langversion versendet (nur bei Stellcode gleich
     * 0), so gibt das Byte die Anzahl der
     * Komponenten an, fr die in der Folge Stellzustnde bermittelt werden.
     * Die Langversion ist nur fr WZG nach Anzeigeprinzip c)-e) zulssig, fr
     * Anzeigen nach Prinzip a) und b) ist
     * sie nicht anwendbar und fr Clusterkanle werden individuelle Stellbefehle
     * an die jeweiligen DEs direkt
     * adressiert.
     * Inhalt : Stellzustand einer Komponente:
     * Fr WZG vom Anzeigeprinzip a) und b) (feste Zeicheninhalte) ist das
     * Telegramm in der Langversion nicht
     * zulssig, da es bedeutungslos wre.
     * Fr WZG vom Anzeigeprinzip c) und d) (Wechseltext) besteht der Stellzustand
     * einer Komponente aus jeweils
     * nur einem Byte und ist wie folgt aufgebaut :
     * <pre>
     * Byte 1            Textzeichen in ASCII-Code            siehe Anhang 7,
     * Kapitel 3
     * </pre>
     * Fr WZG vom Anzeigeprinzip e) (Gruppen) besteht der Stellzustand einer
     * Komponente aus jeweils 2 Bytes
     * und ist wie folgt aufgebaut:<br>
     * <pre>
     * Byte 1            WVZ-Code                             siehe Anhang 7,
     * Kapitel 1
     * Byte 2            Funktionsbyte                        siehe oben
     * </pre>
     * Das Funktionsbyte ist definiert wie oben. Komponenten nach Anzeigeprinzip a)
     * melden in Antwortrichtung
     * den Sachverhalt Strung erkannt - Prisma nicht bewegbar / Blinker defekt
     * mit gesetztem Bit 2 im
     * Funktionsbyte. Kann der eingestellte WVZ-Code vom Gert noch ermittelt
     * werden, so wird er in Byte 1
     * bertragen. Ansonsten wird der WVZ-Code 245 undefinierte Stellung
     * gemeldet.
     * Fr Clusterkanle ist die Nachricht in der Langversion nicht zulssig, da
     * die Stellbefehle an die einzelnen DEs
     * direkt adressiert werden knnen.<br>
     * <br>
     * Die folgende Tabelle stellt bersichtlich alle Varianten des Telegramms
     * zusammen, erlutert deren Funktion und gibt
     * an, fr welche Kombination aus Anzeige und Steuerungsprinzip die jeweilige
     * Verwendung zulssig ist. Dabei sind
     * Felder -- gekennzeichnet, wenn sie im betreffenden Kontext nicht relevant
     * sind. Mit - gekennzeichnete Felder in der
     * Spalte "Steuerungsprinzip" bedeuten, dass die jeweiligen Stellbefehle
     * negativ quittiert werden, die Nummer in
     * Klammern gibt den zu verwendenden Fehlercode an. Felder mit zul kennzeichnen
     * die Zulssigkeit des
     * entsprechenden Telegramms fr das jeweilige DE. Eine hinter zul angefgte
     * Nummer  verweist auf
     * einen Telegrammtyp aus frheren Versionen der TLS, die bei Altlagen im
     * jeweiligen Kontext noch zulssig sind.<br>
     * <pre>
     * ID   Typ   Lnge  Stellcode  n      Funktion                       Anzeige
     *    f     Steuerungsprinzip
     *                                                                    prinzip
     * <p/>
     *          1        2       3
     * 5R   55    5      1-254      --     Stellbefehl mit Codenummer     a
     *    --    --       zul     --
     *                              --                                    b
     *    --    --       zul 48  --
     *                              --                                    c,d,Cl
     *    --    -(45)    zul     zul
     *                              --                                    e
     *    --    -(45)    zul 58  zul 58
     *                   0          --     Stellbefehl mit freier         a-e
     *    --    -(44)    -(43)   -(44)
     *                              --     Definition                     Cl
     *    --    zul      -(43)   zul
     *            6+n*f  0          1 --   Stellbefehl mit freier         a,b
     *    --    --       -(46)   --
     *                              1 213  Definition                     c,d
     *    1     zul 50   -(46)   zul 50
     *                              1 106                                 e
     *    2     zul      -(46)   zul
     *                              1 --                                  Cl
     *    --    -(46)    -(46)   -(46)
     * 5A         5      1-254      --     Stellzustand mit Codenummer    a
     *    --    --       zul     --
     *                              --                                    b
     *    --    --       zul 48  --
     *                              --                                    c,d,Cl
     *    --    --       zul     zul
     *                              --                                    e
     *    --    --       zul 58  zul 58
     *                   0          --     Stellzustand mit freier        a-e
     *    --    --       --      --
     *                              --     Definition                     Cl
     *    --    --       zul     zul
     *            6+n*f  0          1 --   Stellzustand mit freier        a,b
     *    --    --       --      --
     *                              1 213  Definition                     c,d
     *    1     zul 50   --      zul 50
     *                              1 106                                 e
     *    2     zul      --      zul
     *                              1 --                                  Cl
     *    --    --       --      --
     * 21R        2      --         --     Abruf Stellzustand             alle
     *    --    zul      zul     zul
     * </pre>
     *
     * @param result Der vom Datenverteiler empgangene Datensatz, der in einen De-Block konvertiert wird.
     *
     * @return De-Block mit den konvertierten Daten.
     */
    public DeBlock Fg004Typ055Id005AtgTlsWzgStellZustandAspTlsVorgabe(ResultData result) {
        DeBlock deBlock      = null;
        DeBlock deBlockSuper = Fg004Typ033Id003AtgTlsWzgGrundEinstellungAspParameterSoll(result);

        // DeBlock erzeugen
        try {
            deBlock = new DeBlock(deBlockSuper.getDeData(), _de.getEak().getSm().getOsi7Adress(), _de.getFg(), 55, 5, _de.getDeChannel());
            debug.finest("TLS-DE-Block: " + HexDumper.toString(deBlock.getTlsDeDataBlock()));
        }
        catch (IllegalOsi7TelegramStructureException e) {
            debug.warning("Fehler beim Erzeugen eines De-Blocks zum Versand: " + e);
        }

        return deBlock;
    }

    /**
     * FG 4 : Typ: 55 Id: 5 Name: Stellzustand (Abruf / Antwort). TLS-Datenbeschreibung:<br>
     * <pre>
     * Position         Bezeichnung                                    Erluterung
     * Byte 1           Lnge DE-Block                                 [5 | 6+f*n]
     * Byte 2           Daten-Endgerte-Kanal (DE)                     [ 1..254 ]
     * Byte 3           Typ der DE-Daten                               [ 55 ]
     * Byte 4           Anzeigeprinzip                                 siehe unten
     * Byte 5           Stellcode
     * anzuzeigendes WVZ/Programm
     * Byte 6           Funktionsbyte                                  siehe unten
     * Byte 7           n : Anzahl Komponenten                         1 -[213 |
     * 106] siehe unten
     * f Bytes...       Stellzustand der 1.Komponente                  siehe unten
     * f Bytes          Stellzustand der n.Komponente
     * </pre>
     * Inhalt Byte : Anzeigeprinzip<br> Das Byte gibt das Anzeigeprinzip des adressierten/sendenden DEs an und ermglicht
     * damit die genaue Interpretation des folgenden Telegramminhalts. Es ist folgendermaen codiert:<br>
     * <pre>
     * Anzeigeprinzip        Wert
     *              a        0
     *              b        1
     *              c        2
     *              d        3
     *              e        4
     *        Cluster        8
     * </pre>
     * Erhlt ein DE in Abrufrichtung eine Nachricht mit einem Wert, der nicht seinem Anzeigeprinzip entspricht, so wird
     * die Nachricht mit Code 21 falsches Anzeigeprinzip negativ quittiert.<br> <br> Inhalt Byte: Stellcode<br> Der
     * Inhalt des Bytes ist abhngig davon, ob das adressierte DE ein Clusterkanal ist bzw. vom Anzeigeprinzip des
     * DEs:<br>
     * <pre>
     * Clusterkanle :   Nummer des Programms, das eingestellt werden soll. Ist der
     * Wert 0, so behalten alle DEs
     *                   den Zustand , den sie zuletzt hatten. In der Folge
     * akzeptieren diese DEs individuell an sie
     *                   gerichtete Stellbefehle und ndern ihren Zustand
     * entsprechend. Ist als Wert eine
     *                   Programmnummer zwischen 1 und 254 angegeben, so werden
     * alle DEs entsprechend der
     *                   Definition dieses Programms eingestellt. Ausnahmen hiervon
     * bilden lediglich DEs, fr
     *                   welche die entsprechende Programmdefinition der Wert 255
     * "dont care" enthlt. Diese
     *                   DEs behalten ihren Zustand und sind in der Folge durch
     * individuelle Stellbefehle steuerbar.
     *                   Der Wert 255 ist nicht zulssig und wird mit Code 43
     * negativ quittiert.
     * Anzeigeprinzip a und b :   WVZ-Code entsprechend Anhang 7, Kapitel 1. Gibt
     * den WVZ-Code an, den das
     *                   DE darstellen soll. Die Nachricht wird nur akzeptiert,
     * wenn der Stellcode des Clusterkanals
     *                   den Wert 0 enthlt oder die Definition des am Cluster
     * eingestellten Programms fr dieses
     *                   DE den Wert 255 "dont care" angibt. In allen anderen
     * Fllen wird der Stellbefehl mit Code
     *                   10 "Befehl bzw. Programm in dieser Betriebsart nicht
     * ausfhrbar" negativ quittiert. Der
     *                   Wert 0 ist fr DEs dieser Anzeigeprinzipien nicht zulssig
     * und wird ebenfalls negativ
     *                   quittiert.
     * Anzeigeprinzip c und d :   Stellcode des vordefinierten Textes, den das DE
     * darstellen soll. Die Nachricht
     *                   wird nur akzeptiert, wenn der eingestellte Stellcode des
     * Clusterkanals den Wert 0 enthlt
     *                   oder die Definition des am Cluster eingestellten Programms
     * fr dieses DE den Wert 255
     *                   "dont care" angibt. In allen anderen Fllen wird der
     * Stellbefehl mit Code 10 "Befehl bzw.
     *                   Programm in dieser Betriebsart nicht ausfhrbar" negativ
     * quittiert. Ist der Wert 0, so wird
     *                   das Telegramm in der Langversion bertragen und enthlt
     * den darzustellenden Text als
     *                   ASCII-String.
     * Anzeigeprinzip e :   Stellcode der vordefinierten Gruppendefinition, die das
     * DE darstellen soll. Die
     *                   Nachricht wird nur akzeptiert, wenn der eingestellte
     * Stellcode des Clusterkanals den Wert 0
     *                   enthlt oder die Definition des am Cluster eingestellten
     * Programms fr dieses DE den Wert
     *                   255 "dont care" angibt. In allen anderen Fllen wird der
     * Stellbefehl mit Code 10 "Befehl
     *                   bzw. Programm in dieser Betriebsart nicht ausfhrbar"
     * negativ quittiert. Ist der Wert 0, so
     *                   wird das Telegramm in der Langversion bertragen und
     * enthlt fr jede Komponente den
     *                   entsprechenden Stellzustand.
     * </pre>
     * Zulssige Werte:
     * <pre>
     *                       Cluster                      a,b
     * c,d,e
     * Steuerungsprinzip 1   0                            --
     * 0
     * Steuerungsprinzip 2   1-254                        Anhang 7, Kap. 1
     * 1-254
     * Steuerungsprinzip 3   0,1-254                      --
     * 0,1-254
     * </pre>
     * Inhalt Funktionsbyte:<br>
     * <pre>
     *    7        6        5        4        3        2        1        0
     *               Blinkzeit                      Status       Funktion
     * </pre>
     * Funktion
     * <pre>
     * Bit    Wert   Bedeutung                           Kommentar
     * 0..1   0      ausschalten                         nur fr DEs nach
     * Anzeigeprinzip b und d
     *        1      einschalten                         sowie Blinker nach
     * Anzeigeprinzip a
     *        2      blinken                             nur bei DEs nach
     * Anzeigeprinzip b und d, optional
     *        3      reserviert
     * DEs, die in Abrufrichtung ein Funktionsbyte mit einem Wert erhalten, der
     * gem ihrem Anzeigeprinzip
     * nicht zulssig ist, antworten mit einer negativen Quittung, Code 14
     * "Funktionsbyte unzulssig".
     * </pre>
     * Status (nur in Antwortrichtung):
     * <pre>
     * Bit    Wert   Bedeutung                           Kommentar
     * 2      0      kein Fehler aufgetreten
     *        1      Fehler aufgetreten
     * 3      2      Programm abgeschlossen              wird nur verwendet von
     * Clusterkanlen und DEs, die
     *        3      Programm noch nicht abgeschlossen   Programmschaltungen
     * ausfhren
     * Blinkzeit<br>
     * <pre>
     * Bit    Wert   Bedeutung                           Kommentar
     * 4..7   5-15   Blinkzeit in 200ms (1..3 s)         nur fr DEs nach
     * Anzeigeprinzip b und d
     *        0                                          wenn Funktion ungleich
     * "blinken"
     * Die Blinkzeit ist die gesamte Periodendauer, d.h. die Summe aus Ein- und
     * Auszeit. Das Verhltnis Ein-
     * /Auszeit wird nach Vorgabe des Anlagenbetreibers am WZG eingestellt.
     * Eine sinnvolle Abstimmung der Periodendauern fr verschiedene WVZ ist
     * Aufgabe der Zentrale.
     * </pre>
     * Inhalt Anzahl Komponenten :
     * Wird die Nachricht in der Langversion versendet (nur bei Stellcode gleich
     * 0), so gibt das Byte die Anzahl der
     * Komponenten an, fr die in der Folge Stellzustnde bermittelt werden.
     * Die Langversion ist nur fr WZG nach Anzeigeprinzip c)-e) zulssig, fr
     * Anzeigen nach Prinzip a) und b) ist
     * sie nicht anwendbar und fr Clusterkanle werden individuelle Stellbefehle
     * an die jeweiligen DEs direkt
     * adressiert.
     * Inhalt : Stellzustand einer Komponente:
     * Fr WZG vom Anzeigeprinzip a) und b) (feste Zeicheninhalte) ist das
     * Telegramm in der Langversion nicht
     * zulssig, da es bedeutungslos wre.
     * Fr WZG vom Anzeigeprinzip c) und d) (Wechseltext) besteht der Stellzustand
     * einer Komponente aus jeweils
     * nur einem Byte und ist wie folgt aufgebaut :
     * <pre>
     * Byte 1            Textzeichen in ASCII-Code            siehe Anhang 7,
     * Kapitel 3
     * </pre>
     * Fr WZG vom Anzeigeprinzip e) (Gruppen) besteht der Stellzustand einer
     * Komponente aus jeweils 2 Bytes
     * und ist wie folgt aufgebaut:<br>
     * <pre>
     * Byte 1            WVZ-Code                             siehe Anhang 7,
     * Kapitel 1
     * Byte 2            Funktionsbyte                        siehe oben
     * </pre>
     * Das Funktionsbyte ist definiert wie oben. Komponenten nach Anzeigeprinzip a)
     * melden in Antwortrichtung
     * den Sachverhalt Strung erkannt - Prisma nicht bewegbar / Blinker defekt
     * mit gesetztem Bit 2 im
     * Funktionsbyte. Kann der eingestellte WVZ-Code vom Gert noch ermittelt
     * werden, so wird er in Byte 1
     * bertragen. Ansonsten wird der WVZ-Code 245 undefinierte Stellung
     * gemeldet.
     * Fr Clusterkanle ist die Nachricht in der Langversion nicht zulssig, da
     * die Stellbefehle an die einzelnen DEs
     * direkt adressiert werden knnen.<br>
     * <br>
     * Die folgende Tabelle stellt bersichtlich alle Varianten des Telegramms
     * zusammen, erlutert deren Funktion und gibt
     * an, fr welche Kombination aus Anzeige und Steuerungsprinzip die jeweilige
     * Verwendung zulssig ist. Dabei sind
     * Felder -- gekennzeichnet, wenn sie im betreffenden Kontext nicht relevant
     * sind. Mit - gekennzeichnete Felder in der
     * Spalte "Steuerungsprinzip" bedeuten, dass die jeweiligen Stellbefehle
     * negativ quittiert werden, die Nummer in
     * Klammern gibt den zu verwendenden Fehlercode an. Felder mit zul kennzeichnen
     * die Zulssigkeit des
     * entsprechenden Telegramms fr das jeweilige DE. Eine hinter zul angefgte
     * Nummer  verweist auf
     * einen Telegrammtyp aus frheren Versionen der TLS, die bei Altlagen im
     * jeweiligen Kontext noch zulssig sind.<br>
     * <pre>
     * ID   Typ   Lnge  Stellcode  n      Funktion                       Anzeige
     *    f     Steuerungsprinzip
     *                                                                    prinzip
     * <p/>
     *          1        2       3
     * 5R   55    5      1-254      --     Stellbefehl mit Codenummer     a
     *    --    --       zul     --
     *                              --                                    b
     *    --    --       zul 48  --
     *                              --                                    c,d,Cl
     *    --    -(45)    zul     zul
     *                              --                                    e
     *    --    -(45)    zul 58  zul 58
     *                   0          --     Stellbefehl mit freier         a-e
     *    --    -(44)    -(43)   -(44)
     *                              --     Definition                     Cl
     *    --    zul      -(43)   zul
     *            6+n*f  0          1 --   Stellbefehl mit freier         a,b
     *    --    --       -(46)   --
     *                              1 213  Definition                     c,d
     *    1     zul 50   -(46)   zul 50
     *                              1 106                                 e
     *    2     zul      -(46)   zul
     *                              1 --                                  Cl
     *    --    -(46)    -(46)   -(46)
     * 5A         5      1-254      --     Stellzustand mit Codenummer    a
     *    --    --       zul     --
     *                              --                                    b
     *    --    --       zul 48  --
     *                              --                                    c,d,Cl
     *    --    --       zul     zul
     *                              --                                    e
     *    --    --       zul 58  zul 58
     *                   0          --     Stellzustand mit freier        a-e
     *    --    --       --      --
     *                              --     Definition                     Cl
     *    --    --       zul     zul
     *            6+n*f  0          1 --   Stellzustand mit freier        a,b
     *    --    --       --      --
     *                              1 213  Definition                     c,d
     *    1     zul 50   --      zul 50
     *                              1 106                                 e
     *    2     zul      --      zul
     *                              1 --                                  Cl
     *    --    --       --      --
     * 21R        2      --         --     Abruf Stellzustand             alle
     *    --    zul      zul     zul
     * </pre>
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz fr die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung fr die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg004Typ055Id005AtgTlsWzgStellZustandAspTlsZustand(DeBlock deBlock, Data data, DataDescription dataDescription) {

        // Analog zur Grundeinstellung
        ResultData resultData = Fg004Typ033Id003AtgTlsWzgGrundEinstellungAspParameterIst(deBlock, data, dataDescription);

        // Ergebnisdatensatz erzeugen...
        return resultData;
    }

    /**
     * FG 4 : Typ: 55 Id: 37 Name: Stellzustand (Abruf / Antwort). TLS-Datenbeschreibung:<br> Standardverarbeitung wie bei
     * Id 5, aber als "nachgelieferte" Daten an den DaV bergeben.
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz fr die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung fr die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg004Typ055Id037AtgTlsWzgStellZustandAspTlsZustand(DeBlock deBlock, Data data, DataDescription dataDescription) {

        // Standardverarbeitung wie bei Id 5, aber als "nachgelieferte" Daten an den DaV bergeben.
        ResultData resultData = Fg004Typ055Id005AtgTlsWzgStellZustandAspTlsZustand(deBlock, data, dataDescription);

        if (resultData != null) {
            return new ResultData(_de.getDe(), dataDescription, resultData.getDataTime(), resultData.getData(), true);
        } else {
            return resultData;
        }
    }

    /**
     * FG 4 : Typ: 58 Id: 5 und Id 37 Name: Stellprogramm (Abruf / Antwort). TLS-Datenbeschreibung:<br>
     * <pre>
     * Position         Bezeichnung                             Erluterung
     * Byte 1           Lnge DE-Block                          [ 4 ]
     * Byte 2           Daten-Endgerte-Kanal (DE)              [ 193..222, 223,
     * (255), (evt. 1..254, 255)1 ]
     * Byte 3           Typ der DE-Daten                        [ 58 ]
     * Byte 4           Programm-Nummer                         [ 1 - 254, 255 ]
     * Byte 5           Programmstatus                          [ siehe unten ]
     * 1) es wird der Clusterkanal gem der Definition Seite A6.1.1  3, Kapitel
     * 1.4 angewendet, lediglich bei , die nach
     * dem Prinzip der alten FG5 (TLS Ausgabe 1993) arbeiten, knnen die E/A-Kanle
     * direkt adressiert werden
     * </pre>
     * Inhalt Byte Programm-Nummer:<br> Programm-Nummer des projektierten Programms in das umgeschaltet werden soll. Die
     * Verwendung der Programmnummer 255 wird in Anhang 6, Teil 1 Kapitel 3.5.5 und 3.5.6 erlutert<br> Inhalt Byte
     * Programmstatus:<br>
     * <pre>
     *    7        6        5        4        3        2        1        0
     *                       reserviert                           Status
     * Status
     * Bit 0:        0 = Programm abgeschlossen
     *               1 = Programm noch nicht abgeschlossen
     * Bit 1:        0 = kein Fehler aufgetreten
     *               1 = Fehler aufgetreten
     * In Stellbefehlen (Abrufrichtung) ist das Byte funktionslos und wird mit 0
     * belegt.
     * </pre>
     *
     * @param result Der vom Datenverteiler empgangene Datensatz, der in einen De-Block konvertiert wird.
     *
     * @return De-Block mit den konvertierten Daten.
     */
    public DeBlock Fg004Typ058Id005AtgTlsWzgStellProgrammAspTlsVorgabe(ResultData result) {
        DeBlock deBlock = null;

        // Datenblock beseht aus folgenden Inhalten:
        int _ProgrammNummer_;
        int _Status_ = 0;

        // Umsetzung
        Data data = result.getData();  // Datensatz holen

        _ProgrammNummer_ = data.getItem("WVZProgramm").getUnscaledValue("Nummer").intValue();

        int _ProgrammStatus_ = data.getItem("WVZProgramm").getItem("Status").getUnscaledValue("Programm").intValue();
        int _FehlerStatus_   = data.getItem("WVZProgramm").getItem("Status").getUnscaledValue("Fehler").intValue();

        if (_ProgrammStatus_ == 0) {

            // nichts machen (Bit 1 = 0)
        } else {
            _Status_ |= 0x01;
        }

        if (_FehlerStatus_ == 0) {

            // nichts machen (Bit 2 = 0)
        } else {
            _Status_ |= 0x02;
        }

        // DeBlock als TlsData aufbauen...
        TlsData deApplicationData = new TlsData(2);

        deApplicationData.writeIntAsByte(_ProgrammNummer_);
        deApplicationData.writeIntAsByte(_Status_);

        // DeBlock erzeugen
        try {
            deBlock = new DeBlock(deApplicationData, _de.getEak().getSm().getOsi7Adress(), _de.getFg(), 58, 5, _de.getDeChannel());
            debug.finest("TLS-DE-Block: " + HexDumper.toString(deBlock.getTlsDeDataBlock()));
        }
        catch (IllegalOsi7TelegramStructureException e) {
            debug.warning("Fehler beim Erzeugen eines De-Blocks zum Versand: " + e);
        }

        return deBlock;
    }

    /**
     * FG 4 : Typ: 58 Id: 5 und Id 37 Name: Stellprogramm (Abruf / Antwort). TLS-Datenbeschreibung:<br>
     * <pre>
     * Position         Bezeichnung                             Erluterung
     * Byte 1           Lnge DE-Block                          [ 4 ]
     * Byte 2           Daten-Endgerte-Kanal (DE)              [ 193..222, 223,
     * (255), (evt. 1..254, 255)1 ]
     * Byte 3           Typ der DE-Daten                        [ 58 ]
     * Byte 4           Programm-Nummer                         [ 1 - 254, 255 ]
     * Byte 5           Programmstatus                          [ siehe unten ]
     * 1) es wird der Clusterkanal gem der Definition Seite A6.1.1  3, Kapitel
     * 1.4 angewendet, lediglich bei , die nach
     * dem Prinzip der alten FG5 (TLS Ausgabe 1993) arbeiten, knnen die E/A-Kanle
     * direkt adressiert werden
     * </pre>
     * Inhalt Byte Programm-Nummer:<br> Programm-Nummer des projektierten Programms in das umgeschaltet werden soll. Die
     * Verwendung der Programmnummer 255 wird in Anhang 6, Teil 1 Kapitel 3.5.5 und 3.5.6 erlutert<br> Inhalt Byte
     * Programmstatus:<br>
     * <pre>
     *    7        6        5        4        3        2        1        0
     *                       reserviert                           Status
     * Status
     * Bit 0:        0 = Programm abgeschlossen
     *               1 = Programm noch nicht abgeschlossen
     * Bit 1:        0 = kein Fehler aufgetreten
     *               1 = Fehler aufgetreten
     * In Stellbefehlen (Abrufrichtung) ist das Byte funktionslos und wird mit 0
     * belegt.
     * </pre>
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz fr die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung fr die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg004Typ058Id005AtgTlsWzgStellProgrammAspTlsZustand(DeBlock deBlock, Data data, DataDescription dataDescription) {
        ResultData resultData = null;

        // Programm-Nr aus Telegramm -----------------------------------------------------------------------------------
        int _ProgrammNummer_ = deBlock.getDeData().readUnsignedByteAsInt();

        data.getItem("WVZProgramm").getUnscaledValue("Nummer").set(_ProgrammNummer_);

        // ProgrammStatus aus Telegramm --------------------------------------------------------------------------------
        int _ProgrammStatus = deBlock.getDeData().readUnsignedByteAsInt();

        if ((_ProgrammStatus & 0x01) == 0) {
            data.getItem("WVZProgramm").getItem("Status").getUnscaledValue("Programm").setText("abgeschlossen");
        } else {
            data.getItem("WVZProgramm").getItem("Status").getUnscaledValue("Programm").setText("in Arbeit");
        }

        if ((_ProgrammStatus & 0x02) == 0) {
            data.getItem("WVZProgramm").getItem("Status").getUnscaledValue("Fehler").setText("ok");
        } else {
            data.getItem("WVZProgramm").getItem("Status").getUnscaledValue("Fehler").setText("Fehler");
        }

        // Ergebnisdatensatz erzeugen...
        resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);

        return resultData;
    }

    /**
     * FG 4 : Typ: 58 Id: 5 und Id 37 Name: Stellprogramm (Abruf / Antwort). TLS-Datenbeschreibung:<br>
     * Standardverarbeitung wie bei Id 5, aber als "nachgelieferte" Daten an den DaV bergeben.
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz fr die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung fr die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg004Typ058Id037AtgTlsWzgStellProgrammAspTlsZustand(DeBlock deBlock, Data data, DataDescription dataDescription) {

        // Standardverarbeitung wie bei Id 5, aber als "nachgelieferte" Daten an den DaV bergeben.
        ResultData resultData = Fg004Typ058Id005AtgTlsWzgStellProgrammAspTlsZustand(deBlock, data, dataDescription);

        if (resultData != null) {
            return new ResultData(_de.getDe(), dataDescription, resultData.getDataTime(), resultData.getData(), true);
        } else {
            return resultData;
        }
    }

    /**
     * Entspricht {@link Fg004Default#Fg255Typ029Id002AtgTlsGloKanalSteuerungAspParameterIst(de.bsvrz.kex.tls.osi7.DeBlock,
     * de.bsvrz.dav.daf.main.Data, de.bsvrz.dav.daf.main.DataDescription)}, jedoch werden die Daten als nachgeliefert gekennzeichnet.
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz fr die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung fr die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg255Typ029Id034AtgTlsGloKanalSteuerungAspParameterIst(DeBlock deBlock, Data data, DataDescription dataDescription) {

        // Generierte Methode: Standardverarbeitung wie bei Id 2, aber als "nachgelieferte" Daten an den DaV bergeben.
        ResultData resultData = Fg255Typ029Id002AtgTlsGloKanalSteuerungAspParameterIst(deBlock, data, dataDescription);

        return new ResultData(_de.getDe(), dataDescription, resultData.getDataTime(), resultData.getData(), true);
    }

    /**
     * Entspricht {@link Fg004Default#Fg255Typ036Id003AtgTlsGloGeographischeKenndatenAspTlsZustand(de.bsvrz.kex.tls.osi7.DeBlock,
     * de.bsvrz.dav.daf.main.Data, de.bsvrz.dav.daf.main.DataDescription)}, jedoch werden die Daten als nachgeliefert gekennzeichnet.
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz fr die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung fr die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg255Typ036Id035AtgTlsGloGeographischeKenndatenAspTlsZustand(DeBlock deBlock, Data data, DataDescription dataDescription) {

        // Generierte Methode: Standardverarbeitung wie bei Id 3, aber als "nachgelieferte" Daten an den DaV bergeben.
        ResultData resultData = Fg255Typ036Id003AtgTlsGloGeographischeKenndatenAspTlsZustand(deBlock, data, dataDescription);

        return new ResultData(_de.getDe(), dataDescription, resultData.getDataTime(), resultData.getData(), true);
    }

    //~ GET METHODEN ==========================================================

    // ------------------------------------------------------------------------
    // GETTER / SETTER METHODEN
    // ------------------------------------------------------------------------
    public int getValidWvzCode() {
        return _validWvzCode;
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 14.02.12
